VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsFluid"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private pvtPhaseOil As Boolean           'Phases present: Oil/Gas
Private pvtPhaseGas As Boolean           'Phases present: Gas Reservoir
Private pvtPhaseWater As Boolean         'Phases present: Water

Private pvtStdP As Single                'Standard Pressure (psia)

Private pvtStdTU As String               'Standard Temperature Units (F or C)
Private pvtStdTf As Single               'Standard Temperature (F)
Private pvtStdTr As Single               'Standard Temperature (Rankine)
Private pvtStdTc As Single               'Standard Temperature (C)

Private pvtResTU As String               'Reservoir Temperature Units (F or C)
Private pvtResTf As Single               'Reservoir Temperature (F)
Private pvtResTr As Single               'Reservoir Temperature (Rankine)
Private pvtResTc As Single               'Reservoir Temperature (C)
'
'---Class Initialize
'
Private Sub Class_Initialize()
  PhaseOil = True
  PhaseGas = False
  PhaseWater = False
  
  StdTU = "F"
  StdTf = 60
  StdP = 14.7
  ResTU = "F"
  ResTf = 150
End Sub
'
'============================== Properties ====================================
'
'
'---Phases Present
'
Public Property Get PhaseOil() As Boolean
  PhaseOil = pvtPhaseOil
End Property

Public Property Let PhaseOil(ByVal vNewValue As Boolean)
  pvtPhaseOil = vNewValue
End Property

Public Property Get PhaseGas() As Boolean
  PhaseGas = pvtPhaseGas
End Property

Public Property Let PhaseGas(ByVal vNewValue As Boolean)
  pvtPhaseGas = vNewValue
End Property

Public Property Get PhaseWater() As Boolean
  PhaseWater = pvtPhaseWater
End Property

Public Property Let PhaseWater(ByVal vNewValue As Boolean)
  pvtPhaseWater = vNewValue
End Property
'
'---Standard Pressure
'
Public Property Get StdP() As Single
  StdP = pvtStdP
End Property

Private Property Let StdP(ByVal vNewValue As Single)
  pvtStdP = vNewValue
End Property

'
'---Standard Temperature Units
'
Public Property Get StdTU() As String
  StdTU = pvtStdTU
End Property

Public Property Let StdTU(ByVal vNewValue As String)
  If vNewValue <> pvtStdTU Then
    Select Case vNewValue
      Case "F", "f": StdTf = StdTc
      Case "C", "c": StdTc = StdTf
    End Select
    
    pvtStdTU = UCase(vNewValue)
  End If
End Property
'
'---Standard Temperature (F)
'
Public Property Get StdTf() As Single
  StdTf = pvtStdTf
End Property

Private Property Let StdTf(ByVal vNewValue As Single)
  pvtStdTf = vNewValue
  pvtStdTr = pvtStdTf + 460#
  pvtStdTc = (pvtStdTf - 32#) / 1.8
End Property
'
'---Standard Temperature (C)
'
Public Property Get StdTc() As Single
  StdTc = pvtStdTc
End Property

Private Property Let StdTc(ByVal vNewValue As Single)
  pvtStdTc = vNewValue
  pvtStdTf = (pvtStdTc * 1.8) + 32#
  pvtStdTr = pvtStdTf + 460#
End Property
'
'---Standard Temperature (Rankine)
'
Public Property Get StdTr() As Single
  StdTr = pvtStdTr
End Property
'
'---Reservoir Temperature Units
'
Public Property Get ResTU() As String
  ResTU = pvtResTU
End Property

Public Property Let ResTU(ByVal vNewValue As String)
  If vNewValue <> pvtResTU Then
    Select Case vNewValue
      Case "F", "f": ResTf = ResTc
      Case "C", "c": ResTc = ResTf
    End Select
    
    pvtResTU = UCase(vNewValue)
  End If
End Property
'
'---Reservoir Temperature (F)
'
Public Property Get ResTf() As Single
  ResTf = pvtResTf
End Property

Private Property Let ResTf(ByVal vNewValue As Single)
  pvtResTf = vNewValue
  pvtResTr = pvtResTf + 460#
  pvtResTc = (pvtResTf - 32#) / 1.8
End Property
'
'---Reservoir Temperature (C)
'
Public Property Get ResTc() As Single
  ResTc = pvtResTc
End Property

Private Property Let ResTc(ByVal vNewValue As Single)
  pvtResTc = vNewValue
  pvtResTf = (pvtResTc * 1.8) + 32#
  pvtResTr = pvtResTf + 460#
End Property
'
'---Reservoir Temperature (Rankine)
'
Public Property Get ResTr() As Single
  ResTr = pvtResTr
End Property
'
'============================== Methods ====================================
'
'
'---Set Standard pressure method
'
Public Sub SetStdP(pValue As String)
  StdP = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set Standard temperature method
'
Public Sub SetStdT(tValue As String)
  Select Case StdTU
    Case "F", "f": StdTf = objUtil.Str2Nbr(tValue)
    Case "C", "c": StdTc = objUtil.Str2Nbr(tValue)
  End Select
End Sub
'
'---Set Reservoir temperature method
'
Public Sub SetResT(tValue As String)
  Select Case ResTU
    Case "F", "f": ResTf = objUtil.Str2Nbr(tValue)
    Case "C", "c": ResTc = objUtil.Str2Nbr(tValue)
  End Select
End Sub

