VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsGas"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Option Base 1

Private pvtGasEntry As String          'Gas Entry Option - G=Gravity, C=Composition
Private pvtPlusEntry As Long           'PLUS entry option - 0=No plus value, 1-12
                                       '                    index of HC plus value
Private pvtGasGrav As Single           'Gas Gravity
Private pvtPLUSmw As Single            'PLUS entry molecular weight
Private pvtPLUSgg As Single            'PLUS entry gas gravity

Private pvtmfN As Single               'Mole Fraction - Nitrogen
Private pvtmfCO2 As Single             'Mole Fraction - Carbon dioxide
Private pvtmfH2S As Single             'Mole Fraction - Hydrogn sulfide

Private pvtGC(1 To 15) As Single       'Gas composition array
Private pvtSumGC As Single             'Sum of composition array

Private pvtPPR(1 To 30) As Single      'Pseudo-Reduced pressure
Private pvtZ(1 To 30) As Single        'Gas deviation factor
Private pvtPZ(1 To 30) As Single       'Pressure/Z
Private pvtBG(1 To 30) As Single       'Volume factor
Private pvtMUG(1 To 30) As Single      'Viscosity
Private pvtCG(1 To 30) As Single       'Compressibility
Private pvtRHOG(1 To 30) As Single     'Density
Private pvtRMP(1 To 30) As Single      'Real potential

Private pvtPTR As Single               'Pseudo-Reduced temperature
Private pvtSTDenGas As Single          'Stock tank gas density
Private pvtpC As Single                'Critical pressure
Private pvttC As Single                'Critical temperature
Private pvtGO As Boolean               'Gas properties available (GO=TRUE)
'
'---Class Initialize
'
Private Sub Class_Initialize()
  GasEntry = "G"
  PlusEntry = 0
  PLUSmw = 105.8
  PLUSgg = 0.7
  GasGrav = 0.7
  STDenGas = 0.05
  mfN = 0
  mfCO2 = 0
  mfH2S = 0
  GO = False
End Sub
'
'============================== Properties ====================================
'
'
'---Gas entry option
'

Public Property Get GasEntry() As String
  GasEntry = pvtGasEntry
End Property

Public Property Let GasEntry(ByVal vNewValue As String)
  pvtGasEntry = UCase(vNewValue)
End Property
'
'---PLUS entry option index
'
Public Property Get PlusEntry() As Long
  PlusEntry = pvtPlusEntry
End Property

Public Property Let PlusEntry(ByVal vNewValue As Long)
  pvtPlusEntry = vNewValue
End Property
'
'---PLUS entry molecular weight
'
Public Property Get PLUSmw() As Single
  PLUSmw = pvtPLUSmw
End Property

Private Property Let PLUSmw(ByVal vNewValue As Single)
  pvtPLUSmw = vNewValue
End Property
'
'---PLUS entry gas gravity
'
Public Property Get PLUSgg() As Single
  PLUSgg = pvtPLUSgg
End Property

Public Property Let PLUSgg(ByVal vNewValue As Single)
  pvtPLUSgg = vNewValue
End Property
'
'---Gas Gravity
'
Public Property Get GasGrav() As Single
  GasGrav = pvtGasGrav
End Property

Private Property Let GasGrav(ByVal vNewValue As Single)
  pvtGasGrav = vNewValue
End Property
'
'---Mole fraction - Nitrogen
'
Public Property Get mfN() As Single
  mfN = pvtmfN
End Property

Private Property Let mfN(ByVal vNewValue As Single)
  pvtmfN = vNewValue
End Property
'
'---Mole fraction - CO2
'
Public Property Get mfCO2() As Single
  mfCO2 = pvtmfCO2
End Property

Private Property Let mfCO2(ByVal vNewValue As Single)
  pvtmfCO2 = vNewValue
End Property
'
'---Sum of gas composition array
'
Public Property Get SumGC() As Single
  SumGC = pvtSumGC
End Property

Private Property Let SumGC(ByVal vNewValue As Single)
  pvtSumGC = vNewValue
End Property
'
'---Mole fraction - H2S
'
Public Property Get mfH2S() As Single
  mfH2S = pvtmfH2S
End Property

Private Property Let mfH2S(ByVal vNewValue As Single)
  pvtmfH2S = vNewValue
End Property
'
'---Stock tank gas density
'
Public Property Get STDenGas() As Single
  STDenGas = pvtSTDenGas
End Property

Private Property Let STDenGas(ByVal vNewValue As Single)
  pvtSTDenGas = vNewValue
End Property
'
'---Critical pressure
'
Public Property Get pC() As Single
  pC = pvtpC
End Property

Private Property Let pC(ByVal vNewValue As Single)
  pvtpC = vNewValue
End Property
'
'---Critical temperature
'
Public Property Get tC() As Single
  tC = pvttC
End Property

Private Property Let tC(ByVal vNewValue As Single)
  pvttC = vNewValue
End Property

Public Property Get PTR() As Single
  PTR = pvtPTR
End Property
'
'---Gas propertes calc'ed
'
Public Property Get GO() As Boolean
  GO = pvtGO
End Property

Private Property Let GO(ByVal vNewValue As Boolean)
  pvtGO = vNewValue
End Property
'
'============================== Methods ====================================
'
'
'---Set gas gravity method
'
Public Sub SetGasGrav(strValue As String)
  GasGrav = objUtil.Str2Nbr(strValue)
End Sub
'
'---Set PLUS entry molecular wt method
'
Public Sub SetPLUSmw(strValue As String)
  PLUSmw = objUtil.Str2Nbr(strValue)
End Sub
'
'---Set PLUS entry gas gravity method
'
Public Sub SetPLUSgg(strValue As String)
  PLUSgg = objUtil.Str2Nbr(strValue)
End Sub
'
'---Set mole fraction - N
'
Public Sub SetmfN(strValue As String)
  mfN = objUtil.Str2Nbr(strValue)
End Sub
'
'---Set mole fraction - CO2
'
Public Sub SetmfCO2(strValue As String)
  mfCO2 = objUtil.Str2Nbr(strValue)
End Sub
'
'---Set mole fraction - H2S
'
Public Sub SetmfH2S(strValue As String)
  mfH2S = objUtil.Str2Nbr(strValue)
End Sub
'
'---Add/Update Gas composition array element
'
Public Function SetGC(strValue As String, N As Integer)
  Dim ii As Integer
  
  pvtGC(N) = objUtil.Str2Nbr(strValue)
  SumGC = 0
  For ii = 1 To 15
    SumGC = SumGC + pvtGC(ii)
  Next ii
End Function

Public Function GetGC(N As Integer) As Single
  GetGC = pvtGC(N)
End Function
     
Private Sub ZFACT(TR As Single, PR As Single, Z As Single, Err As Integer)

' ZFACT converted from FORTRAN version provided by MHA (Greg Hueni)
'--------------------------------------------------------------------
'  This subroutine estimates the Z factor by using a technique
'  developed by Dranchuk, Purvis, and Robison.  It had the lowest
'  average error of eight methods commonly used to estimate the
'  Z factor.
'  RE:  1.  Dranchuk, P.M., Purvis, R. A., and Robinson, D. B.,
'     "Computer Calculations of Natural Gas Compressibility Factors"
'  Using the Standing and Katz Correlation,", Institute of Petroleum
'  Technical Series, No. IP 74-008, 1974.
'       2.  Takacs, G., "Comparisons Made for Computer Z-Factor
' Calculations, " Oil and Gas Journal, Dec 20, 1976, pp. 64-66."
'--------------------------------------------------------------------
'
' INPUTS:
'    TR  - <REAL> The Pseudo reduced temperature of the gas mixture in
'          degrees Rankine.  Range:  1.05<=TR<3.0
'    PR  - <REAL> The Pseudo redured pressure of the gas mixture in
'          psia.  Range:  0<PR<30
' OUTPUTS:
'    Z   - <REAL> The gas compressibility factor of the gas mixture.
'    ERR - <INTEGER> The error code.
'          ERR = 0  No error.
'          ERR = 1  TR out of range.  An attempt is made to estimate
'                   a value for the Z factor regardless.
'          ERR = 2  PR out of range.  An attempt is made to estimate
'                   a value for the Z factor regardless.
'          ERR = 3  Iteration procedure could not converge.
'
'  CALLS:  None.
'
'-------------------------------------------------------------------

  Dim A As Single, B As Single, C As Single, D As Single
  Dim E As Single, F As Single, G As Single, DR As Single, FUNCT As Single
  Dim DFUNCT As Single, TOLER As Single, DR3 As Single
  Dim ITER As Integer

  Err = 0
  If TR < 1.05 Or TR >= 3# Then Err = 1
  
  If PR <= 0# Or PR >= 30# Then Err = 2

  A = 0.06423
  B = 0.5353 * TR - 0.6123
  C = 0.3151 * TR - 1.0467 - 0.5783 / (TR * TR)
  D = TR
  E = 0.6816 / (TR * TR)
  G = 0.27 * PR

  DR = 0.27 * PR / TR
  TOLER = 0.000001

  For ITER = 1 To 50
    F = 0.6845 * DR * DR
    DR3 = DR * DR * DR

    FUNCT = (((A * DR3 + B + E * (1 + F) * Exp(-F)) * DR + C) * DR + D) * DR - G

    If Abs(FUNCT) < TOLER Then
      Z = 0.27 * PR / (DR * TR)
      Exit Sub
    End If
         
    DFUNCT = ((6# * A * DR3 + 3# * B + E * (3# + F * (3# - 2# * F)) * _
             Exp(-F)) * DR + 2# * C) * DR + D
    DR = DR - FUNCT / DFUNCT
  Next ITER
  Err = 3

  Z = 0.27 * PR / (DR * TR)

End Sub

Private Sub GASTCPC(ghC As Single, gasMW As Single)

'Converted from FORTRAN version provided by MHA (Greg Hueni)
'
'   SUBROUTINE TO CALCULATE CRIFICAL TEMPERATURE AND PRESSURE
'     BASED ON STANDING AND KATZ APPROACH WITH WEICKERT-AZIZ
'     COORECTIONS FOR PRESENCE OF NON HYDROCARBONS. SEE HP41
'     FLUID PACK CORRELATIONS.  ALSO CALCULATES GAS MOL WT.
'
'   INPUTS: IRESTYP    (0=GAS RESERVOIR; 1=OIL RESERVOIR)
'           G          FULL WELLSTREAM GAS GRAVITY
'           GHC        HYDROCARBON GAS GRAVITY
'           YN2        MOLE FRACTION NITROGEN
'           YCO2       MOLE FRACTION CARBON DIOXIDE
'           YH2S       MOLE FRACTION HYDROGEN SULFIDE
'
'   NOTE: WILL USE GHC IF NON-ZERO OR WILL CALCULATE FROM G
'         IF GHC = 0.
'
'   OUTPUTS: TC        CRITICAL TEMPERATURE
'            PC        CRITICAL PRESSURE
'            GASMW     MOLECULAR WEIGHT OF FULL WELLSTREAM GAS
'
'   MOD 0: AUG 6, 1994     GBH
'
'==============================================================
'
  Dim TCHC As Single, PCHC As Single, CWA As Single
  
  If GasGrav <> ghC Then
    If GasGrav <> 0# And ghC = 0# Then
      ghC = (GasGrav - 0.9672 * mfN - 1.5195 * mfCO2 - 1.1765 * mfH2S) / _
            (1# - mfN - mfCO2 - mfH2S)
      gasMW = 28.96 * GasGrav
  
    ElseIf ((GasGrav = 0#) And (ghC <> 0#)) Then
      GasGrav = ghC * (1# - mfN - mfCO2 - mfH2S) + (0.9672 * mfN + 1.5195 * mfCO2 + _
          1.1765 * mfH2S)
      gasMW = 28.96 * GasGrav
    End If
  End If

  If objFluid.PhaseGas Then
    TCHC = 187# + 330# * ghC - 71.5 * ghC ^ 2#
    PCHC = 706# - 51.7 * ghC - 11.1 * ghC ^ 2#
  End If

  If objFluid.PhaseOil Then
    TCHC = 168# + 325# * ghC - 12.5 * ghC ^ 2#
    PCHC = 677# - 15# * ghC - 37.5 * ghC ^ 2#
  End If

  tC = (1# - mfN - mfCO2 - mfH2S) * TCHC + 227.3 * mfN + 547.6 * mfCO2 + 672.4 * mfH2S
  pC = (1# - mfN - mfCO2 - mfH2S) * PCHC + 493# * mfN + 1071# * mfCO2 + 1306# * mfH2S

  If mfH2S <> 0# Then
    CWA = 120 * ((mfCO2 + mfH2S) ^ 0.9 - (mfCO2 + mfH2S) ^ 1.6) + 15# * _
          (mfCO2 ^ 0.5 - mfH2S ^ 4#)
    tC = tC - CWA
    pC = (pC * tC) / ((tC + CWA) + mfH2S * (1# - mfH2S) * CWA)
  End If
End Sub

Private Sub GASCOM(PRES As Single, RESTEMP As Single, CMPG As Single)
'
'Converted from FORTRAN version provided by MHA (Greg Hueni)
'
'  This subroutine calculates Gas Compressibility for Real Natural Gases
'
'  PRES  = Gas pressure (PSIA)
'  RESTEMP  = Gas temperature (degrees Rankine)
'  SPG  = Gas specific gravity (air = 1)
'  CMPG = Gas compressibility (1 / PSI)
'  ZED  = Gas deviation factor (compressibility factor)
'
'      1   1 dZ
'  C = - - - --
'      P   Z dP
'
'  In this subroutine, use dP = 50 and calculate dZ = Z(P+25) - Z(P-25)
'
' CALLS - ZFACT
'

  Dim PPR As Single, PTR As Single, ZED As Single, DP As Single
  Dim PRS1 As Single, PPR1 As Single, ZED1 As Single
  Dim PRS2 As Single, PPR2 As Single, ZED2 As Single
  Dim Err As Integer

  PPR = PRES / pC
  PTR = RESTEMP / tC
  Call ZFACT(PTR, PPR, ZED, Err)

  If PRES < 50# Then
    DP = PRES + PRES
    PRS1 = 0#
    PRS2 = DP
    PPR1 = 0#
    ZED1 = 1#
    PPR2 = DP / pC
  Else
    DP = 50#
    PRS1 = PRES + 25#
    PRS2 = PRES - 25#
    PPR1 = PRS1 / pC
    PPR2 = PRS2 / pC
    Call ZFACT(PTR, PPR1, ZED1, Err)
  End If
      
  Call ZFACT(PTR, PPR2, ZED2, Err)

  CMPG = 1# / PRES - (ZED1 - ZED2) / (DP * ZED)

End Sub

Private Function GASVIS(TEM As Single, PRS As Single, ZED As Single, SPG As Single, _
                   VISG As Single) As Integer
'
'Converted from FORTRAN version provided by MHA (Greg Hueni)
'
'  This subroutine calculates gas viscosity
'  Reference: Lee, et al., JPT Aug 1966 PG 997-1000
'
'  TEM = Temperature (Deg R)
'  PRS = Pressure(PSIA)
'  ZED = real gas correction (Z-factor)
'  SPG = Gas specific gravity (air = 1)
'  RHO = Gas Density (GM/CC)
'  VISG= Gas viscosity (CP)
'
'  GASVIS returns error codes as follows:
'
'       0 - OK
'       1 - Input temp out of range
'       2 - Input pressure out of range
'       3 - Input Z-factor out of range
'       4 - Input gas gravity out of range
'       5 - Error unknown
'
  Dim RHO As Single
  Dim K As Single, W As Single, X As Single, Y As Single
  Dim Err As Integer
'
'---Validate input
'
  Err = 0
  If TEM < 460# Or TEM > 1500# Then Err = 1
  If PRS < 0# Or PRS > 20000# Then Err = 2
  If ZED < 0.25 Or ZED > 3# Then Err = 3
  If SPG < 0.45 Or SPG > 2# Then Err = 4
  GASVIS = Err
  
  If Err = 0 Then
    On Error GoTo ErrExit
  
    VISG = 0#
    W = SPG * 28.96
  
    If PRS > 12# Then RHO = W * PRS / (ZED * 670# * TEM)
    If PRS <= 12# Then RHO = W * 12# / (ZED * 670# * TEM)
      
    K = ((7.77 + 0.0063 * W) * TEM ^ 1.5) / (122.4 + 12.9 * W + TEM)
    X = 2.57 + 1914.5 / TEM + 0.0095 * W
    Y = 1.11 + 0.04 * X
    VISG = 0.0001 * K * Exp(X * RHO ^ Y)
  End If
  Exit Function

ErrExit:
  GASVIS = 5
End Function

Private Function GASRMP(PRS As Single, TEM As Single, SPG As Single, _
                        GRMP As Single) As Integer

'Converted from FORTRAN version provided by MHA (Greg Hueni)
'
'   This function calculates the Real gas pseudo-pressure (potential)
'     M(P) which is 2 times the integral of PRS/(VISG*ZED) dPRS from
'     PRSMIN (= 0) to PRS using Romberg integration.
'
' DEFINITIONS
' PRS - Pressure(PSIA)
'     TEM - Temperature (degrees Rankine)
'     SPG - Gas specific gravity (air = 1)
'
'  Error return:
'            0 - OK
'     1 thru 5 - Viscosity error (see GASVIS)
'            6 - No convergence
'            7 - Unknow error
'
' CALLS - GASVIS, ZFACT
'

  Dim MAXIT As Integer, i As Integer, J As Integer, K As Integer, N As Integer
  Dim Err As Integer
  Dim R(1 To 21) As Single, TOL As Single, PRSMIN As Single, h As Single
  Dim PRSPR As Single, TEMPR As Single, ZED As Single
  Dim VISG As Single, OLD1 As Single, Sum As Single, PRSARG As Single
  Dim DENOM As Single, OLD2 As Single
  Dim VisErr As Integer

  GASRMP = 0
  GRMP = 0#
  MAXIT = 20
  TOL = 0.000001

  PRSMIN = 0#
  h = PRS - PRSMIN
'
'---CALL ZFACT, GASVIS for PRSMIN (if a different value is wanted)
'
  On Error GoTo ErrExit
  
  PRSPR = PRS / pC
  TEMPR = TEM / tC
  Call ZFACT(TEMPR, PRSPR, ZED, Err)
  VisErr = GASVIS(TEM, PRS, ZED, SPG, VISG)
  If VisErr > 0 Then
    GASRMP = VisErr
    Exit Function
  End If
'
'---R(1) must be altered if a non-zero PRSMIN is used.  Using the integral
'---bounds as A and B and an integrand of F(X),  R(1)=H/2*(F(A)+F(B)).
'
  R(1) = (h / 2#) * (2# * PRS) / (VISG * ZED)
  N = 1

  For K = 1 To MAXIT
    OLD1 = R(1)
    Sum = 0#
    PRSARG = PRSMIN + h / 2#

    For J = 1 To N
      PRSPR = PRSARG / pC
      TEMPR = TEM / tC
      Call ZFACT(TEMPR, PRSPR, ZED, Err)
      VisErr = GASVIS(TEM, PRSARG, ZED, SPG, VISG)
      If VisErr > 0 Then
        GASRMP = VisErr
        Exit Function
      End If
      Sum = Sum + (2# * PRSARG) / (VISG * ZED)
      PRSARG = PRSARG + h
    Next J
          
    R(1) = (OLD1 + h * Sum) / 2#
    DENOM = 4#

    For i = 2 To K + 1
      OLD2 = R(i)
      R(i) = R(i - 1) + (R(i - 1) - OLD1) / (DENOM - 1#)
      OLD1 = OLD2
      DENOM = DENOM * 4#
    Next i

    h = h / 2#
    N = N * 2
    If Abs((R(K + 1) - R(K)) / R(K + 1)) < TOL Then
      GRMP = R(K + 1)
      Exit Function
    End If
  Next K
  GASRMP = 6
  Exit Function

ErrExit:
  GASRMP = 7

End Function

Public Function CalcGas() As Integer
  Dim CP, CT, CMW
  Dim ghC As Single, gasMW As Single
  Dim K As Integer
  Dim Err As Integer, VisErr As Integer, RmpErr As Integer
  Dim resTempR As Single
  Dim CurPrs As Single
  Dim TbPlus As Single
  Dim PGG As Single
  Dim PGG2 As Single
  Dim PMW As Single
'
'---Set gas composition constants
'
  GO = False
  CP = Array(667.8, 707.8, 616.3, 550.7, 529.1, 488.6, 490.4, 436.9, 396.8, _
             360.6, 332#, 304#, 493#, 1070.9, 1306#)
             
  CT = Array(-116.6, 90.1, 206#, 305.7, 275#, 385.7, 369.1, 453.7, 512.8, _
              564.2, 610.7, 652.1, -232.4, 87.9, 212.7)
              
  CMW = Array(16.043, 30.07, 44.097, 58.124, 58.124, 72.151, 72.151, _
              86.178, 100.21, 114.23, 128.26, 142.29, 28.013, 44.01, 34.076)
'
'---Calc critical temperature and pressure (explicit Gas composition).   Check if
'---a hydrocarbon "plus" value is present
'
  If objGas.GasEntry = "C" Then
    pC = 0#
    tC = 0#
    ghC = 0#
    gasMW = 0#
    GasGrav = 0#
    
    If objGas.PlusEntry > 0 Then
      PGG = objGas.PLUSgg
      PGG2 = PGG * PGG
      PMW = objGas.PLUSmw
      K = objGas.PlusEntry
      
      CMW(K) = objGas.PLUSmw
      TbPlus = (4.5579 * PMW ^ 0.15178 * PGG ^ 0.15427) ^ 3
      CP(K) = Exp(8.3634 - (0.0566 / PGG) - (0.24244 + (2.2898 / PGG) + _
              (0.11857 / PGG2)) * (TbPlus / 1000#) + _
              (1.4685 + (3.648 / PGG) + (0.47227 / PGG2)) * (TbPlus ^ 2 / 10000000#) - _
              (0.42019 + (1.6977 / PGG2)) * (TbPlus ^ 3 / 10000000000#))
      CT(K) = (341.7 + 811# * PGG) + (0.4244 + 0.1174 * PGG) * TbPlus + _
              (0.4669 - 3.2623 * PGG) * (100000# / TbPlus)
      
    End If
    For K = 1 To 12                             'Calc hydrocarbon gas gravity
      gasMW = gasMW + objGas.GetGC(K) * CMW(K)
    Next K
    ghC = gasMW / 28.96
    
    For K = 13 To 15                            'Calc total gas gravity
      gasMW = gasMW + objGas.GetGC(K) * CMW(K)
    Next K
    GasGrav = gasMW / 28.96
    
    For K = 1 To 15                             'Critical T and P
      tC = tC + objGas.GetGC(K) * CT(K)
      pC = pC + objGas.GetGC(K) * CP(K)
    Next K
    tC = tC + 460#
'
'---Calc critical temperature and pressure (gravity and inerts)
'
  Else
    ghC = (GasGrav - 0.9672 * objGas.mfN - 1.5195 * objGas.mfCO2 - _
           1.1765 * objGas.mfH2S) / (1# - objGas.mfN - objGas.mfCO2 - objGas.mfH2S)
    gasMW = 28.96 * GasGrav
    
    Call GASTCPC(ghC, gasMW)
  End If
'
'---Calc correlation for each pressure
'
  STDenGas = objFluid.StdP * gasMW / (10.73 * objFluid.StdTr)
  resTempR = objFluid.ResTr
  pvtPTR = resTempR / tC
  
  For K = 1 To objPrs.NPRSo
    CurPrs = objPrs.GetPRSo(K)
    
    pvtPPR(K) = CurPrs / pC
    Call ZFACT(pvtPTR, pvtPPR(K), pvtZ(K), Err)
    pvtPZ(K) = CurPrs / pvtZ(K)
    pvtBG(K) = pvtZ(K) * resTempR * objFluid.StdP / (objFluid.StdTr * CurPrs)
    pvtRHOG(K) = CurPrs * gasMW / (pvtZ(K) * 10.73 * resTempR)
    Call GASCOM(CurPrs, resTempR, pvtCG(K))
    VisErr = GASVIS(resTempR, CurPrs, pvtZ(K), GasGrav, pvtMUG(K))
    If VisErr > 0 Then
      CalcGas = VisErr
      Exit Function
    End If
    
    RmpErr = GASRMP(CurPrs, resTempR, GasGrav, pvtRMP(K))
    If RmpErr > 0 Then
      CalcGas = RmpErr
      Exit Function
    End If
  Next K
  GO = True
End Function
'
'---Return calculated value methods
'
Public Function GetPPR(N As Integer) As Single
  GetPPR = pvtPPR(N)
End Function

Public Function GetZ(N As Integer) As Single
  GetZ = pvtZ(N)
End Function

Public Function GetPZ(N As Integer) As Single
  GetPZ = pvtPZ(N)
End Function

Public Function GetBG(N As Integer) As Single
  GetBG = pvtBG(N)
End Function

Public Function GetMUG(N As Integer) As Single
  GetMUG = pvtMUG(N)
End Function

Public Function GetCG(N As Integer) As Single
  GetCG = pvtCG(N)
End Function

Public Function GetRHOG(N As Integer) As Single
  GetRHOG = pvtRHOG(N)
End Function

Public Function GetRMP(N As Integer) As Single
  GetRMP = pvtRMP(N) / 1000000#
End Function

