VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsOil"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'
'Title: clsOil
'
'Author: D.V. Allred (Analytics CIS, 07/Jan/2002)
'
'Description: Oil properties input and calculations
'
'Assumptions: Object created in frmMain
'
Option Explicit

Private pvtAPI As Single                 'API oil gravity
Private pvtOilSG As Single               'Oil specific gravity

Private pvtSepGG As Single               'Separator gas gravity
Private pvtSepGGn As Single              'Separator gas gravity (normalized)
Private pvtSepP As Single                'Separator pressure
Private pvtSepTf As Single               'Separator temperature (F)
Private pvtSepTc As Single               'Separator temperature (C)
Private pvtSepTr As Single               'Separator temperature (R)
Private pvtSepTU As String               'Separator Temperature Units (F or C)

Private pvtRSI As Single                 'RSI (Initial Dissolved Gas-Oil Ratio)
Private pvtRSITank As Single             'Derived RSI tank
Private pvtRSISep As Single              'Derived RSI separator
Private pvtPbp As Single                 'Bubble point pressure
Private pvtRSOS(1 To 30) As Single       'Saturated dissolved gas/oil ratio
Private pvtRSO(1 To 30) As Single        'Under-Saturated dissolved gas/oil ratio
Private pvtCO(1 To 30) As Single         'Under-Saturated oil compressibility at Pbp
Private pvtBOS(1 To 30) As Single        'Saturated oil formation volume factor
Private pvtBO(1 To 30) As Single         'Under-Saturated oil formation volume factor
Private pvtMUOS(1 To 30) As Single       'Saturated oil viscosity
Private pvtMUO(1 To 30) As Single        'Under-Saturated oil viscosity
Private pvtRHOOS(1 To 30) As Single      'Saturated oil density
Private pvtRHOO(1 To 30) As Single       'Under-Saturated oil density
Private pvtSTDenOil As Single            'Stock tank oil density
Private pvtSTDenGas As Single            'Stock tank gas density
Private pvtGO As Boolean                 'Oil properties available (GO=TRUE)
'
'---Class Initialize
'
Private Sub Class_Initialize()
  API = 35#
  SepGG = 0.9
  SepP = 50#
  SepTU = "F"
  SepTf = 75#
  RSI = 683.9
  STDenOil = 53#
  GO = False
End Sub
'
'============================== Properties ====================================
'
'
'---API
'
Public Property Get API() As Single
  API = pvtAPI
End Property

Private Property Let API(ByVal vNewValue As Single)
  pvtAPI = vNewValue
  pvtOilSG = 141.5 / (131.5 + pvtAPI)
End Property
'
'---Oil specific gravity
'
Public Property Get OilSG() As Single
  OilSG = pvtOilSG
End Property

Private Property Let OilSG(ByVal vNewValue As Single)
  pvtOilSG = vNewValue
  If pvtOilSG > 0# Then pvtAPI = (141.5 / pvtOilSG) - 131.5
End Property
'
'---Separator gas gravity
'
Public Property Get SepGG() As Single
  SepGG = pvtSepGG
End Property

Private Property Let SepGG(ByVal vNewValue As Single)
  pvtSepGG = vNewValue
End Property
'
'---Separator gas gravity (normalized)
'
Public Property Get SepGGn() As Single
  SepGGn = pvtSepGGn
End Property

Private Property Let SepGGn(ByVal vNewValue As Single)
  pvtSepGGn = vNewValue
End Property
'
'---Separator pressure
'
Public Property Get SepP() As Single
  SepP = pvtSepP
End Property

Private Property Let SepP(ByVal vNewValue As Single)
  pvtSepP = vNewValue
End Property
'
'---Separator Temperature Units
'
Public Property Get SepTU() As String
  SepTU = pvtSepTU
End Property

Public Property Let SepTU(ByVal vNewValue As String)
  If vNewValue <> pvtSepTU Then
    Select Case vNewValue
      Case "F", "f": SepTf = SepTc
      Case "C", "c": SepTc = SepTf
    End Select
    
    pvtSepTU = UCase(vNewValue)
  End If
End Property
'
'---Separator Temperature (F)
'
Public Property Get SepTf() As Single
  SepTf = pvtSepTf
End Property

Private Property Let SepTf(ByVal vNewValue As Single)
  pvtSepTf = vNewValue
  pvtSepTr = pvtSepTf + 460#
  pvtSepTc = (pvtSepTf - 32#) / 1.8
End Property
'
'---Separator Temperature (C)
'
Public Property Get SepTc() As Single
  SepTc = pvtSepTc
End Property

Private Property Let SepTc(ByVal vNewValue As Single)
  pvtSepTc = vNewValue
  pvtSepTf = (pvtSepTc * 1.8) + 32#
  pvtSepTr = pvtSepTf + 460#
End Property
'
'---Separator Temperature (Rankine)
'
Public Property Get SepTr() As Single
  SepTr = pvtSepTr
End Property
'
'---RSI.  Calculate Pbp, RsiTank and RsiSep
'
Public Property Get RSI() As Single
  RSI = pvtRSI
End Property

Private Property Let RSI(ByVal vNewValue As Single)
  Dim x As Single
  
  pvtRSI = vNewValue

  RSITank = 0.3818 - 5.506 * objUtil.aLog10(OilSG) _
                   + 2.902 * objUtil.aLog10(SepGG) _
                   + 1.327 * objUtil.aLog10(SepP) _
                   - 0.7355 * objUtil.aLog10(SepTf)
  RSITank = 10 ^ RSITank
  
  x = ((pvtRSI / SepGG) ^ 0.83) * (10 ^ (0.00091 * objFluid.ResTf - 0.0125 * API))
  pvtPbp = 18.2 * (x - 1.4)
  RSISep = pvtRSI - RSITank
End Property
'
'---RSI Tank
'
Public Property Get RSITank() As Single
  RSITank = pvtRSITank
End Property

Private Property Let RSITank(ByVal vNewValue As Single)
  pvtRSITank = vNewValue
End Property
'
'---RSI Separator
'
Public Property Get RSISep() As Single
  RSISep = pvtRSISep
End Property

Private Property Let RSISep(ByVal vNewValue As Single)
  pvtRSISep = vNewValue
End Property
'
'---Bubble point presuure.  Calculate Rsi, RsiTank and RsiSep for new value
'
Public Property Get Pbp() As Single
  Pbp = pvtPbp
End Property

Private Property Let Pbp(ByVal vNewValue As Single)
  Dim x As Single
  
  pvtPbp = vNewValue

  RSITank = 0.3818 - 5.506 * objUtil.aLog10(OilSG) _
                   + 2.902 * objUtil.aLog10(SepGG) _
                   + 1.327 * objUtil.aLog10(SepP) _
                   - 0.7355 * objUtil.aLog10(SepTf)
  RSITank = 10 ^ RSITank
  
  x = (pvtPbp / 18.2) + 1.4
  x = x / (10 ^ (0.00091 * objFluid.ResTf - 0.0125 * API))
  
  pvtRSI = x ^ (1# / 0.83) * SepGG
  RSISep = RSI - RSITank
End Property
'
'---Stock tank oil density
'
Public Property Get STDenOil() As Single
  STDenOil = pvtSTDenOil
End Property

Private Property Let STDenOil(ByVal vNewValue As Single)
  pvtSTDenOil = vNewValue
End Property
'
'---Stock tank gas density
'
Public Property Get STDenGas() As Single
  STDenGas = pvtSTDenGas
End Property

Private Property Let STDenGas(ByVal vNewValue As Single)
  pvtSTDenGas = vNewValue
End Property
'
'---Oil propertes calc'ed
'
Public Property Get GO() As Boolean
  GO = pvtGO
End Property

Private Property Let GO(ByVal vNewValue As Boolean)
  pvtGO = vNewValue
End Property
'
'============================== Methods ====================================
'
'
'---Set API method
'
Public Sub SetAPI(pValue As String)
  API = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set Oil specific gravity method
'
Public Sub SetOilSG(pValue As String)
  OilSG = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set separator gas gravity method
'
Public Sub SetSepGG(pValue As String)
  SepGG = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set separator pressure method
'
Public Sub SetSepP(pValue As String)
  SepP = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set separator temperature method
'
Public Sub SetSepT(tValue As String)
  Select Case SepTU
    Case "F", "f": SepTf = objUtil.Str2Nbr(tValue)
    Case "C", "c": SepTc = objUtil.Str2Nbr(tValue)
  End Select
End Sub
'
'---Set Bubble point pressure method
'
Public Sub SetPbp(pValue As String)
  Pbp = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set Rsi method
'
Public Sub SetRsi(rValue As String)
  RSI = objUtil.Str2Nbr(rValue)
End Sub
'
'---Calc Oil Properties
'
Public Function CalcOil() As Integer
  Dim K As Integer
  Dim x, A, B, visoDead As Single
  Dim CurPrs As Single
  Dim BPPi As Integer
  Dim ResTf As Single
'
'---Set constants
'
  GO = False
  BPPi = objPrs.BPPi                                'Bubble point pressure index
  ResTf = objFluid.ResTf                            'Reservoir temp (F)
  STDenOil = OilSG * 62.4                           'Stock tank oil density
  STDenGas = objFluid.StdP * 28.96 * _
             SepGG / (10.73 * objFluid.StdTr)       'Stock tank gas density
  
  visoDead = 1.8653 - 0.02508 * API - 0.5644 * objUtil.aLog10(ResTf)
  visoDead = 10 ^ (10 ^ visoDead) - 1#
  SepGGn = SepGG * (1# + 0.00005912 * API * SepTf * _
              objUtil.aLog10(SepP / 114.7))
'
'---For each pressure in the array, calculate...
'
  On Error GoTo ErrExit
  
  For K = objPrs.NPRSo To 1 Step -1
    CurPrs = objPrs.GetPRSo(K)
'
'---Saturated dissolved gas-oil ratio
'
    x = (CurPrs / 18.2) + 1.4
    x = x / (10 ^ (0.00091 * ResTf - 0.0125 * API))
    pvtRSOS(K) = x ^ (1# / 0.83) * SepGG
'
'---Under-Saturated dissolved gas-oil ratio
'
    If K >= BPPi Then
      pvtRSO(K) = pvtRSOS(K)
    Else
      pvtRSO(K) = pvtRSOS(BPPi)
    End If
'
'---Under-Saturated Oil compressibility
'
    pvtCO(K) = (-1433# + 5# * pvtRSO(K) + 17.2 * ResTf - 1180# * _
              objGas.GasGrav + 12.61 * API) / (100000# * CurPrs)
'
'---Saturated Oil formation volume factor
'
    x = pvtRSOS(K) * ((SepGG / OilSG) ^ 0.5) + 1.25 * ResTf
    pvtBOS(K) = 0.9759 + 0.00012 * (x ^ 1.2)
'
'---Under-Saturated Oil formation volume factor
'
    If K >= BPPi Then
      pvtBO(K) = pvtBOS(K)
    Else
'      pvtBO(K) = pvtBOS(BPPi) * Exp(pvtCO(BPPi) * (Pbp - CurPrs))
      pvtBO(K) = pvtBO(K + 1) * Exp(((pvtCO(K) + pvtCO(K + 1)) / 2#) * _
                                    (objPrs.GetPRSo(K + 1) - CurPrs))
    End If
'
'---Saturated Oil viscosity
'
    A = 10.715 * (pvtRSOS(K) + 100#) ^ (-0.515)
    B = 5.44 * (pvtRSOS(K) + 150#) ^ (-0.338)
    pvtMUOS(K) = A * visoDead ^ B
'
'---Under-Saturated Oil viscosity
'
    If K >= BPPi Then
      pvtMUO(K) = pvtMUOS(K)
    Else
      A = 2.6 * (CurPrs ^ 1.187) * Exp(-0.0000898 * CurPrs - 11.513)
      pvtMUO(K) = pvtMUOS(BPPi) * (CurPrs / Pbp) ^ A
    End If
'
'---Saturated and under-saturated oil density
'
    pvtRHOOS(K) = (STDenOil + STDenGas * pvtRSOS(K) / 5.615) / pvtBOS(K)
    pvtRHOO(K) = (STDenOil + STDenGas * pvtRSO(K) / 5.615) / pvtBO(K)

  Next K
  GO = True
  Exit Function
  
ErrExit:
  Call ErrMSGx("Error at pressure " & CurPrs & " - Please adjust input paramemters!", _
              "Oil Properties Error")
End Function
'
'---Return calculated value methods
'
Public Function GetBO(N As Integer) As Single
  GetBO = pvtBO(N)
End Function

Public Function GetBOS(N As Integer) As Single
  GetBOS = pvtBOS(N)
End Function

Public Function GetRSO(N As Integer) As Single
  GetRSO = pvtRSO(N)
End Function

Public Function GetRSOS(N As Integer) As Single
  GetRSOS = pvtRSOS(N)
End Function

Public Function GetCO(N As Integer) As Single
  GetCO = pvtCO(N)
End Function

Public Function GetMUO(N As Integer) As Single
  GetMUO = pvtMUO(N)
End Function

Public Function GetMUOS(N As Integer) As Single
  GetMUOS = pvtMUOS(N)
End Function

Public Function GetRHOO(N As Integer) As Single
  GetRHOO = pvtRHOO(N)
End Function

Public Function GetRHOOS(N As Integer) As Single
  GetRHOOS = pvtRHOOS(N)
End Function

