VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsWater"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private pvtPPM As Long                   'Water salinity (parts/million)
Private pvtPER As Single                 'Water salinity (wt% solids)

Private pvtBW(1 To 30) As Single         'Volume factor
Private pvtRSW(1 To 30) As Single        'Dissolved Gas/oil ratio
Private pvtCW(1 To 30) As Single         'Compressibility
Private pvtMUW(1 To 30) As Single        'Viscosity
Private pvtRHOW(1 To 30) As Single       'Density
Private pvtSTDenWat As Single            'Stock tank density (water)
Private pvtSTDenGas As Single            'Stock tank density (gas)
Private pvtGO As Boolean                 'Water properties available (GO=TRUE)
'
'---Class Initialize
'
Private Sub Class_Initialize()
  PPM = 10000
  STDenWat = 62.4
  GO = False
End Sub
'
'============================== Properties ====================================
'
'
'---Parts/Million
'
Public Property Get PPM() As Long
  PPM = pvtPPM
End Property

Private Property Let PPM(ByVal vNewValue As Long)
  pvtPPM = vNewValue
  pvtPER = CSng(pvtPPM) / 10000#
End Property
'
'---Wt % solids
'
Public Property Get PER() As Single
  PER = pvtPER
End Property

Private Property Let PER(ByVal vNewValue As Single)
  pvtPER = vNewValue
  pvtPPM = CLng(pvtPER * 10000#)
End Property
'
'---Stock tank water density
'
Public Property Get STDenWat() As Single
  STDenWat = pvtSTDenWat
End Property

Private Property Let STDenWat(ByVal vNewValue As Single)
  pvtSTDenWat = vNewValue
End Property
'
'---Stock tank gas density
'
Public Property Get STDenGas() As Single
  STDenGas = pvtSTDenGas
End Property

Private Property Let STDenGas(ByVal vNewValue As Single)
  pvtSTDenGas = vNewValue
End Property
'
'---Water propertes calc'ed
'
Public Property Get GO() As Boolean
  GO = pvtGO
End Property

Private Property Let GO(ByVal vNewValue As Boolean)
  pvtGO = vNewValue
End Property
'
'============================== Methods ====================================
'
'
'---Set PPM method
'
Public Sub SetPPM(pValue As String)
  PPM = CLng(objUtil.Str2Nbr(pValue))
End Sub
'
'---Set PER method
'
Public Sub SetPER(pValue As String)
  PER = objUtil.Str2Nbr(pValue)
End Sub
'
'---Calc water properties
'
Public Function CalcWater()
  Dim K As Integer
  Dim A As Single, B As Single, C As Single, D As Single, E As Single, X As Single
  Dim Solids As Single, vwTfac As Single, vwPfac As Single
  Dim MUW1atm As Single
  Dim resTempf As Single, CurPrs As Single
'
'---Set constants
'
  GO = False
  resTempf = objFluid.ResTf
  vwTfac = 0.000133391 * resTempf + 0.000000550654 * resTempf ^ 2# - 0.010001
  STDenWat = 62.368 + 0.438603 * PER + 0.00160074 * PER ^ 2#
  STDenGas = objFluid.StdP * 28.96 * 0.6 / (objFluid.StdTr * 10.73)
  Solids = PER * 10000# * STDenWat / 62.368
    
  X = -0.0840655 * PER * resTempf ^ (-0.285854)
  
  A = 8.15839 - 0.0612265 * resTempf + 0.000191663 * _
      resTempf ^ 2# - 0.00000021654 * resTempf ^ 3#
  
  B = 0.0101021 - 0.0000744241 * resTempf + 0.000000305553 * resTempf ^ 2# - _
      0.000000000294883 * resTempf ^ 3#
  
  C = -0.0000001 * (9.02505 - 0.130237 * resTempf + 0.000853425 * resTempf ^ 2# - _
      0.00000234122 * resTempf ^ 3# + 0.00000000237049 * resTempf ^ 4#)
  
  D = 109.574 - 8.40564 * PER + 0.31331 * PER ^ 2# + 0.00872213 * PER ^ 3#
  E = 1.12166 - 0.0263951 * PER + 0.000679461 * PER ^ 2# + _
      0.0000547119 * PER ^ 3# - 0.00000155586 * PER ^ 4#
  MUW1atm = D * resTempf ^ (-E)
'
'---Calc volume factor, gas dissolved in water, compressibility, density & viscosity
'
  For K = 1 To objPrs.NPRSo
    CurPrs = objPrs.GetPRSo(K)
    vwPfac = (-0.00000000195301 * CurPrs * resTempf) - _
             (1.72834E-13 * resTempf * CurPrs ^ 2#) - _
             (0.000000358922 * CurPrs) - _
             (0.000000000225341 * CurPrs ^ 2#)
    pvtBW(K) = (1# + vwPfac) * (1# + vwTfac)
    
    pvtRSW(K) = (A + B * CurPrs + C * CurPrs ^ 2#) * 10# ^ X
    pvtCW(K) = 1# / (7.033 * CurPrs + 0.5415 * Solids - 537# * resTempf + 403300#)
    pvtRHOW(K) = (STDenWat + (pvtRSW(K) / 5.165) * STDenGas) / pvtBW(K)
    pvtMUW(K) = MUW1atm * _
                (0.9994 + 0.000040295 * CurPrs + 0.0000000031062 * CurPrs ^ 2#)
  Next K
  GO = True
  
End Function
'
'---Return calculated value methods
'
Public Function GetBW(N As Integer) As Single
  GetBW = pvtBW(N)
End Function

Public Function GetRSW(N As Integer) As Single
  GetRSW = pvtRSW(N)
End Function

Public Function GetCW(N As Integer) As Single
  GetCW = pvtCW(N)
End Function

Public Function GetMUW(N As Integer) As Single
  GetMUW = pvtMUW(N)
End Function

Public Function GetRHOW(N As Integer) As Single
  GetRHOW = pvtRHOW(N)
End Function

