﻿Option Explicit On

Public Class clsECL
    '
    'Title: clsECL
    '
    'Author: D.V. Allred (Analytics CIS, 03/Jun/2015)
    '
    'Description: Produce MHA-P3 Eclipse export files
    '
    'Assumptions: None
    '
    Private objFILE As System.IO.StreamWriter

    Private pvtPVDO As Boolean               'Output PVDO - Oil with no gas
    Private pvtPVTO As Boolean               'Output PVTO - Oil with gas
    Private pvtPVDG As Boolean               'Output PVDG - Gas reservoir
    Private pvtPVTW As Boolean               'Output PVTW - Water
    Private pvtDENS As Boolean               'Output DENSITY
    Private pvtSWOF As Boolean               'Output SWOF - Water/Oil rock
    Private pvtSGOF As Boolean               'Output SGOF - Gas/Oil rock
    Private pvtROCK As Boolean               'Output ROCK - Rock compressibilty

    Private pvtFolderName As String          'Default Output folder name
    Private pvtFileName As String            'Default Output file name
    Private pvtPVDOFile As String            'ECL PVDO file name
    Private pvtPVTOFile As String            'ECL PVTO file name
    Private pvtPVDGFile As String            'ECL PVDG file name
    Private pvtPVTWFile As String            'ECL PVTW file name
    Private pvtDENSFile As String            'ECL DENSITY file name
    Private pvtSWOFFile As String            'ECL SWOF file name
    Private pvtSGOFFile As String            'ECL SGOF file name
    Private pvtROCKFile As String            'ECL ROCK file name
    '
    '---Class Initialize/Terminate
    '
    Public Sub New()
        pvtFolderName = "C:\MHA\Eclipse\"
        Filename = pvtFolderName & "Eclipse"
        My.Computer.FileSystem.CreateDirectory(pvtFolderName)

        pvtPVDO = False
        pvtPVTO = False
        pvtPVDG = False
        pvtPVTW = False
        pvtDENS = False
        pvtSWOF = False
        pvtSGOF = False
        pvtROCK = False
    End Sub

    Protected Overrides Sub Finalize()
        '    pvtWells = Nothing
    End Sub
    '
    '============================== Properties ====================================
    '
    '
    '---Default filename
    '
    Public Property Filename() As String
        Get
            Filename = pvtFileName
        End Get

        Set(newValue As String)
            pvtFileName = newValue
            pvtPVDOFile = pvtFileName & ".pvdo"
            pvtPVTOFile = pvtFileName & ".pvto"
            pvtPVDGFile = pvtFileName & ".pvdg"
            pvtPVTWFile = pvtFileName & ".pvtw"
            pvtDENSFile = pvtFileName & ".dens"
            pvtSWOFFile = pvtFileName & ".swof"
            pvtSGOFFile = pvtFileName & ".sgof"
            pvtROCKFile = pvtFileName & ".rock"

        End Set
    End Property
    '
    '---filename properties
    '
    Public Property DENSFile() As String
        Get
            DENSFile = pvtDENSFile
        End Get
        Set(newValue As String)
            pvtDENSFile = newValue
        End Set
    End Property

    Public Property PVDOFile() As String
        Get
            PVDOFile = pvtPVDOFile
        End Get
        Set(newValue As String)
            pvtPVDOFile = newValue
        End Set
    End Property

    Public Property PVTOFile() As String
        Get
            PVTOFile = pvtPVTOFile
        End Get
        Set(newValue As String)
            pvtPVTOFile = newValue
        End Set
    End Property

    Public Property PVDGFile() As String
        Get
            PVDGFile = pvtPVDGFile
        End Get
        Set(newValue As String)
            pvtPVDGFile = newValue
        End Set
    End Property

    Public Property PVTWFile() As String
        Get
            PVTWFile = pvtPVTWFile
        End Get
        Set(newValue As String)
            pvtPVTWFile = newValue
        End Set
    End Property

    Public Property SWOFFile() As String
        Get
            SWOFFile = pvtSWOFFile
        End Get
        Set(newValue As String)
            pvtSWOFFile = newValue
        End Set
    End Property

    Public Property SGOFFile() As String
        Get
            SGOFFile = pvtSGOFFile
        End Get
        Set(newValue As String)
            pvtSGOFFile = newValue
        End Set
    End Property

    Public Property ROCKFile() As String
        Get
            ROCKFile = pvtROCKFile
        End Get
        Set(newValue As String)
            pvtROCKFile = newValue
        End Set
    End Property
    '
    '---Properties flags
    '
    Public Property PVDO() As Boolean
        Get
            PVDO = pvtPVDO
        End Get

        Set(newValue As Boolean)
            pvtPVDO = newValue
        End Set
    End Property

    Public Property PVTO() As Boolean
        Get
            PVTO = pvtPVTO
        End Get

        Set(newValue As Boolean)
            pvtPVTO = newValue
        End Set
    End Property

    Public Property PVDG() As Boolean
        Get
            PVDG = pvtPVDG
        End Get

        Set(newValue As Boolean)
            pvtPVDG = newValue
        End Set
    End Property

    Public Property PVTW() As Boolean
        Get
            PVTW = pvtPVTW
        End Get

        Set(newValue As Boolean)
            pvtPVTW = newValue
        End Set
    End Property

    Public Property DENS() As Boolean
        Get
            DENS = pvtDENS
        End Get

        Set(newValue As Boolean)
            pvtDENS = newValue
        End Set
    End Property

    Public Property SWOF() As Boolean
        Get
            SWOF = pvtSWOF
        End Get

        Set(newValue As Boolean)
            pvtSWOF = newValue
        End Set
    End Property

    Public Property SGOF() As Boolean
        Get
            SGOF = pvtSGOF
        End Get

        Set(newValue As Boolean)
            pvtSGOF = newValue
        End Set
    End Property

    Public Property ROCK() As Boolean
        Get
            ROCK = pvtROCK
        End Get

        Set(newValue As Boolean)
            pvtROCK = newValue
        End Set
    End Property
    '
    '============================== Methods ====================================
    '
    '
    '---ECL Export method
    '
    Public Sub Execute()
        Dim ii, iiREF As Int32
        Dim maxPi As Int32 = 0
        Dim kk As Int32
        Dim X, Y, A, Z As Single
        Dim fileIDs As String
        Dim DMUWDP As Single
        Dim strVAL As String = ""
        Dim strLBL As String = ""

        fileIDs = ""
        '
        '---PVDO - Oil with no gas
        '
        If pvtPVDO Then
            fileIDs = "PVDO "
            objFILE = My.Computer.FileSystem.OpenTextFileWriter(pvtPVDOFile, False)

            objFILE.WriteLine("PVDO")

            objFILE.Write(vbNewLine)
            objFILE.WriteLine("-- STOCK TANK OIL DENSITY (LB/SCF) = " & STDENOIL.ToString("F3").PadLeft(10, " "))
            objFILE.Write(vbNewLine)

            objFILE.WriteLine("--   PRESS(PSIA)    BO(RB/MCF)    MUO(CP)")

            For ii = IPBP To 1 Step -1
                '               objFILE.Write(objUtil.Nbr2Str(objPrs.GetPRSo(ii), 15, 1))
                '               objFILE.Write(objUtil.Nbr2Str(objOil.GetBO(ii), 14, 4))
                '               objFILE.WriteLine(objUtil.Nbr2Str(objOil.GetMUO(ii), 11, 3))

                objFILE.Write(PVTPRS(ii).ToString("F1").PadLeft(15, " "))
                objFILE.Write(BO(ii).ToString("F4").PadLeft(14, " "))
                objFILE.WriteLine(MUO(ii).ToString("F3").PadLeft(11, " "))
            Next ii
            objFILE.WriteLine("/")
            objFILE.Write(vbNewLine & vbNewLine & vbNewLine)

            X = RSO(IPBP) / 1000.0

            objFILE.WriteLine("RSCONST")
            objFILE.Write(X.ToString("F3").PadLeft(12, " ") & PBP.ToString("F1").PadLeft(9, " ") & "     /")
            objFILE.Write(vbNewLine)

            objFILE.Close()
        End If
        '
        '---PVTO - Oil with dissolved gas
        '
        If pvtPVTO Then
            fileIDs = fileIDs & "PVTO "
            objFILE = My.Computer.FileSystem.OpenTextFileWriter(pvtPVTOFile, False)

            objFILE.WriteLine("PVTO")
            objFILE.Write(vbNewLine)

            objFILE.WriteLine("-- STOCK TANK OIL DENSITY (LB/SCF) = " & STDENOIL.ToString("F3").PadLeft(10, " "))
            objFILE.Write(vbNewLine)

            objFILE.WriteLine("--  RSO(MCF/B)   PRESS(PSIA)    BO(RB/MCF)    MUO(CP)")

            For ii = NPVTPRS To 1 Step -1
                X = RSOS(ii) / 1000.0

                objFILE.Write(X.ToString("F3").PadLeft(14, " "))
                objFILE.Write(PVTPRS(ii).ToString("F1").PadLeft(15, " "))
                objFILE.Write(BOS(ii).ToString("F4").PadLeft(14, " "))
                objFILE.Write(MUOS(ii).ToString("F3").PadLeft(11, " "))
                '
                '---Output undersaturated values at Pbp
                '
                If ii = IPBP Then
                    objFILE.Write(vbNewLine)
                    For kk = ii - 1 To 1 Step -1
                        objFILE.Write(PVTPRS(kk).ToString("F1").PadLeft(29, " "))
                        objFILE.Write(BO(kk).ToString("F4").PadLeft(14, " "))
                        objFILE.Write(MUO(kk).ToString("F3").PadLeft(11, " "))
                        If kk > 1 Then
                            objFILE.WriteLine(" ")
                        End If
                    Next kk
                End If

                If ii > 1 Then
                    objFILE.WriteLine("   /")
                Else
                    objFILE.WriteLine(" ")
                End If
            Next ii

            X = PVTPRS(1) + 200.0
            Y = BOS(1) * Math.Exp(CO(1) * (-200.0))
            A = 2.6 * (X ^ 1.187) * Math.Exp(-0.0000898 * X - 11.513)
            Z = MUOS(1) * (X / PVTPRS(1)) ^ A

            objFILE.Write(Space(18) & X.ToString("F1").PadLeft(11, " "))
            objFILE.Write(Y.ToString("F4").PadLeft(14, " "))
            objFILE.WriteLine(Z.ToString("F3").PadLeft(11, " ") & "   /")
            objFILE.WriteLine("/")
            objFILE.Write(vbNewLine)

            objFILE.Close()
        End If
        '
        '---PVDG - Gas reservoir
        '
        If pvtPVDG Then
            fileIDs = fileIDs & "PVDG "
            objFILE = My.Computer.FileSystem.OpenTextFileWriter(pvtPVDGFile, False)

            objFILE.WriteLine("PVDG")

            objFILE.Write(vbNewLine)
            objFILE.WriteLine("-- STOCK TANK GAS DENSITY (LB/SCF) = " & STDENGAS.ToString("F3").PadLeft(7, " "))
            objFILE.Write(vbNewLine)

            objFILE.WriteLine("--  PRESSURE(PSIA)   BG(RB/MCF)  MUG(CP)")

            For ii = NPVTPRS To 1 Step -1
                X = BG(ii) * 1000.0# / 5.615

                objFILE.Write(PVTPRS(ii).ToString("F1").PadLeft(15, " "))
                objFILE.Write(X.ToString("F5").PadLeft(15, " "))
                objFILE.WriteLine(MUG(ii).ToString("F4").PadLeft(9, " "))
            Next ii
            objFILE.WriteLine("/")
            objFILE.Write(vbNewLine)

            objFILE.Close()
        End If
        '
        '---PVTW - Water
        '
        If pvtPVTW Then
            fileIDs = fileIDs & "PVTW "
            objFILE = My.Computer.FileSystem.OpenTextFileWriter(pvtPVTWFile, False)
            objFILE.WriteLine("PVTW")

            objFILE.Write(vbNewLine)
            objFILE.WriteLine("-- STOCK TANK WATER DENSITY (LB/SCF) = " & STDENWAT.ToString("F3").PadLeft(7, " "))
            objFILE.Write(vbNewLine)

            objFILE.WriteLine("--  Pref(PSIA)   BWref(RB/STB)   CWref(PSI-1)   MUWref(CP)      DMUWDP")

            iiREF = 1
            If NPVTPRS > 2 Then iiREF = Math.Round(NPVTPRS / 2.0, 0)
            DMUWDP = 0.0
            If iiREF > 1 Then
                DMUWDP = ((MUW(iiREF + 1) - MUW(iiREF - 1)) / (PVTPRS(iiREF + 1) - PVTPRS(iiREF - 1))) / MUW(iiREF)
            End If

            objFILE.Write(PVTPRS(iiREF).ToString("F1").PadLeft(14, " "))
            objFILE.Write(BW(iiREF).ToString("F4").PadLeft(16, " "))
            objFILE.Write(CW(iiREF).ToString("F7").PadLeft(15, " "))
            objFILE.Write(MUW(iiREF).ToString("F3").PadLeft(13, " "))
            objFILE.WriteLine(DMUWDP.ToString("F7").PadLeft(12, " ") & "    /")
            objFILE.Write(vbNewLine)

            objFILE.Close()
        End If
        '
        '---DENSITY
        '
        If pvtDENS Then
            fileIDs = fileIDs & "DENSITY "
            objFILE = My.Computer.FileSystem.OpenTextFileWriter(pvtDENSFile, False)
            objFILE.WriteLine("DENSITY")

            objFILE.Write(vbNewLine)

            objFILE.WriteLine("--  StockTank Oil   StockTank Water   StockTank Gas")

            objFILE.Write(STDENOIL.ToString("F3").PadLeft(17, " "))
            objFILE.Write(STDENWAT.ToString("F3").PadLeft(18, " "))
            objFILE.WriteLine(STDENGAS.ToString("F4").PadLeft(16, " ") & "    /")
            objFILE.Write(vbNewLine)

            objFILE.Close()
        End If
        '
        '---SWOF - Water/Oil rock
        '
        If pvtSWOF Then
            fileIDs = fileIDs & "SWOF "
            objFILE = My.Computer.FileSystem.OpenTextFileWriter(pvtSWOFFile, False)
            objFILE.WriteLine("SWOF")

            objFILE.Write(vbNewLine)
            objFILE.WriteLine("-- Relative Permeability from Correlations")

            objFILE.WriteLine("-- Fluids: Oil/Water System")

            Select Case optRockDisplace
                Case 0 : objFILE.WriteLine("-- Displacement: Drainage")
                Case 1 : objFILE.WriteLine("-- Displacement: Imbibition")
            End Select

            If optRockCorr = 0 Then
                objFILE.WriteLine("-- Lithology: N/A")
            Else
                Select Case optRockLith
                    Case 0 : objFILE.WriteLine("-- Lithology: Sandstone")
                    Case 1 : objFILE.WriteLine("-- Lithology: Limestone")
                End Select
            End If

            objFILE.WriteLine("-- Connate Water: " & SWC.ToString("F2").PadLeft(8, " "))

            Select Case optRockWet
                Case 0 : objFILE.WriteLine("-- Wettability: Water Wet")
                Case 1 : objFILE.WriteLine("-- Wettability: Oil Wet")
            End Select

            objFILE.WriteLine("-- Residual Oil to Water: " & SORW.ToString("F2").PadLeft(8, " "))

            Select Case optRockCorr
                Case 0
                    objFILE.WriteLine("-- Correlation Type: Corey")
                    objFILE.WriteLine("-- K Exponent: " & CoreyK.ToString("F2").PadLeft(5, " "))
                    objFILE.WriteLine("-- N Exponent: " & CoreyN.ToString("F2").PadLeft(5, " "))
                Case 1
                    objFILE.WriteLine("-- Correlation Type: Honarpour")
                    objFILE.WriteLine("-- Porosity (frac): " & HRPoro.ToString("F2").PadLeft(6, " "))
                    objFILE.WriteLine("-- Air Perm (md): " & HRAir.ToString("F2").PadLeft(6, " "))
                    objFILE.WriteLine("-- Krg @Sorg: " & HRKrg.ToString("F2").PadLeft(6, " "))
            End Select

            objFILE.Write(vbNewLine)
            objFILE.WriteLine("--      SWAT     KRWAT     KROIL      PCOW")

            For ii = 1 To NWet
                objFILE.Write(pvtSWet(ii).ToString("F2").PadLeft(12, " "))
                objFILE.Write(pvtKRWat(ii).ToString("F4").PadLeft(10, " "))
                objFILE.Write(pvtKROil(ii).ToString("F4").PadLeft(10, " "))
                objFILE.WriteLine("     0.000")
            Next ii

            objFILE.WriteLine("/")
            objFILE.Write(vbNewLine)

            objFILE.Close()
        End If
        '
        '---SGOF - Gas/Oil rock
        '
        If pvtSGOF Then
            fileIDs = fileIDs & "SGOF "
            objFILE = My.Computer.FileSystem.OpenTextFileWriter(pvtSGOFFile, False)
            objFILE.WriteLine("SGOF")

            objFILE.Write(vbNewLine)
            objFILE.WriteLine("-- Relative Permeability from Correlations")

            objFILE.WriteLine("-- Fluids: Gas/Oil System")

            Select Case optRockDisplace
                Case 0 : objFILE.WriteLine("-- Displacement: Drainage")
                Case 1 : objFILE.WriteLine("-- Displacement: Imbibition")
            End Select

            If optRockCorr = 0 Then
                objFILE.WriteLine("-- Lithology: N/A")
            Else
                Select Case optRockLith
                    Case 0 : objFILE.WriteLine("-- Lithology: Sandstone")
                    Case 1 : objFILE.WriteLine("-- Lithology: Limestone")
                End Select
            End If

            objFILE.WriteLine("-- Connate Water: " & SWC.ToString("F2").PadLeft(8, " "))
            objFILE.WriteLine("-- Residual Oil to Gas: " & SORG.ToString("F2").PadLeft(8, " "))
            objFILE.WriteLine("-- Critical Gas: " & SGCrit.ToString("F2").PadLeft(8, " "))

            Select Case optRockCorr
                Case 0
                    objFILE.WriteLine("-- Correlation Type: Corey")
                    objFILE.WriteLine("-- K Exponent: " & CoreyK.ToString("F2").PadLeft(5, " "))
                    objFILE.WriteLine("-- N Exponent: " & CoreyN.ToString("F2").PadLeft(5, " "))
                Case 1
                    objFILE.WriteLine("-- Correlation Type: Honarpour")
                    objFILE.WriteLine("-- Porosity (frac): " & HRPoro.ToString("F2").PadLeft(6, " "))
                    objFILE.WriteLine("-- Air Perm (md): " & HRAir.ToString("F2").PadLeft(6, " "))
                    objFILE.WriteLine("-- Krg @Sorg: " & HRKrg.ToString("F2").PadLeft(6, " "))
            End Select

            objFILE.Write(vbNewLine)
            objFILE.WriteLine("--      SGAS     KRGAS     KROIL      PCOG")

            For ii = 1 To NWet
                objFILE.Write(pvtSNWet(ii).ToString("F4").PadLeft(12, " "))
                objFILE.Write(pvtKRGas(ii).ToString("F4").PadLeft(10, " "))
                objFILE.Write(pvtKROil(ii).ToString("F4").PadLeft(10, " "))
                objFILE.WriteLine("     0.000")
            Next ii

            objFILE.WriteLine("/")
            objFILE.Write(vbNewLine)
            objFILE.Close()
        End If
        '
        '---ROCK - rock compressiblity
        '
        If pvtROCK Then
            fileIDs = fileIDs & "ROCK "
            objFILE = My.Computer.FileSystem.OpenTextFileWriter(pvtROCKFile, False)
            objFILE.WriteLine("ROCK")

            objFILE.Write(vbNewLine)
            objFILE.WriteLine("-- Rock Compressibility - Pressure, Compressibility")

            objFILE.Write(NewmanPres.ToString("F0").PadLeft(10, " "))
            objFILE.WriteLine("   " & NewmanCF.ToString("E7") & "    /")

            objFILE.Write(vbNewLine)
            objFILE.Close()
        End If

        objFILE = Nothing
        Call InfoMSG("Eclipse output complete!", "Files: " & fileIDs)
    End Sub

End Class
