﻿Option Explicit On
'Imports System.Drawing.Bitmap

Public Class clsGrid
    '
    'Title: clsGRID
    '
    'Author: D.V. Allred (Analytics CIS, 09/Oct/2002)
    '
    'Description: Helps manage an DataGridView control.  Provides methods to build and
    '             maintain grid displays of data as well as row highlighting and
    '             formatting.
    '
    'Assumptions: None
    '
    Private pvtFXG As DataGridView               'Grid control
    Private pvtColCount As Int32                 'Nbr of grid columns
    Private pvtRowIDX As Int32                   'Row counter used during AddRow method
    Private pvtCurrRow As Int32                  'Current grid row
    Private pvtPrevRow As Int32                  'Previous grid row
    Private pvtColorBac As Color                 'Background color
    Private pvtColorSel As Color                 'Row highlight color
    Private pvtRowHeight As Int32                'Row height
    Private pvtCellData(,) As String             'Cell data (optional)
    Private pvtRowNumber As Boolean = False      'Include row number in grid col zero

    Private pvtWells As Collection               'Well class collection for processing
    '
    '---Class Initialize/Terminate
    '
    Public Sub New()
        pvtWells = New Collection
    End Sub

    Protected Overrides Sub Finalize()
        pvtWells = Nothing
    End Sub
    '
    '============================== Properties ====================================
    '
    '
    '---Current row property (ie. user clicked)
    '
    Public ReadOnly Property CurrRow() As Int32
        Get
            CurrRow = pvtCurrRow
        End Get
    End Property
    '
    '---Total row count property
    '
    Public ReadOnly Property TotalRows() As Int32
        Get
            '            TotalRows = pvtRowIDX
            TotalRows = pvtFXG.RowCount

        End Get
    End Property
    '
    '---Cell data property
    '
    Public ReadOnly Property CellData(rowIN As Int32, colIN As Int32) As String
        Get
            CellData = pvtCellData(colIN, rowIN)
        End Get
    End Property
    '
    '---Return specified Cell text property
    '
    Public ReadOnly Property GetCellText(rowIN As Int32, colIN As Int32) As String
        Get
            GetCellText = pvtFXG(colIN, rowIN).Value

            'Dim SaveRow As Int32
            'Dim SaveCol As Int32

            'SaveRow = pvtFXG.CurrentCell.RowIndex
            'SaveCol = pvtFXG.CurrentCell.ColumnIndex


            'pvtFXG.CurrentCell = pvtFXG(colIN, rowIN)
            'GetCellText = pvtFXG.Text

            'pvtFXG.CurrentCell = pvtFXG(SaveCol, SaveRow)

        End Get
    End Property
    '
    '============================== Methods ====================================
    '

    '
    '--Format display grid (Headers and sizes)
    '
    Public Sub GridInit(ByVal NewFXG As DataGridView, countIN As Int32, bacColorIN As Color, _
                        selColorIN As Color, heightIN As Int32, _
                        Optional rowNumberIN As Boolean = True)

        pvtFXG = NewFXG
        pvtColCount = countIN
        pvtColorBac = bacColorIN
        pvtColorSel = selColorIN
        pvtRowHeight = heightIN
        pvtRowNumber = rowNumberIN

        pvtRowIDX = 0
        pvtCurrRow = 0
        pvtPrevRow = 0

        pvtFXG.Rows.Clear()
        pvtFXG.RowCount = 0

        pvtFXG.DefaultCellStyle.Font = New Font("Tahoma", 8, FontStyle.Regular)

        pvtFXG.ColumnCount = pvtColCount
        pvtFXG.ColumnHeadersVisible = True
        '        pvtFXG.ColumnHeadersHeight = 500
        '        Dim row As DataGridViewRow = pvtFXG.Rows(0)
        '       row.MinimumHeight = 50

        pvtFXG.RowHeadersVisible = True
        pvtFXG.RowHeadersWidth = 20
        pvtFXG.AllowDrop = True

        ReDim pvtCellData(pvtColCount, 1)

    End Sub
    '
    '--Format columns and column header
    '
    Public Sub ColHeader(colNBR As Int32, colWIDTH As Int32, colTITLE As String, _
                         colALIGN As DataGridViewContentAlignment)

        '        pvtFXG.Row = 0
        '        pvtFXG.Col = colNBR
        '        pvtFXG.colWIDTH(colNBR) = colWIDTH
        '        pvtFXG.Text = colTITLE
        '        pvtFXG.CellAlignment = colALIGN
        '        pvtFXG.ColAlignment(colNBR) = colALIGN

        pvtFXG.Columns(colNBR).Name = colTITLE
        pvtFXG.Columns(colNBR).Width = colWIDTH

        pvtFXG.Columns(colNBR).HeaderCell.Style.Alignment = colALIGN
        pvtFXG.Columns(colNBR).DefaultCellStyle.Alignment = colALIGN

        'Dim Style As New DataGridViewCellStyle
        'Style.Alignment = colALIGN
        'pvtFXG.Columns(colNBR).DefaultCellStyle = Style

    End Sub
    '
    '---Highlight current row
    '
    Public Sub HiLight()
        Dim ii As Int32
        Dim iRow As Int32

        iRow = pvtFXG.CurrentRow.Index
        For ii = pvtColCount - 1 To 1 Step -1
            '            pvtFXG.Col = ii
            '            pvtFXG.CellBackColor = pvtColorSel

            pvtFXG(ii, iRow).Style.BackColor = pvtColorSel
        Next ii

    End Sub
    '
    '---Unhighlight previous row
    '
    Public Sub UnHiLight()
        Dim ii As Int32

        pvtCurrRow = pvtFXG.CurrentRow.Index

        If pvtPrevRow > 0 Then
            '            pvtFXG.Row = pvtPrevRow

            For ii = pvtColCount - 1 To 1 Step -1
                '                pvtFXG.Col = ii
                '                pvtFXG.CellBackColor = pvtColorBac

                pvtFXG(ii, pvtPrevRow).Style.BackColor = pvtColorBac
            Next ii
            '            pvtFXG.Row = pvtCurrRow
        End If
        pvtPrevRow = pvtCurrRow

    End Sub
    '
    '---Remove current row
    '
    Public Sub DeleteRow()

        If IsNothing(pvtFXG.CurrentCell) Then Exit Sub

        '        pvtFXG.Rows(pvtFXG.CurrentCell.RowIndex).Selected = True

        For Each row As DataGridViewRow In pvtFXG.SelectedRows
            pvtFXG.Rows.Remove(row)
        Next
        pvtRowIDX = pvtFXG.RowCount

        If pvtRowIDX > 0 Then
            pvtCurrRow = pvtFXG.CurrentCell.RowIndex
            pvtPrevRow = pvtCurrRow
        Else
            pvtPrevRow = 0
        End If

        'If pvtRowIDX > 1 Then
        '    pvtFXG.RemoveItem pvtCurrRow
        '    pvtRowIDX = pvtRowIDX - 1
        '    pvtFXG.Rows = pvtRowIDX + 1
        '    pvtCurrRow = pvtFXG.Row
        '    pvtPrevRow = pvtCurrRow
        '    HiLight()
        'Else
        '    For ii = pvtColCount - 1 To 1 Step -1
        '        pvtFXG.Col = ii
        '        pvtFXG.Text = ""
        '    Next ii
        '    pvtRowIDX = pvtRowIDX - 1
        '    pvtPrevRow = 0
        'End If
    End Sub

    Public Sub DeleteAll()

        pvtFXG.Rows.Clear()

        pvtRowIDX = 0
        pvtCurrRow = 0
        pvtPrevRow = 0
    End Sub
    '
    '---Add cell picture
    '
    Public Sub AddCellPic(rowIN As Int32, colIN As Int32, picIN As PictureBox, _
                          cellDataIN As String)

        pvtFXG.Rows(rowIN).Cells(colIN).ValueType = GetType(Image)
        pvtFXG.Rows(rowIN).Cells(colIN).Value = picIN.Image

        'pvtFXG.Row = rowIN
        'pvtFXG.Col = colIN
        'pvtFXG.CellPictureAlignment = flexAlignCenterCenter
        'pvtFXG.CellPicture = picIN.Picture
        pvtCellData(colIN, rowIN) = cellDataIN
    End Sub
    '
    '---Make cell bold
    '
    Public Sub CellBold(rowIN As Int32, colIN As Int32)

        pvtFXG(colIN, rowIN).Style.Font = New Font("Tahoma", 8, FontStyle.Bold)
    End Sub
    '
    '---Make cell normal
    '
    Public Sub CellUnBold(rowIN As Int32, colIN As Int32)
        pvtFXG(colIN, rowIN).Style.Font = New Font("Tahoma", 8, FontStyle.Regular)
    End Sub
    '
    '---Add row to grid.  strIN must be formatted with tabs as column separators
    '
    Public Sub AddRow(strIN As String(), Optional heiIN As Int32 = 0)

        If pvtRowNumber Then strIN(0) = (pvtRowIDX + 1).ToString

        pvtFXG.Rows.Add(strIN)
        pvtRowIDX = pvtFXG.RowCount
        pvtFXG.AutoResizeRow(pvtRowIDX - 1)

        If heiIN > 0 Then
            Dim row As DataGridViewRow = pvtFXG.CurrentRow
            row.MinimumHeight = pvtRowHeight
        End If
        ReDim Preserve pvtCellData(pvtColCount, pvtRowIDX)


        'pvtRowIDX = pvtRowIDX + 1
        'If pvtRowNumber Then
        '    pvtFXG.AddItem(pvtRowIDX & strIN, pvtRowIDX)
        'Else
        '    pvtFXG.AddItem(strIN, pvtRowIDX)
        'End If
        'pvtFXG.RowData(pvtRowIDX) = rowdataIN

        'If heiIN <= 0 Then
        '    pvtFXG.RowHeight(pvtRowIDX) = pvtRowHeight
        'Else
        '    pvtFXG.RowHeight(pvtRowIDX) = heiIN
        'End If

        'pvtFXG.Rows = pvtRowIDX + 1
        'ReDim Preserve pvtCellData(pvtColCount, pvtRowIDX)
    End Sub
    '
    '---Return row (columns separated by tabs)
    '
    Public Function GetRow(rowIN As Int32) As String
        Dim strBUFF As String

        strBUFF = ""
        Dim row As DataGridViewRow = pvtFXG.Rows(rowIN)

        For Each cell As DataGridViewCell In row.Cells
            strBUFF = strBUFF & vbTab & cell.Value.ToString
        Next
        GetRow = strBUFF
    End Function
    '
    '---Return row height
    '
    Public Function GetRowHeight(rowIN As Int32) As Int32
        GetRowHeight = pvtFXG.Rows(rowIN).Height
    End Function
    '
    '---Set row height
    '
    Public Sub SetRowHeight(rowIN As Int32, heightIN As Int32)
        pvtFXG.Rows(rowIN).Height = heightIN
    End Sub
    '
    '---Insert message into display grid
    '
    Public Sub gridMSG(msgIN As String, colIN As Int32, colorIN As Color)
        Dim strBUFF As String() = New String() {" "}

        Call AddRow(strBUFF)
        pvtFXG(colIN, pvtRowIDX - 1).Value = msgIN
        pvtFXG(colIN, pvtRowIDX - 1).Style.ForeColor = colorIN
        CellBold(pvtRowIDX - 1, colIN)

        pvtFXG.AutoResizeRow(pvtRowIDX - 1)
    End Sub
    '
    '---Return cell
    '
    Public Function GetCell(rowIN As Int32, colIN As Int32) As String

        '        pvtFXG.Row = rowIN
        '        pvtFXG.Col = colIN
        Try
            GetCell = pvtFXG(colIN, rowIN).Value

        Catch ex As Exception
            GetCell = ""
        End Try
    End Function

End Class
