﻿'
'Name: P3
'
'Author: D.V. Allred (Analytics CIS, 01-Feb-2015)
'
'Description: Petroleum Properties Progam for MHA
'
'Database: none
'
'Assumptions: None
'
'Mod History
'
'    Vers  Who      When Mod
'  ------ ---- --------- -------------------------------------------------------------------
'    1.60  DVA 01-Jul-02 Production release (downloadable from MHA website)
'    2.00  DVA 15-Dec-14 Source lost: rewrite started
'    2.01  DVA 10-Apr-15 Alpha test for MHA
'    2.02  DVA 29-May-15 Beta test for MHA
'    2.03  DVA 03-Jun-15 Add Eclipse export
'    2.04  DVA 07-Jul-15 Add Sutton/Wichert/Aziz correlations
'    2.06  DVA 13-Jul-15 Rock Properties plus Compressiblity
'    2.08  DVA 03-Aug-15 - Fix Proceed button in Oil Input (prevent dup tab creation)
'                        - Add URL tooltip to all MHA logo's
'                        - Cosemetic updates
'                        - Update defaults, use 5.614586 in oil density calc
'    2.0.9 DVA 21-Sep-15 - Rock Properties updates:
'                          - new Lithology labels
'                          - Default Critical Gas should be 0.03
'                          - Correleations --> Correlations (sp)
'                          - Lithology: N/A where Corey selected
'                          - Citations form
'                          - Add rock properties: Water displacing Gas
'    2.1.0 DVA 02-Mar-16 - Hide current Rock Properties and set as "future" development
'                        - Release to MHA
'
Option Explicit On

Public Class frmStart
 
    Dim optPressure, optGas As Int32
    Dim optPBP As Int32
    Dim WTSOLIDS As Single
    '
    '---Build tab display according to phases present flags
    '
    Private Sub BuildTabs()
        Dim iTab As Int32
        '
        '---Clear current mapping
        '
        'For ii = Me.tabMain.TabCount - 1 To 3 Step -1
        '    Try
        '        Me.tabMain.TabPages.RemoveAt(ii)
        '    Catch ex As Exception

        '    End Try
        'Next ii

        Me.tabMain.TabPages.Remove(tpOilInput)
        Me.tabMain.TabPages.Remove(tpGasInput)
        Me.tabMain.TabPages.Remove(tpWaterInput)
        Me.tabMain.TabPages.Remove(tpFluidProp)
        iTab = 3

        Try
            '
            '---Phases present have six possible settings combinations (Oil, Oil/Gas, Oil/Gas/Water, Gas, Gas/Water, Water).
            '---If Oil is present, no other input is enabled until oil input is complete
            '
            If flagOil Then
                Me.tabMain.TabPages.Insert(iTab, tpOilInput)
                Exit Sub
            End If

            If flagGas Then
                Me.tabMain.TabPages.Insert(iTab, tpGasInput)
                iTab = iTab + 1
            End If

            If flagWater Then
                Me.tabMain.TabPages.Insert(iTab, tpWaterInput)
                iTab = iTab + 1
            End If
            '
            '---Add properties tab if any phases chosen!
            '
            If iTab > 3 Then Me.tabMain.TabPages.Insert(iTab, tpFluidProp)

        Catch ex As Exception
            ErrMSG(Err.Description, "Tab build error")
        End Try

    End Sub

    Private Sub frmStart_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Dim ii As Int32
        '
        '---Init P3
        '
        userCLICK = False
        strP3version = My.Application.Info.ProductName & " (V" & My.Application.Info.Version.Major & "." & My.Application.Info.Version.Minor & "." & My.Application.Info.Version.MinorRevision & ")"
        Me.Text = strP3version
        Me.mnuReport.Enabled = False
        Me.mnuCSV.Enabled = False
        Me.mnuEclipse.Enabled = False
        '
        '---Future dev (rock properties)
        '
        Me.tabMain.TabPages.Remove(tpRockInput)
        Me.tabMain.TabPages.Remove(tpRockProp)
        '
        '---Init phases page
        '
        flagOil = True
        Me.chkPhaseOil.Checked = True
        flagGas = False
        flagWater = False

        BuildTabs()
        Me.tabMain.SelectTab(0)

        STDPRS = 14.7
        Me.txtStdPres.Text = STDPRS

        STDTMP = 60.0
        Me.txtStdTemp.Text = STDTMP

        RESTEMP = 150.0
        Me.txtResTemp.Text = RESTEMP
        '
        '---Init Pressure input page
        '
        optPressure = 0
        Me.rdoSPR.Checked = True
        Me.rdoEDV.Checked = False

        MaxPrs = 5000.0
        Me.txtMaxPres.Text = MaxPrs

        IniPrs = 0.0
        Me.txtInitPres.Text = IniPrs

        MinPrs = 14.7
        Me.txtMinPres.Text = MinPrs

        For ii = 1 To 20
            PVTPRS(ii) = 0.0
            Dim ctrls() As Control = Me.Controls.Find("txtPV" & ii, True)

            If ctrls.Length > 0 Then ctrls(0).Text = PVTPRS(ii)
        Next ii
        '
        '---Gas input page
        '
        optGas = 0
        rdoGI.Checked = True
        rdoComp.Checked = False

        G = 0.7
        Me.txtGG.Text = G

        YN2 = 0.0
        Me.txtN.Text = YN2

        YCO2 = 0.0
        Me.txtCD.Text = YCO2

        YH2S = 0.0
        Me.txtHS.Text = YH2S

        flagPlus = flagGas
        idxPlus = -1

        PlusMolWt = 105.8
        Me.txtPlusMW.Text = PlusMolWt

        PlusSG = 0.7
        Me.txtPlusSG.Text = PlusSG
        '
        '---Init Water input page
        '
        WTSOLIDS = 1.0
        '
        '---Init Oil input page
        '
        API = 35.0
        Me.txtOilAPI.Text = API

        SGOILSC = 0.8498
        Me.txtOilSG.Text = SGOILSC

        PSEP = 50.0
        Me.txtSepP.Text = PSEP

        TSEP = 75.0
        Me.txtSepT.Text = TSEP

        GSEP = 0.9
        Me.txtSepGG.Text = GSEP

        RSI = 683.9
        Me.txtRsi.Text = RSI

        CalcRSITank()
        '
        '---Init Rock Input page
        '
        flagRock = True
        Me.rdoOilWater.Checked = True
        Me.rdoGasOil.Checked = False

        Me.rdoDrain.Checked = True
        Me.rdoImb.Checked = False

        Me.rdoSand.Checked = True
        Me.rdoLime.Checked = False

        Me.rdoWaterWet.Checked = True
        Me.rdoOilWet.Checked = False

        SWC = 0.2
        Me.txtConnate.Text = SWC

        SORW = 0.4
        Me.txtResOtoW.Text = SORW

        SORG = 0.4
        Me.txtResOtoG.Text = SORG

        SGCrit = 0.03
        Me.txtCritGas.Text = SGCrit

        CoreyK = 3.0
        Me.txtCoreyK.Text = CoreyK

        CoreyN = 4.0
        Me.txtCoreyN.Text = CoreyN

        HRPoro = 0.25
        Me.txtHonarpourPor.Text = HRPoro

        HRAir = 50.0
        Me.txtHonarpourPerm.Text = HRAir

        HRKrg = 0.5
        Me.txtHonarpourKrg.Text = HRKrg

        Me.rdoCompSand.Checked = True
        Me.rdoCompLime.Checked = False

        NewmanPor = 0.2
        Me.txtNewmanPor.Text = NewmanPor

        NewmanPres = 2000.0
        Me.txtNewmanPres.Text = NewmanPres
        '
        '---Init Fluid properties grid
        '
        gridOP.GridInit(Me.dgvOP, 10, bacCOLOR, selCOLOR, 30, False)
        gridOP.ColHeader(0, 85, "Pressure (psia)", aliRC)
        gridOP.ColHeader(1, 85, "Vol Fact (rb/stb)", aliRC)
        gridOP.ColHeader(2, 85, "GOR (scf/stb)", aliRC)
        gridOP.ColHeader(3, 85, "Viscosity (cp)", aliRC)
        gridOP.ColHeader(4, 85, "Density (lb/ft3)", aliRC)
        gridOP.ColHeader(5, 25, "", aliRC)
        gridOP.ColHeader(6, 85, "Vol Fact (rb/stb)", aliRC)
        gridOP.ColHeader(7, 85, "GOR (scf/stb)", aliRC)
        gridOP.ColHeader(8, 85, "Viscosity (cp)", aliRC)
        gridOP.ColHeader(9, 85, "Density (lb/ft3)", aliRC)

        For Each dgvCOL As DataGridViewColumn In Me.dgvOP.Columns
            dgvCOL.SortMode = DataGridViewColumnSortMode.NotSortable
        Next

        gridGP.GridInit(Me.dgvGP, 9, bacCOLOR, selCOLOR, 30, False)
        gridGP.ColHeader(0, 85, "Pressure (psia)", aliRC)
        gridGP.ColHeader(1, 85, "Pseudo Pres", aliRC)
        gridGP.ColHeader(2, 85, "Z Factor", aliRC)
        gridGP.ColHeader(3, 85, "P/Z (psia)", aliRC)
        gridGP.ColHeader(4, 85, "Vol Fact (rcf/scf)", aliRC)
        gridGP.ColHeader(5, 85, "Viscosity (cp)", aliRC)
        gridGP.ColHeader(6, 85, "Compressibility (psi-1)", aliRC)
        gridGP.ColHeader(7, 85, "Density (lb/rcf)", aliRC)
        gridGP.ColHeader(8, 85, "m(p)/1E06 (psia^2/cp)", aliRC)

        For Each dgvCOL As DataGridViewColumn In Me.dgvGP.Columns
            dgvCOL.SortMode = DataGridViewColumnSortMode.NotSortable
        Next
    
        gridWP.GridInit(Me.dgvWP, 6, bacCOLOR, selCOLOR, 30, False)
        gridWP.ColHeader(0, 95, "Pressure (psia)", aliRC)
        gridWP.ColHeader(1, 95, "Vol Fact (rb/stb)", aliRC)
        gridWP.ColHeader(2, 95, "GOR (scf/stb)", aliRC)
        gridWP.ColHeader(3, 95, "Viscosity (cp)", aliRC)
        gridWP.ColHeader(4, 95, "Compressibility (psi-1)", aliRC)
        gridWP.ColHeader(5, 95, "Density (lb/ft3)", aliRC)

        For Each dgvCOL As DataGridViewColumn In Me.dgvWP.Columns
            dgvCOL.SortMode = DataGridViewColumnSortMode.NotSortable
        Next

        userCLICK = True
    End Sub


    Private Sub LinkLabel1_LinkClicked(sender As Object, e As LinkLabelLinkClickedEventArgs) Handles LinkLabel1.LinkClicked
        Process.Start("http://www.mhausa.com/")
    End Sub

    Private Sub LinkLabel2_LinkClicked(sender As Object, e As LinkLabelLinkClickedEventArgs) Handles LinkLabel2.LinkClicked
        Process.Start("http://www.mhausa.com/index.php/resources/software")
    End Sub


    Private Sub ExitToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles ExitToolStripMenuItem.Click
        Me.Close()
    End Sub

    Private Sub LinkLabel3_LinkClicked(sender As Object, e As LinkLabelLinkClickedEventArgs) Handles LinkLabel3.LinkClicked
        Process.Start("http://www.mhausa.com/index.php/contact-form")
    End Sub

    Private Sub txtPbp_Enter(sender As Object, e As EventArgs) Handles txtPbp.Enter
        Me.cmdRsi.Visible = True
        Me.cmdPbp.Visible = False
    End Sub

    Private Sub txtRsi_Enter(sender As Object, e As EventArgs) Handles txtRsi.Enter
        Me.cmdRsi.Visible = False
        Me.cmdPbp.Visible = True
    End Sub

    Private Sub cmdPbp_Click(sender As Object, e As EventArgs) Handles cmdPbp.Click
        Dim X As Single

        Me.cmdGO.Enabled = True
        optPBP = 1
        Try
            X = ((RSI / GSEP) ^ 0.83) * (10 ^ (0.00091 * RESTEMP - 0.0125 * API))
            PBP = 18.2 * (X - 1.4)
            Me.txtPbp.Text = PBP.ToString("F1")

        Catch ex As Exception
            ErrMSG("Please check entry of Reservoir temp, Gas Gravity and API!", "RSI error")
        End Try
    End Sub

    Private Sub cmdRsi_Click(sender As Object, e As EventArgs) Handles cmdRsi.Click
        Dim X As Single

        Me.cmdGO.Enabled = True
        optPBP = 2
        Try
            X = (PBP / 18.2) + 1.4
            X = X / (10 ^ (0.00091 * RESTEMP - 0.0125 * API))
            RSI = X ^ (1.0 / 0.83) * GSEP
            Me.txtRsi.Text = RSI.ToString("F1")

        Catch ex As Exception
            ErrMSG("Please check entry of Reservoir temp, Gas Gravity and API!", "PBP error")
        End Try
    End Sub

    Private Sub PictureBox4_Click(sender As Object, e As EventArgs) Handles PictureBox4.Click
        Process.Start("http://www.mhausa.com/")
    End Sub

    Private Sub PictureBox1_Click(sender As Object, e As EventArgs) Handles PictureBox1.Click
        Process.Start("http://www.mhausa.com/")
    End Sub

    Private Sub PictureBox2_Click(sender As Object, e As EventArgs) Handles PictureBox2.Click
        Process.Start("http://www.mhausa.com/")
    End Sub

    Private Sub PictureBox3_Click(sender As Object, e As EventArgs) Handles PictureBox3.Click
        Process.Start("http://www.mhausa.com/")
    End Sub

    Private Sub PictureBox5_Click(sender As Object, e As EventArgs) Handles PictureBox5.Click
        Process.Start("http://www.mhausa.com/")
    End Sub

    Private Sub PictureBox6_Click(sender As Object, e As EventArgs) Handles PictureBox6.Click
        Process.Start("http://www.mhausa.com/")
    End Sub

    Private Sub PictureBox7_Click(sender As Object, e As EventArgs)
        Process.Start("http://www.mhausa.com/")
    End Sub

    Private Sub rdoGI_CheckedChanged(sender As Object, e As EventArgs) Handles rdoGI.CheckedChanged
        If Me.rdoGI.Checked Then
            Me.gbxGGI.Enabled = True
            Me.gbxComp.Enabled = False
            optGas = 0
            Me.gbxPlus.Visible = False
            Me.txtGG.Focus()
        End If
    End Sub

    Private Sub rdoComp_CheckedChanged(sender As Object, e As EventArgs) Handles rdoComp.CheckedChanged
        If rdoComp.Checked Then
            Me.gbxComp.Enabled = True
            Me.gbxGGI.Enabled = False
            optGas = 1
            flagNormalize = False
            Me.txtGC1.Focus()
        End If
    End Sub

    Private Sub rdoSPR_CheckedChanged(sender As Object, e As EventArgs) Handles rdoSPR.CheckedChanged
        If Me.rdoSPR.Checked Then
            Me.gbxRange.Enabled = True
            Me.gbxPV.Enabled = False
            Me.txtMaxPres.Focus()
            optPressure = 0
        End If
    End Sub

    Private Sub rdoEDV_CheckedChanged(sender As Object, e As EventArgs) Handles rdoEDV.CheckedChanged
        If Me.rdoEDV.Checked Then
            Me.gbxRange.Enabled = False
            Me.gbxPV.Enabled = True
            Me.txtPV1.Focus()
            optPressure = 1
        End If
    End Sub

    Private Sub cbxGC1_CheckedChanged_1(sender As Object, e As EventArgs) Handles cbxGC1.CheckedChanged
        SetPlus(Me.cbxGC1)
    End Sub
    '
    '---Process "Plus Fractional" gas comp entry
    '
    Private Sub SetPlus(cbxIN As CheckBox)
        Dim obj As Control
        Dim cbxID As Int32
        '
        '---Force only one checkbox to be set and enable "plus fraction" data entry panel
        '
        cbxID = 100
        For Each obj In Me.gbxComp.Controls
            If obj.Name.StartsWith("cbxGC") Then
                If cbxIN.Checked Then
                    cbxID = cbxIN.Name.Substring(5)
                    Me.gbxPlus.Visible = True
                    flagPlus = True
                    idxPlus = cbxID

                    If obj.Name <> cbxIN.Name Then DirectCast(obj, CheckBox).Checked = False
                Else
                    Me.gbxPlus.Visible = False                                                  'No checkbox's set
                    flagPlus = False
                    idxPlus = -1
                End If
            End If
        Next
        '
        '---Disable gas comp entry boxes below specified checkbox
        '
        For Each obj In Me.gbxComp.Controls
            If obj.Name.StartsWith("txtGC") Then
                obj.Enabled = True
                If obj.Name.Substring(5) > cbxID Then
                    obj.Text = "0.0000"
                    obj.Enabled = False
                End If
            End If
        Next
        CalcSumGC()

    End Sub
    '
    '---Normalize Gas Comp entry values to 1.0
    '
    Private Sub cmdNormal_Click(sender As Object, e As EventArgs) Handles cmdNormal.Click
        Dim nSum As Single
        Dim nTemp As Single

        nSum = Me.txtSum.Text

        For Each obj As Control In Me.gbxComp.Controls
            If obj.Name.StartsWith("txtGC") Then
                If obj.Text > 0 Then
                    nTemp = obj.Text / nSum
                    obj.Text = Format(nTemp / 1.0, "0.0000")
                End If
            End If
        Next

        Me.txtNGC.Text = (Me.txtNGC.Text / nSum).ToString("F4")
        Me.txtDX.Text = (Me.txtDX.Text / nSum).ToString("F4")
        Me.txtSU.Text = (Me.txtSU.Text / nSum).ToString("F4")

        CalcSumGC()
        flagNormalize = True

    End Sub

    Private Sub CalcSumGC()
        Dim nSum As Single

        Try
            nSum = 0

            For Each obj As Control In Me.gbxComp.Controls
                If obj.Name.StartsWith("txtGC") Then
                    nSum = nSum + obj.Text
                End If
            Next
            '
            '---Add non-hydrocarbon items
            '
            nSum = nSum + Me.txtNGC.Text + Me.txtDX.Text + Me.txtSU.Text

            Me.txtSum.Text = nSum.ToString("F4")

        Catch ex As Exception

        End Try
    End Sub

    Private Sub cbxGC2_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC2.CheckedChanged
        SetPlus(Me.cbxGC2)
    End Sub

    Private Sub cbxGC3_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC3.CheckedChanged
        SetPlus(Me.cbxGC3)
    End Sub

    Private Sub cbxGC4_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC4.CheckedChanged
        SetPlus(Me.cbxGC4)
    End Sub

    Private Sub cbxGC5_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC5.CheckedChanged
        SetPlus(Me.cbxGC5)
    End Sub

    Private Sub cbxGC6_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC6.CheckedChanged
        SetPlus(Me.cbxGC6)
    End Sub

    Private Sub cbxGC7_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC7.CheckedChanged
        SetPlus(Me.cbxGC7)
    End Sub

    Private Sub cbxGC8_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC8.CheckedChanged
        SetPlus(Me.cbxGC8)
    End Sub

    Private Sub cbxGC9_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC9.CheckedChanged
        SetPlus(Me.cbxGC9)
    End Sub

    Private Sub cbxGC10_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC10.CheckedChanged
        SetPlus(Me.cbxGC10)
    End Sub

    Private Sub cbxGC11_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC11.CheckedChanged
        SetPlus(Me.cbxGC11)
    End Sub

    Private Sub cbxGC12_CheckedChanged(sender As Object, e As EventArgs) Handles cbxGC12.CheckedChanged
        SetPlus(Me.cbxGC12)
    End Sub

    Private Sub txtGC1_Enter(sender As Object, e As EventArgs) Handles txtGC1.Enter
        Me.txtGC1.SelectAll()
    End Sub

    Private Sub txtGC1_TextChanged(sender As Object, e As EventArgs) Handles txtGC1.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC2_TextChanged(sender As Object, e As EventArgs) Handles txtGC2.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC3_TextChanged(sender As Object, e As EventArgs) Handles txtGC3.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC4_TextChanged(sender As Object, e As EventArgs) Handles txtGC4.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC5_TextChanged(sender As Object, e As EventArgs) Handles txtGC5.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC6_TextChanged(sender As Object, e As EventArgs) Handles txtGC6.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC7_TextChanged(sender As Object, e As EventArgs) Handles txtGC7.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC8_TextChanged(sender As Object, e As EventArgs) Handles txtGC8.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC9_TextChanged(sender As Object, e As EventArgs) Handles txtGC9.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC10_TextChanged(sender As Object, e As EventArgs) Handles txtGC10.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC11_TextChanged(sender As Object, e As EventArgs) Handles txtGC11.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtGC12_TextChanged(sender As Object, e As EventArgs) Handles txtGC12.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtNGC_TextChanged(sender As Object, e As EventArgs) Handles txtNGC.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtDX_TextChanged(sender As Object, e As EventArgs) Handles txtDX.TextChanged
        CalcSumGC()
    End Sub

    Private Sub txtSU_TextChanged(sender As Object, e As EventArgs) Handles txtSU.TextChanged
        CalcSumGC()
    End Sub
    '
    '---Check if user input is numeric and optionally in range
    '
    Private Function ChkNumeric(txtIN As TextBox, e As System.ComponentModel.CancelEventArgs, Optional minIN As Single = -1, Optional maxIN As Single = -1) As Boolean
        Dim nTemp As Single

        ChkNumeric = True

        If Not IsNumeric(txtIN.Text) Then
            ErrMSG("Please enter a numeric value.", "Invalid Number")
            e.Cancel = True
            ChkNumeric = False
            Exit Function
        End If
        '
        '---Check if in range
        '
        If minIN <> maxIN Then
            nTemp = txtIN.Text
            If nTemp < minIN Or nTemp > maxIN Then
                ErrMSG("Please enter a value between " & minIN & " and " & maxIN & ".", "Range Error")
                e.Cancel = True
                ChkNumeric = False
                Exit Function
            End If
        End If

    End Function

    Private Sub txtGC1_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC1.Validating
        ChkNumeric(Me.txtGC1, e)
    End Sub

    Private Sub txtGC2_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC2.Validating
        ChkNumeric(Me.txtGC2, e)
    End Sub

    Private Sub txtGC3_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC3.Validating
        ChkNumeric(Me.txtGC3, e)
    End Sub

    Private Sub txtGC4_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC4.Validating
        ChkNumeric(Me.txtGC4, e)
    End Sub

    Private Sub txtGC5_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC5.Validating
        ChkNumeric(Me.txtGC5, e)
    End Sub

    Private Sub txtGC6_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC6.Validating
        ChkNumeric(Me.txtGC6, e)
    End Sub

    Private Sub txtGC7_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC7.Validating
        ChkNumeric(Me.txtGC7, e)
    End Sub

    Private Sub txtGC8_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC8.Validating
        ChkNumeric(Me.txtGC8, e)
    End Sub

    Private Sub txtGC9_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC9.Validating
        ChkNumeric(Me.txtGC9, e)
    End Sub

    Private Sub txtGC10_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC10.Validating
        ChkNumeric(Me.txtGC10, e)
    End Sub

    Private Sub txtGC11_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC11.Validating
        ChkNumeric(Me.txtGC11, e)
    End Sub

    Private Sub txtGC12_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGC12.Validating
        ChkNumeric(Me.txtGC12, e)
    End Sub

    Private Sub txtNGC_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtNGC.Validating
        ChkNumeric(Me.txtNGC, e)
    End Sub

    Private Sub txtDX_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtDX.Validating
        ChkNumeric(Me.txtDX, e)
    End Sub

    Private Sub txtSU_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtSU.Validating
        ChkNumeric(Me.txtSU, e)
    End Sub

    Private Sub txtGG_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtGG.Validating
        ' 0.45 Or SPG > 2.0
        If ChkNumeric(Me.txtGG, e, 0.45, 2.0) Then G = CSng(Me.txtGG.Text)
    End Sub

    Private Sub txtN_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtN.Validating
        If ChkNumeric(Me.txtN, e) Then YN2 = CSng(Me.txtN.Text)
    End Sub

    Private Sub txtCD_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtCD.Validating
        If ChkNumeric(Me.txtCD, e) Then YCO2 = CSng(Me.txtCD.Text)
    End Sub

    Private Sub txtHS_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtHS.Validating
        If ChkNumeric(Me.txtHS, e) Then YH2S = CSng(Me.txtHS.Text)
    End Sub

    Private Sub txtPlusMW_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPlusMW.Validating
        If ChkNumeric(Me.txtPlusMW, e) Then PlusMolWt = CSng(Me.txtPlusMW.Text)
    End Sub

    Private Sub txtPlusSG_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPlusSG.Validating
        If ChkNumeric(Me.txtPlusSG, e) Then PlusSG = CSng(Me.txtPlusSG.Text)
    End Sub

    Private Sub txtStdPres_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtStdPres.Validating
        If ChkNumeric(Me.txtStdPres, e) Then STDPRS = CSng(Me.txtStdPres.Text)
    End Sub

    Private Sub txtStdTemp_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtStdTemp.Validating
        If ChkNumeric(Me.txtStdTemp, e) Then STDTMP = CSng(Me.txtStdTemp.Text)
    End Sub

    Private Sub txtResTemp_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtResTemp.Validating
        If ChkNumeric(Me.txtResTemp, e) Then RESTEMP = CSng(Me.txtResTemp.Text)
    End Sub

    Private Sub txtMaxPres_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtMaxPres.Validating
        If ChkNumeric(Me.txtMaxPres, e) Then MaxPrs = CSng(Me.txtMaxPres.Text)
    End Sub

    Private Sub txtMinPres_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtMinPres.Validating
        If ChkNumeric(Me.txtMinPres, e) Then MinPrs = CSng(Me.txtMinPres.Text)
    End Sub

    Private Sub txtInitPres_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtInitPres.Validating
        If ChkNumeric(Me.txtInitPres, e) Then IniPrs = CSng(Me.txtInitPres.Text)
    End Sub

    Private Sub txtPV1_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV1.Validating
        If ChkNumeric(Me.txtPV1, e) Then PVTPRS(1) = CSng(Me.txtPV1.Text)
    End Sub

    Private Sub txtPV2_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV2.Validating
        If ChkNumeric(Me.txtPV2, e) Then PVTPRS(2) = CSng(Me.txtPV2.Text)
    End Sub

    Private Sub txtPV3_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV3.Validating
        If ChkNumeric(Me.txtPV3, e) Then PVTPRS(3) = CSng(Me.txtPV3.Text)
    End Sub

    Private Sub txtPV4_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV4.Validating
        If ChkNumeric(Me.txtPV4, e) Then PVTPRS(4) = CSng(Me.txtPV4.Text)
    End Sub

    Private Sub txtPV5_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV5.Validating
        If ChkNumeric(Me.txtPV5, e) Then PVTPRS(5) = CSng(Me.txtPV5.Text)
    End Sub

    Private Sub txtPV6_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV6.Validating
        If ChkNumeric(Me.txtPV6, e) Then PVTPRS(6) = CSng(Me.txtPV6.Text)
    End Sub

    Private Sub txtPV7_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV7.Validating
        If ChkNumeric(Me.txtPV7, e) Then PVTPRS(7) = CSng(Me.txtPV7.Text)
    End Sub

    Private Sub txtPV8_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV8.Validating
        If ChkNumeric(Me.txtPV8, e) Then PVTPRS(8) = CSng(Me.txtPV8.Text)
    End Sub

    Private Sub txtPV9_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV9.Validating
        If ChkNumeric(Me.txtPV9, e) Then PVTPRS(9) = CSng(Me.txtPV9.Text)
    End Sub

    Private Sub txtPV10_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV10.Validating
        If ChkNumeric(Me.txtPV10, e) Then PVTPRS(10) = CSng(Me.txtPV10.Text)
    End Sub

    Private Sub txtPV11_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV11.Validating
        If ChkNumeric(Me.txtPV11, e) Then PVTPRS(11) = CSng(Me.txtPV11.Text)
    End Sub

    Private Sub txtPV12_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV12.Validating
        If ChkNumeric(Me.txtPV12, e) Then PVTPRS(12) = CSng(Me.txtPV12.Text)
    End Sub

    Private Sub txtPV13_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV13.Validating
        If ChkNumeric(Me.txtPV13, e) Then PVTPRS(13) = CSng(Me.txtPV13.Text)
    End Sub

    Private Sub txtPV14_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV14.Validating
        If ChkNumeric(Me.txtPV14, e) Then PVTPRS(14) = CSng(Me.txtPV14.Text)
    End Sub

    Private Sub txtPV15_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV15.Validating
        If ChkNumeric(Me.txtPV15, e) Then PVTPRS(15) = CSng(Me.txtPV15.Text)
    End Sub

    Private Sub txtPV16_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV16.Validating
        If ChkNumeric(Me.txtPV16, e) Then PVTPRS(16) = CSng(Me.txtPV16.Text)
    End Sub

    Private Sub txtPV17_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV17.Validating
        If ChkNumeric(Me.txtPV17, e) Then PVTPRS(17) = CSng(Me.txtPV17.Text)
    End Sub

    Private Sub txtPV18_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPV18.Validating
        If ChkNumeric(Me.txtPV18, e) Then PVTPRS(18) = CSng(Me.txtPV18.Text)
    End Sub

    Private Sub txtOilAPI_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtOilAPI.Validating
        If ChkNumeric(Me.txtOilAPI, e) Then
            API = CSng(Me.txtOilAPI.Text)
            SGOILSC = 141.5 / (131.5 + API)
            CalcRSITank()
        End If
    End Sub

    Private Sub txtOilSG_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtOilSG.Validating
        If ChkNumeric(Me.txtOilSG, e) Then
            SGOILSC = CSng(Me.txtOilSG.Text)
            API = (141.5 / SGOILSC) - 131.5
            CalcRSITank()
        End If
    End Sub

    Private Sub txtSepGG_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtSepGG.Validating
        If ChkNumeric(Me.txtSepGG, e) Then
            GSEP = CSng(Me.txtSepGG.Text)
            CalcRSITank()
        End If
    End Sub

    Private Sub txtRsi_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtRsi.Validating
        If ChkNumeric(Me.txtRsi, e) Then RSI = CSng(Me.txtRsi.Text)
    End Sub

    Private Sub txtPbp_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtPbp.Validating
        If ChkNumeric(Me.txtPbp, e) Then PBP = CSng(Me.txtPbp.Text)
    End Sub

    Private Sub txtWSppm_Enter(sender As Object, e As EventArgs) Handles txtWSppm.Enter
        Me.txtWSppm.SelectAll()
    End Sub

    Private Sub txtWSppm_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtWSppm.Validating
        If ChkNumeric(Me.txtWSppm, e) Then WTSOLIDS = CSng(Me.txtWSppm.Text) / 10000
    End Sub

    Private Sub txtWSwt_Enter(sender As Object, e As EventArgs) Handles txtWSwt.Enter
        Me.txtWSwt.SelectAll()
    End Sub

    Private Sub txtWSwt_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtWSwt.Validating
        If ChkNumeric(Me.txtWSwt, e) Then WTSOLIDS = CSng(Me.txtWSwt.Text)
    End Sub

    Private Sub CalcRSITank()
        Try
            SGOILSC = 141.5 / (131.5 + API)
            RSITANK = 0.3818 - 5.506 * Log10(SGOILSC) + 2.902 * Log10(GSEP) + 1.327 * Log10(PSEP) - 0.7355 * Log10(TSEP)
            RSITANK = 10 ^ RSITANK

            Me.txtRSITANK.Text = RSITANK.ToString("F1")

        Catch ex As Exception
            Me.txtRSITANK.Text = ""

        End Try
    End Sub

    Private Sub cmdGO_Click(sender As Object, e As EventArgs) Handles cmdGO.Click
        Dim iTab As Int32
        '
        '---Proceed with gas/water input.  Make Gas Input, Water Input and Fluid Properties tabs available
        '
        iTab = 4

        If Not Me.tabMain.TabPages.Contains(tpGasInput) Then
            Me.tabMain.TabPages.Insert(iTab, tpGasInput)
        End If

        iTab = iTab + 1
        If flagWater And Not Me.tabMain.TabPages.Contains(tpWaterInput) Then
            Me.tabMain.TabPages.Insert(iTab, tpWaterInput)
            iTab = iTab + 1
        End If

        If Not Me.tabMain.TabPages.Contains(tpFluidProp) Then
            Me.tabMain.TabPages.Insert(iTab, tpFluidProp)
        End If


    End Sub

    Private Sub AboutToolStripMenuItem_Click(sender As Object, e As EventArgs) Handles AboutToolStripMenuItem.Click
        frmAbout.ShowDialog()
    End Sub

    Private Sub chkPhaseOil_CheckedChanged(sender As Object, e As EventArgs) Handles chkPhaseOil.CheckedChanged
        If Not userCLICK Then Exit Sub

        If Me.chkPhaseOil.Checked Then
            flagOil = True
        Else
            flagOil = False
        End If
        BuildTabs()
    End Sub

    Private Sub chkPhaseGas_CheckedChanged(sender As Object, e As EventArgs) Handles chkPhaseGas.CheckedChanged
        If Not userCLICK Then Exit Sub

        If Me.chkPhaseGas.Checked Then
            flagGas = True
        Else
            flagGas = False
        End If
        BuildTabs()
    End Sub

    Private Sub chkPhaseH2O_CheckedChanged(sender As Object, e As EventArgs) Handles chkPhaseH2O.CheckedChanged
        If Not userCLICK Then Exit Sub

        If Me.chkPhaseH2O.Checked Then
            flagWater = True
        Else
            flagWater = False
        End If
        BuildTabs()
    End Sub

    Private Sub tabMain_Click(sender As Object, e As EventArgs) Handles tabMain.Click
        Dim curTAB As String
        Dim iTab As Int32

        curTAB = Me.tabMain.SelectedTab.Tag
        If curTAB = "FLUIDPROP" Then
            '
            '---Enable fluid properties tabs according to phases
            '
            Me.tabFluid.TabPages.Remove(tpGasProp)
            Me.tabFluid.TabPages.Remove(tpOilProp)
            Me.tabFluid.TabPages.Remove(tpWaterProp)
            iTab = 0

            If flagOil Then
                Me.tabFluid.TabPages.Insert(iTab, tpOilProp)
                iTab = iTab + 1
            End If

            If flagGas Or flagOil Then
                Me.tabFluid.TabPages.Insert(iTab, tpGasProp)
                iTab = iTab + 1
            End If

            If flagWater Then
                Me.tabFluid.TabPages.Insert(iTab, tpWaterProp)
            End If
            '
            '---Calc fluid properties
            '
            If flagOil Or flagGas Then CalcGas(optPressure, optGas, flagOil)
            If flagOil Then CalcOil(optPBP)
            If flagWater Then CalcWater(optPressure, WTSOLIDS)

            Me.mnuReport.Enabled = True
            Me.mnuCSV.Enabled = True
            Me.mnuEclipse.Enabled = True
        End If

        If curTAB = "ROCKPROP" Then
            CalcRock()
            Me.mnuReport.Enabled = True
            Me.mnuCSV.Enabled = True
            Me.mnuEclipse.Enabled = True
        End If


    End Sub

    Private Sub InitRockForm()
        Me.txtResOtoW.Enabled = False
        Me.lblResOtoW.Enabled = False
        Me.txtResOtoG.Enabled = False
        Me.lblResOtoG.Enabled = False
        Me.txtCritGas.Enabled = False
        Me.lblCritGas.Enabled = False

        Me.rdoWaterWet.Enabled = False
        Me.rdoOilWet.Enabled = False

        Me.rdoCorey.Enabled = False
        Me.rdoHonarpour.Enabled = False

        Me.txtCoreyK.Enabled = False
        Me.lblCoreyK.Enabled = False
        Me.txtCoreyN.Enabled = False
        Me.lblCoreyN.Enabled = False

        If optRockFluids = 0 Then
            Me.rdoWaterWet.Enabled = True
            Me.rdoOilWet.Enabled = True

            If optRockDisplace = 1 Then
                Me.txtResOtoW.Enabled = True
                Me.lblResOtoW.Enabled = True
            End If
        End If

        If optRockFluids = 1 Then
            Me.txtCritGas.Enabled = True
            Me.lblCritGas.Enabled = True

            If optRockDisplace = 1 Then
                Me.txtResOtoG.Enabled = True
                Me.lblResOtoG.Enabled = True
            End If
        End If

        If optRockDisplace = 0 Then
            Me.rdoCorey.Enabled = True
            Me.rdoCorey.Checked = True

            '            Me.optCorr(0).Enabled = True
            '            Me.optCorr(0).Value = True
        End If

        If optRockDisplace = 1 Then
            Me.rdoCorey.Enabled = True
            Me.rdoHonarpour.Enabled = True

            '            Me.optCorr(0).Enabled = True
            '            Me.optCorr(1).Enabled = True
        End If

        If optRockCorr = 0 Then
            Me.txtCoreyK.Enabled = True
            Me.lblCoreyK.Enabled = True
            Me.txtCoreyN.Enabled = True
            Me.lblCoreyN.Enabled = True
        End If
    End Sub

    Private Sub SetCoreyExp()
        Me.txtCoreyK.Enabled = False
        Me.lblCoreyK.Enabled = False
        Me.txtCoreyN.Enabled = False
        Me.lblCoreyN.Enabled = False

        Me.txtHonarpourPerm.Enabled = False
        Me.lblHonarpourPerm.Enabled = False
        Me.txtHonarpourPor.Enabled = False
        Me.lblHonarpourPor.Enabled = False
        Me.txtHonarpourKrg.Enabled = False
        Me.lblHonarpourKrg.Enabled = False
        Me.lblMD.Enabled = False
        Me.lblSORG.Enabled = False

        If optRockCorr = 0 Then
            Me.txtCoreyK.Enabled = True
            Me.lblCoreyK.Enabled = True
            Me.txtCoreyN.Enabled = True
            Me.lblCoreyN.Enabled = True
        End If

        If optRockCorr = 1 Then
            Me.txtHonarpourPerm.Enabled = True
            Me.lblHonarpourPerm.Enabled = True
            Me.txtHonarpourPor.Enabled = True
            Me.lblHonarpourPor.Enabled = True
            Me.txtHonarpourKrg.Enabled = True
            Me.lblHonarpourKrg.Enabled = True
            Me.lblMD.Enabled = True
            Me.lblSORG.Enabled = True
        End If
    End Sub


    Private Sub tabMain_TabIndexChanged(sender As Object, e As EventArgs) Handles tabMain.TabIndexChanged

    End Sub

    Private Sub txtPV1_TextChanged(sender As Object, e As EventArgs) Handles txtPV1.TextChanged

    End Sub

    Private Sub txtGG_TextChanged(sender As Object, e As EventArgs) Handles txtGG.TextChanged

    End Sub

    Private Sub txtHS_TextChanged(sender As Object, e As EventArgs) Handles txtHS.TextChanged

    End Sub

    Private Sub txtMaxPres_TextChanged(sender As Object, e As EventArgs) Handles txtMaxPres.TextChanged

    End Sub

    Private Sub txtInitPres_TextChanged(sender As Object, e As EventArgs) Handles txtInitPres.TextChanged

    End Sub

    Private Sub txtMinPres_TextChanged(sender As Object, e As EventArgs) Handles txtMinPres.TextChanged

    End Sub

    Private Sub txtStdTemp_TextChanged(sender As Object, e As EventArgs) Handles txtStdTemp.TextChanged

    End Sub

    Private Sub txtPlusMW_TextChanged(sender As Object, e As EventArgs) Handles txtPlusMW.TextChanged

    End Sub

    Private Sub mnuReport_Click(sender As Object, e As EventArgs) Handles mnuReport.Click
        frmReport.ShowDialog()
    End Sub

    Private Sub mnuCSV_Click(sender As Object, e As EventArgs) Handles mnuCSV.Click
        frmCSV.ShowDialog()
    End Sub

  
    Private Sub mnuEclipse_Click(sender As Object, e As EventArgs) Handles mnuEclipse.Click
        frmECL.ShowDialog()
    End Sub

    Private Sub Label100_Click(sender As Object, e As EventArgs) Handles Label100.Click

    End Sub

    Private Sub txtWSppm_LostFocus(sender As Object, e As EventArgs) Handles txtWSppm.LostFocus
        Me.txtWSwt.Text = Me.txtWSppm.Text / 10000
    End Sub

    Private Sub txtWSwt_LostFocus(sender As Object, e As EventArgs) Handles txtWSwt.LostFocus
        Me.txtWSppm.Text = Me.txtWSwt.Text * 10000
    End Sub

    Private Sub txtWSppm_TextChanged(sender As Object, e As EventArgs) Handles txtWSppm.TextChanged

    End Sub


    Private Sub txtSepP_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtSepP.Validating
        If ChkNumeric(Me.txtSepP, e) Then
            PSEP = CSng(Me.txtSepP.Text)
            CalcRSITank()
        End If
    End Sub

    Private Sub txtSepT_TextChanged(sender As Object, e As EventArgs) Handles txtSepT.TextChanged

    End Sub

    Private Sub txtSepT_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtSepT.Validating
        If ChkNumeric(Me.txtSepT, e) Then
            TSEP = CSng(Me.txtSepT.Text)
            CalcRSITank()
        End If
    End Sub

    Private Sub txtOilAPI_TextChanged(sender As Object, e As EventArgs) Handles txtOilAPI.TextChanged

    End Sub

    Private Sub txtRsi_TextChanged(sender As Object, e As EventArgs) Handles txtRsi.TextChanged

    End Sub

    Private Sub txtPbp_TextChanged(sender As Object, e As EventArgs) Handles txtPbp.TextChanged

    End Sub

    Private Sub txtOilSG_TextChanged(sender As Object, e As EventArgs) Handles txtOilSG.TextChanged

    End Sub

    Private Sub txtSepGG_TextChanged(sender As Object, e As EventArgs) Handles txtSepGG.TextChanged

    End Sub

    Private Sub txtSepP_TextChanged(sender As Object, e As EventArgs) Handles txtSepP.TextChanged

    End Sub

    Private Sub txtStdPres_TextChanged(sender As Object, e As EventArgs) Handles txtStdPres.TextChanged

    End Sub

    Private Sub txtConnate_TextChanged(sender As Object, e As EventArgs) Handles txtConnate.TextChanged

    End Sub

    Private Sub txtConnate_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtConnate.Validating
        If ChkNumeric(Me.txtConnate, e) Then SWC = CSng(Me.txtConnate.Text)
    End Sub

    Private Sub txtResOtoW_TextChanged(sender As Object, e As EventArgs) Handles txtResOtoW.TextChanged

    End Sub

    Private Sub txtResOtoW_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtResOtoW.Validating
        If ChkNumeric(Me.txtResOtoW, e) Then SORW = CSng(Me.txtResOtoW.Text)
    End Sub

    Private Sub txtResOtoG_TextChanged(sender As Object, e As EventArgs) Handles txtResOtoG.TextChanged

    End Sub

    Private Sub txtResOtoG_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtResOtoG.Validating
        If ChkNumeric(Me.txtResOtoG, e) Then SORG = CSng(Me.txtResOtoG.Text)
    End Sub

    Private Sub txtCritGas_TextChanged(sender As Object, e As EventArgs) Handles txtCritGas.TextChanged

    End Sub

    Private Sub txtCritGas_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtCritGas.Validating
        If ChkNumeric(Me.txtCritGas, e) Then SGCrit = CSng(Me.txtCritGas.Text)
    End Sub

    Private Sub txtCoreyK_TextChanged(sender As Object, e As EventArgs) Handles txtCoreyK.TextChanged

    End Sub

    Private Sub txtCoreyK_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtCoreyK.Validating
        If ChkNumeric(Me.txtCoreyK, e) Then CoreyK = CSng(Me.txtCoreyK.Text)
    End Sub

    Private Sub txtCoreyN_TextChanged(sender As Object, e As EventArgs) Handles txtCoreyN.TextChanged

    End Sub

    Private Sub txtCoreyN_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtCoreyN.Validating
        If ChkNumeric(Me.txtCoreyN, e) Then CoreyN = CSng(Me.txtCoreyN.Text)
    End Sub

    Private Sub txtHonarpourPor_TextChanged(sender As Object, e As EventArgs) Handles txtHonarpourPor.TextChanged

    End Sub

    Private Sub txtHonarpourPor_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtHonarpourPor.Validating
        If ChkNumeric(Me.txtHonarpourPor, e) Then HRPoro = CSng(Me.txtHonarpourPor.Text)
    End Sub

    Private Sub txtHonarpourPerm_TextChanged(sender As Object, e As EventArgs) Handles txtHonarpourPerm.TextChanged

    End Sub

    Private Sub txtHonarpourPerm_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtHonarpourPerm.Validating
        If ChkNumeric(Me.txtHonarpourPerm, e) Then HRAir = CSng(Me.txtHonarpourPerm.Text)
    End Sub

    Private Sub txtHonarpourKrg_TextChanged(sender As Object, e As EventArgs) Handles txtHonarpourKrg.TextChanged

    End Sub

    Private Sub txtHonarpourKrg_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtHonarpourKrg.Validating
        If ChkNumeric(Me.txtHonarpourKrg, e) Then HRKrg = CSng(Me.txtHonarpourKrg.Text)
    End Sub

    Private Sub rdoOilWater_CheckedChanged(sender As Object, e As EventArgs) Handles rdoOilWater.CheckedChanged
        If Me.rdoOilWater.Checked Then
            optRockFluids = 0
            InitRockForm()
        End If
    End Sub

    Private Sub rdoGasOil_CheckedChanged(sender As Object, e As EventArgs) Handles rdoGasOil.CheckedChanged
        If Me.rdoGasOil.Checked Then
            optRockFluids = 1
            InitRockForm()
        End If
    End Sub

    Private Sub rdoDrain_CheckedChanged(sender As Object, e As EventArgs) Handles rdoDrain.CheckedChanged
        If Me.rdoDrain.Checked Then
            optRockDisplace = 0
            InitRockForm()
        End If
    End Sub

    Private Sub rdoImb_CheckedChanged(sender As Object, e As EventArgs) Handles rdoImb.CheckedChanged
        If Me.rdoImb.Checked Then
            optRockDisplace = 1
            InitRockForm()
        End If
    End Sub

    Private Sub rdoSand_CheckedChanged(sender As Object, e As EventArgs) Handles rdoSand.CheckedChanged
        If Me.rdoSand.Checked Then
            optRockLith = 0
            Me.rdoCompSand.Checked = True
        End If
    End Sub

    Private Sub rdoLime_CheckedChanged(sender As Object, e As EventArgs) Handles rdoLime.CheckedChanged
        If Me.rdoLime.Checked Then
            optRockLith = 1
            Me.rdoCompLime.Checked = True
        End If
    End Sub

    Private Sub rdoWaterWet_CheckedChanged(sender As Object, e As EventArgs) Handles rdoWaterWet.CheckedChanged
        If Me.rdoWaterWet.Checked Then
            optRockWet = 0
        End If
    End Sub

    Private Sub rdoOilWet_CheckedChanged(sender As Object, e As EventArgs) Handles rdoOilWet.CheckedChanged
        If Me.rdoOilWet.Checked Then
            optRockWet = 1
        End If
    End Sub

    Private Sub rdoCorey_CheckedChanged(sender As Object, e As EventArgs) Handles rdoCorey.CheckedChanged
        If Me.rdoCorey.Checked Then
            optRockCorr = 0
            SetCoreyExp()
        End If
    End Sub

    Private Sub rdoHonarpour_CheckedChanged(sender As Object, e As EventArgs) Handles rdoHonarpour.CheckedChanged
        If Me.rdoHonarpour.Checked Then
            optRockCorr = 1
            SetCoreyExp()
        End If
    End Sub

    Private Sub txtNewmanPor_TextChanged(sender As Object, e As EventArgs) Handles txtNewmanPor.TextChanged

    End Sub

    Private Sub txtNewmanPor_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtNewmanPor.Validating
        If ChkNumeric(Me.txtNewmanPor, e) Then NewmanPor = CSng(Me.txtNewmanPor.Text)
    End Sub

    Private Sub txtNewmanPres_TextChanged(sender As Object, e As EventArgs) Handles txtNewmanPres.TextChanged

    End Sub

    Private Sub txtNewmanPres_Validating(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles txtNewmanPres.Validating
        If ChkNumeric(Me.txtNewmanPres, e) Then NewmanPres = CSng(Me.txtNewmanPres.Text)
    End Sub

    Private Sub rdoCompSand_CheckedChanged(sender As Object, e As EventArgs) Handles rdoCompSand.CheckedChanged
        If Me.rdoCompSand.Checked Then
            optRockLith = 0
            Me.rdoSand.Checked = True
        End If
    End Sub

    Private Sub rdoCompLime_CheckedChanged(sender As Object, e As EventArgs) Handles rdoCompLime.CheckedChanged
        If Me.rdoCompLime.Checked Then
            optRockLith = 1
            Me.rdoLime.Checked = True
        End If
    End Sub

    Private Sub PictureBox7_Click_1(sender As Object, e As EventArgs) Handles PictureBox7.Click
        Process.Start("http://www.mhausa.com/")
    End Sub

    Private Sub mnuCitations_Click(sender As Object, e As EventArgs) Handles mnuCitations.Click
        frmCitations.ShowDialog()
    End Sub

    Private Sub rdoGasWater_CheckedChanged(sender As Object, e As EventArgs) Handles rdoGasWater.CheckedChanged
        If Me.rdoGasOil.Checked Then
            optRockFluids = 2
            InitRockForm()
        End If
    End Sub
End Class