﻿Option Explicit On
'
'Name: modP3
'
'Author: D.V. Allred (Analytics CIS, 08/May/2015)
'        based on original FORTRAN by G.B. HUENI (MHA circa 1994)
'
'Description: Calculate fluid properties from correlations
'
'Note: All arrays are declared assuming subscript zero will not be used
'
Module modP3
    Private Const PRESSURE_RANGE = 0
    Private Const DISCRETE_VALUES = 1
    Private Const GRAVITY_INERTS = 0
    Private Const COMPOSITION = 1
    Private Const RSI_ENTERED = 1
    Private Const PBP_ENTERED = 2

    Private Const SYS_OILWATER = 0
    Private Const SYS_GASOIL = 1
    Private Const DRAINAGE = 0
    Private Const IMBIBITION = 1
    Private Const SANDSTONE = 0
    Private Const LIMESTONE = 1
    Private Const WATERWET = 0
    Private Const OILWET = 1
    Private Const COREY = 0
    Private Const HONARPOUR = 1

    Private Const Lime_A = 0.853531
    Private Const Lime_B = 1.07538
    Private Const Lime_C = 2303040
    Private Const Sand_A = 0.00009732
    Private Const Sand_B = 0.699993
    Private Const Sand_C = 79.8181

    Public NPVTPRS, IPBP As Int32

    Public MinPrs, MaxPrs, IniPrs, PVTPRS(21), RSO(21), RSOS(21) As Single
    Public BO(21), RHOO(21), CO(21), MUO(21), BOS(21), MUOS(21), RHOOS(21) As Single
    Public PBP, PSEP, TSEP, API, GSEP, RSISEP, GSEPN, RSITANK, RSI, VISODEAD As Single
    Public STDPRS, STDTMP, RESTEMPR, STDENOIL, RESTEMP, SGOILSC, STDENGAS As Single
    Public GHC, YN2, YCO2, YH2S, G, GASMW As Single
    Public TC, PC, Z(21), BG(21), RHOG(21), CG(21), MUG(21) As Single
    Public RMP(21), PZ(21), MOLEFR(16) As Single

    Public BW(21), STDENWAT, RSW(21), CW(21) As Single
    Public RHOW(21), MUW(21) As Single
    Public PWREF, BWREF, CWREF, MUWREF, DMUWDP As Single

    Public idxPlus As Int32
    Public flagPlus As Boolean
    Public PlusMolWt, PlusSG As Single

    Public SWC, SORW, SORG, SGCrit, CoreyK, CoreyN, HRPoro, HRAir, HRKrg, NewmanPor, NewmanPres, NewmanCF As Single
    Public optRockFluids, optRockDisplace, optRockCorr, optRockLith, optRockWet As Int32

    Public pvtSWet(15) As Single       'Output saturation divisions (wet)
    Public pvtSNWet(15) As Single      'Output saturation divisions (non-wet)
    Public NWet As Integer             'Size of output array

    Public pvtSD(15) As Single         'Satuation
    Public pvtKROil(15) As Single      'Permeability - Oil
    Public pvtKRWat(15) As Single      'Permeability - Water
    Public pvtKRGas(15) As Single      'Permeability - Gas
    '
    '---Start Gas properties calculations
    '
    Public Sub CalcGas(optPrs As Int32, optGas As Int32, oilIN As Boolean)
        Dim ii As Int32
        Dim flagNoEntry As Boolean
        Dim iOptOil As Int32
        '        Dim nPSI(21) As Single
        '
        '---Gen pressure range values according to min/max/initial from Pressure Input tab
        '
        If Not GenPressure(optPrs) Then Exit Sub
        '
        '---Process Gas input option COMPOSITION.  Save user entered Composition values from Gas Input tab
        '
        If optGas = COMPOSITION Then
            For ii = 1 To 16
                MOLEFR(ii) = 0.0
            Next
            MOLEFR(1) = CSng(frmStart.txtGC1.Text)
            MOLEFR(2) = CSng(frmStart.txtGC2.Text)
            MOLEFR(3) = CSng(frmStart.txtGC3.Text)
            MOLEFR(4) = CSng(frmStart.txtGC4.Text)
            MOLEFR(5) = CSng(frmStart.txtGC5.Text)
            MOLEFR(6) = CSng(frmStart.txtGC6.Text)
            MOLEFR(7) = CSng(frmStart.txtGC7.Text)
            MOLEFR(8) = CSng(frmStart.txtGC8.Text)
            MOLEFR(9) = CSng(frmStart.txtGC9.Text)
            MOLEFR(10) = CSng(frmStart.txtGC10.Text)
            MOLEFR(11) = CSng(frmStart.txtGC11.Text)
            MOLEFR(12) = CSng(frmStart.txtGC12.Text)

            MOLEFR(13) = CSng(frmStart.txtNGC.Text)
            MOLEFR(14) = CSng(frmStart.txtDX.Text)
            MOLEFR(15) = CSng(frmStart.txtSU.Text)
            '
            '---Check if any gas composition value were actually entered
            '
            flagNoEntry = True
            For ii = 1 To 16
                If MOLEFR(ii) > 0 Then flagNoEntry = False
            Next ii
            If flagNoEntry Then
                ErrMSG("Please enter at least one Gas Composition value before proceeding!", "No data")
                Exit Sub
            End If
            '
            '---Check if values "Normalized"
            '
            If Not flagNormalize Then
                ErrMSG("Please Normalize your Gas Composition values before proceeding!", "Abnormal not Permitted")
                Exit Sub
            End If
        End If

        iOptOil = 0
        If oilIN Then iOptOil = 1

        'GPVT(optGas, iOptOil)
        GPVT2(optGas, iOptOil)

    End Sub
    '
    '---Gen pressure range values according to min/max/initial from Pressure Input tab
    '
    Private Function GenPressure(optPrs As Int32) As Boolean
        Dim nIncr As Single
        Dim ii, jj As Int32
        Dim flagAdd As Boolean
        Dim nTemp As Single
        Dim nPSI(21) As Single

        GenPressure = True
        If NPVTPRS > 0 Then Exit Function

        If optPrs = PRESSURE_RANGE Then
            nIncr = Math.Round((MaxPrs - MinPrs) / 15, 0)
            PVTPRS(1) = MaxPrs
            For ii = 2 To 15
                PVTPRS(ii) = PVTPRS(ii - 1) - nIncr
            Next ii
            PVTPRS(16) = MinPrs
            NPVTPRS = 16
            '
            '---If initial prs is within current range, check if it's already there
            '
            If IniPrs > MinPrs And IniPrs < MaxPrs Then
                flagAdd = True
                For ii = 1 To 16
                    If IniPrs = PVTPRS(ii) Then flagAdd = False
                Next ii
                '
                '---Add initial pressure
                '
                If flagAdd Then
                    For ii = 2 To 16
                        If IniPrs < PVTPRS(ii - 1) And IniPrs > PVTPRS(ii) Then

                            For jj = 17 To ii + 1 Step -1
                                PVTPRS(jj) = PVTPRS(jj - 1)
                            Next jj
                            PVTPRS(ii) = IniPrs
                            Exit For
                        End If
                    Next ii
                    NPVTPRS = 17
                End If
            End If
        End If
        '
        '---Save user entered pressure values from Pressure Input tab
        '
        If optPrs = DISCRETE_VALUES Then
            For ii = 1 To 20
                nPSI(ii) = 0.0
                PVTPRS(ii) = 0.0
            Next
            nPSI(1) = CSng(frmStart.txtPV1.Text)
            nPSI(2) = CSng(frmStart.txtPV2.Text)
            nPSI(3) = CSng(frmStart.txtPV3.Text)
            nPSI(4) = CSng(frmStart.txtPV4.Text)
            nPSI(5) = CSng(frmStart.txtPV5.Text)
            nPSI(6) = CSng(frmStart.txtPV6.Text)
            nPSI(7) = CSng(frmStart.txtPV7.Text)
            nPSI(8) = CSng(frmStart.txtPV8.Text)
            nPSI(9) = CSng(frmStart.txtPV9.Text)
            nPSI(10) = CSng(frmStart.txtPV10.Text)
            nPSI(11) = CSng(frmStart.txtPV11.Text)
            nPSI(12) = CSng(frmStart.txtPV12.Text)
            nPSI(13) = CSng(frmStart.txtPV13.Text)
            nPSI(14) = CSng(frmStart.txtPV14.Text)
            nPSI(15) = CSng(frmStart.txtPV15.Text)
            nPSI(16) = CSng(frmStart.txtPV16.Text)
            nPSI(17) = CSng(frmStart.txtPV17.Text)
            nPSI(18) = CSng(frmStart.txtPV18.Text)
            '
            '---Sort, highest to lowest
            '
            For ii = 1 To 18
                For jj = ii + 1 To 18
                    If nPSI(jj) > nPSI(ii) Then
                        nTemp = nPSI(ii)
                        nPSI(ii) = nPSI(jj)
                        nPSI(jj) = nTemp
                    End If
                Next jj
            Next ii
            '
            '---Move PSI values to PVTPRS and ignore dups
            '
            PVTPRS(1) = nPSI(1)
            NPVTPRS = 1
            For ii = 2 To 18
                If PVTPRS(NPVTPRS) <> nPSI(ii) Then
                    NPVTPRS = NPVTPRS + 1
                    PVTPRS(NPVTPRS) = nPSI(ii)
                End If
            Next ii
            If PVTPRS(NPVTPRS) = 0 Then NPVTPRS = NPVTPRS - 1
            '
            '---Check if user entered any values
            '
            If NPVTPRS = 0 Then
                GenPressure = False
                ErrMSG("Please enter at least one pressure value!", "No Pressure Data")
                Exit Function
            End If
        End If

    End Function
    '
    '---Start Water properties calculations
    '
    Public Sub CalcWater(optPrs As Int32, WTSOLIDS As Single)
        Dim iErr As Int32

        If Not GenPressure(optPrs) Then Exit Sub

        WPVT(WTSOLIDS, iErr)
    End Sub

    Public Sub CalcOil(optPBP As Int32)
    
        OPVT(optPBP)

    End Sub
    '
    '---CALCULATE GAS PROPERTIES FROM CORRELATIONS
    '---Call: GASTCPC to determine critical pressure and temperature,including non-hydrocarbons
    '---      ZFACT to determine gas deviation factor
    '
    Public Function GPVT(IOPT1 As Int32, IOPT2 As Int32) As String

        Dim IERRZ(21) As Single
        Dim ICOMP, ii As Int32
        Dim PPR(21), PTR(21) As Single
        Dim STDTMPR As Single
        Dim strROW() As String
        Dim CTPlus, CPPlus As Single

        Dim CP() As Single = {0, 667.8, 707.8, 616.3, 550.7, 529.1, 488.6, 490.4, 436.9, 396.8, 360.6, 332.0, 304.0, 493.0, 1070.9, 1306.0}
        Dim CT() As Single = {0, -116.6, 90.1, 206.0, 305.7, 275.0, 385.7, 369.1, 453.7, 512.8, 564.2, 610.7, 652.1, -232.4, 87.9, 212.7}
        Dim CMW() As Single = {0, 16.043, 30.07, 44.097, 58.124, 58.124, 72.151, 72.151, 86.178, 100.21, 114.23, 128.26, 142.29, 28.013, 44.01, 34.076}

        Dim retStatus As String = ""
        '
        '---IOPT1=0 -> Use gas gravity, IOPT1=1 -> Use gas composition
        '
        GPVT = "OK"
        RESTEMPR = RESTEMP + 459.67
        STDTMPR = STDTMP + 459.67
        ICOMP = IOPT1

        If ICOMP = 1 Then
            PC = 0.0
            TC = 0.0
            GHC = 0.0
            GASMW = 0.0
            G = 0.0
            '
            '---Process gas comp entry with a "PLUS" fraction
            '
            If flagPlus Then
                For ii = 1 To idxPlus - 1
                    TC = TC + MOLEFR(ii) * CT(ii)
                    PC = PC + MOLEFR(ii) * CP(ii)
                Next ii
                CTPlus = 608 + 364 * Math.Log10(PlusMolWt - 71.2) + (2450 * Math.Log10(PlusMolWt) - 3800) * Math.Log10(PlusSG)
                TC = TC + MOLEFR(idxPlus) * CTPlus
                TC = TC + 459.67

                CPPlus = 1188 - 431 * Math.Log10(PlusMolWt - 61.1) + (2319 - 852 * Math.Log10(PlusMolWt - 53.7)) * (PlusSG - 0.8)
                PC = PC + MOLEFR(idxPlus) * CPPlus

                For ii = 1 To idxPlus - 1
                    GASMW = GASMW + MOLEFR(ii) * CMW(ii)
                Next ii
                GASMW = GASMW + MOLEFR(idxPlus) * PlusMolWt

                '        GHC = GASMW / 28.96

                For ii = 13 To 15                                             'Add Nitrogen, Carbon Dioxide and Hydrogen Sulfide
                    GASMW = GASMW + MOLEFR(ii) * CMW(ii)
                Next ii

                G = GASMW / 28.96
                GHC = (G - 1.1767 * MOLEFR(15) - 1.5196 * MOLEFR(14) - 0.9672 * MOLEFR(13)) / (1 - MOLEFR(15) - MOLEFR(14) - MOLEFR(13))
            Else
                '
                '---Process gas comp entry WITHOUT a "PLUS" fraction
                '
                For ii = 1 To 15
                    TC = TC + MOLEFR(ii) * CT(ii)
                    PC = PC + MOLEFR(ii) * CP(ii)
                Next ii

                TC = TC + 459.67
                For ii = 1 To 12
                    GASMW = GASMW + MOLEFR(ii) * CMW(ii)
                Next ii

                GHC = GASMW / 28.96

                For ii = 13 To 15
                    GASMW = GASMW + MOLEFR(ii) * CMW(ii)
                Next ii

                G = GASMW / 28.96
            End If
        End If
        '
        '---IOPT2=0  ->  Gas Reservoir
        '---IOPT2=1  ->  Oil Reservoir with Free Gas
        '
        If ICOMP = 0 Then
            GHC = (G - 0.9672 * YN2 - 1.5195 * YCO2 - 1.1765 * YH2S) / (1.0 - YN2 - YCO2 - YH2S)
            GASMW = 28.96 * G

            GASTCPC(IOPT2, G, GHC, YN2, YCO2, YH2S, TC, PC, GASMW)
        End If
        '
        '---End of critical temperature and pressure calculation
        '---Perform correlations for specified pressures and temperature
        '
        STDENGAS = STDPRS * GASMW / (10.73 * STDTMPR)

        For ii = 1 To NPVTPRS
            PPR(ii) = PVTPRS(ii) / PC
            PTR(ii) = RESTEMPR / TC
            ZFACT(PTR(ii), PPR(ii), Z(ii), IERRZ(ii))

            PZ(ii) = PVTPRS(ii) / Z(ii)
            BG(ii) = Z(ii) * RESTEMPR * STDPRS / (STDTMPR * PVTPRS(ii))
            RHOG(ii) = PVTPRS(ii) * GASMW / (Z(ii) * 10.73 * RESTEMPR)

            GASCOM(TC, PC, PVTPRS(ii), RESTEMPR, CG(ii))
            If GASVIS(RESTEMPR, PVTPRS(ii), Z(ii), G, MUG(ii)) <> "OK" Then
                GPVT = "GASVIS ERROR"
                Exit Function
            End If

            RMP(ii) = GASRMP(TC, PC, PVTPRS(ii), RESTEMPR, G, retStatus)
            If retStatus <> "OK" Then
                GPVT = "GASRMP error"
                Exit Function
            End If
        Next ii
        '
        '---Display gas properties
        '
        frmStart.txtGPgg.Text = G.ToString("F3")
        frmStart.txtGPRtemp.Text = RESTEMP.ToString("F1")
        frmStart.txtGPcp.Text = PC.ToString("F1")
        frmStart.txtGPct.Text = TC.ToString("F1")
        frmStart.txtGPpt.Text = PTR(1).ToString("F3")
        frmStart.txtGPtgd.Text = STDENGAS.ToString("F5")

        gridGP.DeleteAll()
        For ii = 1 To NPVTPRS
            strROW = New String() {PVTPRS(ii).ToString("F1"), PPR(ii).ToString("F3"), Z(ii).ToString("F3"), PZ(ii).ToString("F1"), BG(ii).ToString("F6"), _
                                   MUG(ii).ToString("F4"), CG(ii).ToString("F6"), RHOG(ii).ToString("F4"), (RMP(ii) / 1000000.0).ToString("F4")}
            gridGP.AddRow(strROW)
        Next
    End Function
    '
    '---CALCULATE GAS PROPERTIES FROM CORRELATIONS
    '---New logic as supplied by K.Wilson (MHA 7/7/2015) - Sutton 2007 and Wichert/Aziz Correlations
    '
    Public Function GPVT2(IOPT1 As Int32, IOPT2 As Int32) As String

        Dim IERRZ(21) As Single
        Dim ICOMP, ii As Int32
        Dim PPR(21), PTR(21) As Single
        Dim STDTMPR As Single
        Dim strROW() As String
        Dim CTPlus, CPPlus As Single
        Dim PC_HC, TC_HC, PC_Mixture, TC_Mixture As Single
        Dim A_Wichert, B_Wichert, E_Wichert As Single

        Dim CP() As Single = {0, 667.8, 707.8, 616.3, 550.7, 529.1, 488.6, 490.4, 436.9, 396.8, 360.6, 332.0, 304.0, 493.0, 1070.9, 1306.0}
        Dim CT() As Single = {0, 343, 549.59, 665.73, 734.13, 765.29, 828.77, 845.47, 913.27, 973.47, 1023.9, 1070.4, 1111.8, 227.16, 547.58, 672.35}
        '        Dim CT() As Single = {0, -116.6, 90.1, 206.0, 305.7, 275.0, 385.7, 369.1, 453.7, 512.8, 564.2, 610.7, 652.1, -232.4, 87.9, 212.7}
        Dim CMW() As Single = {0, 16.043, 30.07, 44.097, 58.124, 58.124, 72.151, 72.151, 86.178, 100.21, 114.23, 128.26, 142.29, 28.013, 44.01, 34.076}

        Dim pc_A As Single = 744.0
        Dim pc_B As Single = -125.4
        Dim pc_C As Single = 5.9

        Dim tc_A As Single = 164.3
        Dim tc_B As Single = 357.7
        Dim tc_C As Single = -67.7

        Dim retStatus As String = ""
        '
        '---IOPT1=0 -> Use gas gravity, IOPT1=1 -> Use gas composition
        '
        GPVT2 = "OK"
        RESTEMPR = RESTEMP + 459.67
        STDTMPR = STDTMP + 459.67
        ICOMP = IOPT1

        If ICOMP = 1 Then                           'Gas Composition
            PC = 0.0
            TC = 0.0
            GHC = 0.0
            GASMW = 0.0
            G = 0.0
            '
            '---Process gas comp entry with a "PLUS" fraction
            '
            If flagPlus Then
                For ii = 1 To idxPlus - 1
                    TC = TC + MOLEFR(ii) * CT(ii)
                    PC = PC + MOLEFR(ii) * CP(ii)
                Next ii
                CTPlus = 608 + 364 * Math.Log10(PlusMolWt - 71.2) + (2450 * Math.Log10(PlusMolWt) - 3800) * Math.Log10(PlusSG)
                TC = TC + MOLEFR(idxPlus) * CTPlus
                TC = TC + 459.67

                CPPlus = 1188 - 431 * Math.Log10(PlusMolWt - 61.1) + (2319 - 852 * Math.Log10(PlusMolWt - 53.7)) * (PlusSG - 0.8)
                PC = PC + MOLEFR(idxPlus) * CPPlus

                For ii = 1 To idxPlus - 1
                    GASMW = GASMW + MOLEFR(ii) * CMW(ii)
                Next ii
                GASMW = GASMW + MOLEFR(idxPlus) * PlusMolWt

                For ii = 13 To 15                                             'Add Nitrogen, Carbon Dioxide and Hydrogen Sulfide
                    GASMW = GASMW + MOLEFR(ii) * CMW(ii)
                Next ii

                G = GASMW / 28.9625
                YN2 = MOLEFR(13)
                YCO2 = MOLEFR(14)
                YH2S = MOLEFR(15)

                GHC = (G - 1.1767 * YH2S - 1.5196 * YCO2 - 0.9672 * YN2) / (1 - YH2S - YCO2 - YN2)
            Else
                '
                '---Process gas comp entry WITHOUT a "PLUS" fraction
                '
                For ii = 1 To 15
                    TC = TC + MOLEFR(ii) * CT(ii)
                    PC = PC + MOLEFR(ii) * CP(ii)
                Next ii

                TC = TC + 459.67
                For ii = 1 To 12
                    GASMW = GASMW + MOLEFR(ii) * CMW(ii)
                Next ii

                GHC = GASMW / 28.96

                For ii = 13 To 15
                    GASMW = GASMW + MOLEFR(ii) * CMW(ii)
                Next ii

                G = GASMW / 28.96
            End If
        End If
        '
        '---Gas Gravity & Inerts
        '
        If ICOMP = 0 Then
            GHC = (G - 1.1767 * YH2S - 1.5196 * YCO2 - 0.9672 * YN2 / (1 - YH2S - YCO2 - YN2))
            GASMW = 28.96 * G
        End If
        '
        '---Sutton 2007 Correlation
        '
        PC_HC = pc_A + pc_B * GHC + pc_C * GHC ^ 2
        TC_HC = tc_A + tc_B * GHC + tc_C * GHC ^ 2

        PC_Mixture = PC_HC * (1.0 - YH2S - YCO2 - YN2) + 1306 * YH2S + 1071 * YCO2 + 493.1 * YN2
        TC_Mixture = TC_HC * (1.0 - YH2S - YCO2 - YN2) + 672.35 * YH2S + 547.58 * YCO2 + 227.16 * YN2
        '
        '---Wichert/Aziz Correlation
        '
        A_Wichert = YH2S + YCO2
        B_Wichert = YH2S
        E_Wichert = 120.0 * (A_Wichert ^ 0.9 - A_Wichert ^ 1.6) + 15.0 * (B_Wichert ^ 0.5 - B_Wichert ^ 4.0)

        TC = TC_Mixture - E_Wichert
        PC = PC_Mixture * TC / (TC_Mixture + B_Wichert * (1.0 - B_Wichert) * E_Wichert)

        ''
        ''---IOPT2=0  ->  Gas Reservoir
        ''---IOPT2=1  ->  Oil Reservoir with Free Gas
        ''
        'If ICOMP = 0 Then
        '    GHC = (G - 0.9672 * YN2 - 1.5195 * YCO2 - 1.1765 * YH2S) / (1.0 - YN2 - YCO2 - YH2S)
        '    GASMW = 28.96 * G

        '    GASTCPC(IOPT2, G, GHC, YN2, YCO2, YH2S, TC, PC, GASMW)
        'End If
        '
        '---End of critical temperature and pressure calculation
        '---Perform correlations for specified pressures and temperature
        '
        STDENGAS = STDPRS * GASMW / (10.73 * STDTMPR)

        For ii = 1 To NPVTPRS
            PPR(ii) = PVTPRS(ii) / PC
            PTR(ii) = RESTEMPR / TC
            ZFACT(PTR(ii), PPR(ii), Z(ii), IERRZ(ii))

            PZ(ii) = PVTPRS(ii) / Z(ii)
            BG(ii) = Z(ii) * RESTEMPR * STDPRS / (STDTMPR * PVTPRS(ii))
            RHOG(ii) = PVTPRS(ii) * GASMW / (Z(ii) * 10.73 * RESTEMPR)

            GASCOM(TC, PC, PVTPRS(ii), RESTEMPR, CG(ii))
            If GASVIS(RESTEMPR, PVTPRS(ii), Z(ii), G, MUG(ii)) <> "OK" Then
                GPVT2 = "GASVIS ERROR"
                Exit Function
            End If

            RMP(ii) = GASRMP(TC, PC, PVTPRS(ii), RESTEMPR, G, retStatus)
            If retStatus <> "OK" Then
                GPVT2 = "GASRMP error"
                Exit Function
            End If
        Next ii
        '
        '---Display gas properties
        '
        frmStart.txtGPgg.Text = G.ToString("F3")
        frmStart.txtGPRtemp.Text = RESTEMP.ToString("F1")
        frmStart.txtGPcp.Text = PC.ToString("F1")
        frmStart.txtGPct.Text = TC.ToString("F1")
        frmStart.txtGPpt.Text = PTR(1).ToString("F3")
        frmStart.txtGPtgd.Text = STDENGAS.ToString("F5")

        gridGP.DeleteAll()
        For ii = 1 To NPVTPRS
            strROW = New String() {PVTPRS(ii).ToString("F1"), PPR(ii).ToString("F3"), Z(ii).ToString("F3"), PZ(ii).ToString("F1"), BG(ii).ToString("F6"), _
                                   MUG(ii).ToString("F4"), CG(ii).ToString("F6"), RHOG(ii).ToString("F4"), (RMP(ii) / 1000000.0).ToString("F4")}
            gridGP.AddRow(strROW)
        Next
    End Function
    '
    '---Add PBP to pressure range.  Note: pressure range is sorted highest to lowest
    '
    Private Function AddPBP(optIN As String)
        Dim ii, jj As Int32

        AddPBP = 0
        Select Case optIN

            Case "LESS"                                                          'Add PBP to end of range
                NPVTPRS = NPVTPRS + 1
                PVTPRS(NPVTPRS) = PBP
                AddPBP = NPVTPRS

            Case "MID"                                                           'Suffle array contents down one subscript to make room
                For ii = 1 To NPVTPRS
                    If PBP > PVTPRS(ii) Then
                        NPVTPRS = NPVTPRS + 1
                        For jj = NPVTPRS To ii + 1 Step -1
                            PVTPRS(jj) = PVTPRS(jj - 1)
                        Next
                        PVTPRS(ii) = PBP
                        AddPBP = ii
                        Exit For
                    End If
                Next ii

            Case "GREATER"                                                       'Suffle array contents down two subscripts to add PBP and PBP+500
                NPVTPRS = NPVTPRS + 2
                For ii = NPVTPRS To 3 Step -1
                    PVTPRS(ii) = PVTPRS(ii - 2)
                Next
                PVTPRS(1) = PBP + 500
                PVTPRS(2) = PBP
                AddPBP = 2
        End Select

    End Function
    '
    '---CALCULATE OIL PROPERTIES FROM CORRELATIONS
    '
    '---Assumptions: Public (common) variables set prior to call: RESTEMP, API, GSEP, PSEP, TSEP, NPVTPRS, PVTPRS, RSI or PBP
    '
    Public Sub OPVT(IOPT1 As Int32)
        Dim X As Single
        Dim A As Single
        Dim B As Single
        Dim ii As Int32
        Dim strRow() As String
        Dim flagPBP As String
        '
        '---Correct separator gor for vent gas
        '
        SGOILSC = 141.5 / (131.5 + API)
        RSITANK = 0.3818 - 5.506 * Log10(SGOILSC) + 2.902 * Log10(GSEP) + 1.327 * Log10(PSEP) - 0.7355 * Log10(TSEP)
        RSITANK = 10 ^ RSITANK
        '
        '---Need to print out rsitank and total rsi for user.  Correction for normalized sep gas gravity not used by mccain
        '
        GSEPN = GSEP * (1.0 + 0.00005912 * API * TSEP * Log10(PSEP / 114.7))
        Try
            '
            '---Calculate bubble point for known Rsi value
            '
            If IOPT1 = RSI_ENTERED Then
                RSISEP = RSI - RSITANK
                X = ((RSI / GSEP) ^ 0.83) * (10 ^ (0.00091 * RESTEMP - 0.0125 * API))
                PBP = 18.2 * (X - 1.4)
            End If
            '
            '---Show bubble point pressure, change if necessary or accept.  Calculate Rsi for known bubble point
            '
            If IOPT1 = PBP_ENTERED Then
                X = (PBP / 18.2) + 1.4
                X = X / (10 ^ (0.00091 * RESTEMP - 0.0125 * API))
                RSI = X ^ (1.0 / 0.83) * GSEP
                RSISEP = RSI - RSITANK
            End If
            '
            '---Check if bubble point pressure (PBP) already exists
            '
            IPBP = -1
            For ii = 1 To NPVTPRS
                If PBP = PVTPRS(ii) Then
                    IPBP = ii
                    Exit For
                End If
            Next ii
            '
            '---Add PBP to table. PBP can lie above, below or between current pressure range
            '
            If IPBP = -1 Then
                flagPBP = "MID"
                If PBP > PVTPRS(1) Then flagPBP = "GREATER"
                If PBP < PVTPRS(NPVTPRS) Then flagPBP = "LESS"

                IPBP = AddPBP(flagPBP)
            End If
            '
            '---Saturated dissolved gas-oil-ratio
            '
            For ii = 1 To NPVTPRS
                X = (PVTPRS(ii) / 18.2) + 1.4
                X = X / (10 ^ (0.00091 * RESTEMP - 0.0125 * API))
                RSOS(ii) = X ^ (1.0 / 0.83) * GSEP
            Next ii
            '
            '---Undersaturated dissolved gas oil ratio
            '
            For ii = 1 To NPVTPRS
                RSO(ii) = RSOS(ii)
            Next ii

            If IPBP < NPVTPRS Then
                For ii = IPBP - 1 To 1 Step -1
                    RSO(ii) = RSO(IPBP)
                Next ii
            End If
            '
            '---Undersaturated oil compressibility at the bubble point pressure
            '
            For ii = 1 To NPVTPRS
                CO(ii) = (-1433.0 + 5.0 * RSOS(ii) + 17.2 * RESTEMP - 1180.0 * GSEPN + 12.61 * API) / (100000.0 * PVTPRS(ii))
            Next ii
            '
            '---Saturated oil formation volume factor (rb/stb)
            '
            For ii = 1 To NPVTPRS
                X = RSOS(ii) * ((GSEP / SGOILSC) ^ 0.5) + 1.25 * RESTEMP
                BOS(ii) = 0.9759 + 0.00012 * X ^ 1.2
            Next ii
            '
            '---Undersaturated oil form vol factor (rb/stb)
            '
            For ii = 1 To NPVTPRS
                BO(ii) = BOS(ii)
            Next ii

            If IPBP < NPVTPRS Then
                For ii = IPBP - 1 To 1 Step -1
                    BO(ii) = BO(IPBP) * Math.Exp(CO(IPBP) * (PBP - PVTPRS(ii)))
                Next ii
            End If
            '
            '---Saturated oil viscosity (cp)
            '
            X = 1.8653 - 0.02508 * API - 0.5644 * Log10(RESTEMP)
            X = 10 ^ (10 ^ X)
            X = X - 1.0
            VISODEAD = X

            For ii = 1 To NPVTPRS
                A = 10.715 * (RSOS(ii) + 100.0) ^ (-0.515)
                B = 5.44 * (RSOS(ii) + 150.0) ^ (-0.338)
                MUOS(ii) = A * VISODEAD ^ B
            Next ii
            '
            '---Undersaturated oil viscosity (cp) 
            '
            For ii = 1 To NPVTPRS
                MUO(ii) = MUOS(ii)
            Next ii

            If IPBP < NPVTPRS Then
                For ii = IPBP - 1 To 1 Step -1
                    A = 2.6 * (PVTPRS(ii) ^ 1.187) * Math.Exp(-0.0000898 * PVTPRS(ii) - 11.513)
                    MUO(ii) = MUO(IPBP) * (PVTPRS(ii) / PBP) ^ A
                Next ii
            End If
            '
            '---Stock tank oil density  (lb/scf)
            '
            STDENOIL = SGOILSC * 62.4
            STDENGAS = STDPRS * 28.96 * GSEP / (10.73 * (STDTMP + 459.67))
            '
            '---Sat and undersat oil density
            '
            For ii = 1 To NPVTPRS
                RHOOS(ii) = (STDENOIL + STDENGAS * RSOS(ii) / 5.614586) / BOS(ii)
                RHOO(ii) = (STDENOIL + STDENGAS * RSO(ii) / 5.614586) / BO(ii)
            Next ii

        Catch ex As Exception
            If ii > NPVTPRS Then ii = NPVTPRS
            ErrMSG("Error at pressure " & PVTPRS(ii) & " - Please adjust input paramemters!", "Oil Properties Error")
        End Try
        '
        '---Display Oil properties
        '
        frmStart.txtOPbp.Text = PBP.ToString("F0")
        frmStart.txtOPComp.Text = CO(IPBP).ToString("F7")
        frmStart.txtOPtod.Text = STDENOIL.ToString("F5")
        frmStart.txtOPGORsep.Text = RSISEP.ToString("F1")
        frmStart.txtOPGORtank.Text = RSITANK.ToString("F1")
        frmStart.txtOPGORinit.Text = RSI.ToString("F1")

        gridOP.DeleteAll()
        For ii = 1 To NPVTPRS
            strRow = New String() {PVTPRS(ii).ToString("F1"), BO(ii).ToString("F4"), RSO(ii).ToString("F1"), MUO(ii).ToString("F3"), RHOO(ii).ToString("F3"), " ", _
                                   BOS(ii).ToString("F4"), RSOS(ii).ToString("F1"), MUOS(ii).ToString("F3"), RHOOS(ii).ToString("F3")}
            gridOP.AddRow(strRow)

            If PVTPRS(ii) = PBP Then
                frmStart.dgvOP.Rows(ii - 1).DefaultCellStyle.BackColor = Color.Pink
            End If
        Next
    End Sub
    '
    '---Estimates the Z factor by using a technique developed by Dranchuk, Purvis, and Robison.  It had the lowest
    '   average error of eight methods commonly used to estimate the Z factor.
    '   RE:  1.  Dranchuk, P.M., Purvis, R. A., and Robinson, D. B.,
    '   "Computer Calculations of Natural Gas Compressibility Factors"
    '   Using the Standing and Katz Correlation,", Institute of Petroleum
    '   Technical Series, No. IP 74-008, 1974.
    '        2.  Takacs, G., "Comparisons Made for Computer Z-Factor
    '        C(Calculations, " Oil and Gas Journal, Dec 20, 1976, pp. 64-66.")
    '
    ' INPUTS
    '    TR  - <REAL> The Pseudo reduced temperature of the gas mixture in
    '          degrees Rankine.  Range:  1.05<=TR<3.0
    '    PR  - <REAL> The Pseudo redured pressure of the gas mixture in
    '          psia.  Range:  0<PR<30
    '  OUTPUTS:
    '    Z   - <REAL> The gas compressibility factor of the gas mixture.
    '    ERR - <INTEGER> The error code.
    '          ERR = 0  No error.
    '          ERR = 1  TR out of range.  An attempt is made to estimate
    '                   a value for the Z factor regardless.
    '          ERR = 2  PR out of range.  An attempt is made to estimate
    '                   a value for the Z factor regardless.
    '          ERR = 3  Iteration procedure could not converge.
    '
    Public Sub ZFACT(ByRef TR As Single, ByRef PR As Single, ByRef Z As Single, ByRef ERR As Int32)

        Dim A, B, C, D, E, F, G, DR, FUNCT, DFUNCT, TOLER, DR3 As Single
        Dim ITER As Int32

        ERR = 0
        If TR < 1.05 Or TR >= 3.0 Then ERR = 1

        If PR <= 0.0 Or PR >= 30.0 Then ERR = 2

        A = 0.06423
        B = 0.5353 * TR - 0.6123
        C = 0.3151 * TR - 1.0467 - 0.5783 / (TR * TR)
        D = TR
        E = 0.6816 / (TR * TR)
        G = 0.27 * PR

        DR = 0.27 * PR / TR
        TOLER = 0.000001
        '        TOLER = 1.E - 6

        For ITER = 1 To 50
            F = 0.6845 * DR * DR
            DR3 = DR * DR * DR
            FUNCT = (((A * DR3 + B + E * (1 + F) * Math.Exp(-F)) * DR + C) * DR + D) * DR - G

            If Math.Abs(FUNCT) < TOLER Then
                Z = 0.27 * PR / (DR * TR)
                Exit Sub
            End If

            DFUNCT = ((6.0 * A * DR3 + 3.0 * B + E * (3.0 + F * (3.0 - 2.0 * F)) * Math.Exp(-F)) * DR + 2.0 * C) * DR + D
            DR = DR - FUNCT / DFUNCT
        Next ITER

        ERR = 3
        Z = 0.27 * PR / (DR * TR)

    End Sub
    '
    '---Calculates Gas Compressibility for Real Natural Gases
    '       
    '  PRES  = Gas pressure (PSIA)
    '  RESTEMP  = Gas temperature (degrees Rankine)
    '  SPG  = Gas specific gravity (air = 1)
    '  CMPG = Gas compressibility (1 / PSI)
    '  ZED  = Gas deviation factor (compressibility factor)
    '
    '      1   1 dZ
    '  C = - - - --
    '      P   Z dP
    '
    '  In this subroutine, use dP = 50 and calculate dZ = Z(P+25) - Z(P-25)
    '
    ' CALLS - ZFACT()
    '
    Public Sub GASCOM(ByRef TC As Single, ByRef PC As Single, ByRef PRES As Single, ByRef RESTEMP As Single, ByRef CMPG As Single)

        Dim PPR, PTR, Err, ZED, DP As Single
        Dim PRS1, PPR1, ZED1 As Single
        Dim PRS2, PPR2, ZED2 As Single

        PPR = PRES / PC
        PTR = RESTEMP / TC
        ZFACT(PTR, PPR, ZED, Err)

        If PRES < 50.0 Then
            DP = PRES + PRES
            PRS1 = 0.0
            PRS2 = DP
            PPR1 = 0.0
            ZED1 = 1.0
            PPR2 = DP / PC
        Else
            DP = 50.0
            PRS1 = PRES + 25.0
            PRS2 = PRES - 25.0
            PPR1 = PRS1 / PC
            PPR2 = PRS2 / PC
            Call ZFACT(PTR, PPR1, ZED1, Err)
        End If

        Call ZFACT(PTR, PPR2, ZED2, Err)

        CMPG = 1.0 / PRES - (ZED1 - ZED2) / (DP * ZED)
    End Sub
    '
    '---Calculates gas viscosity
    '  Reference: Lee, et al., JPT Aug 1966 PG 997-1000
    '
    '  TEM = Temperature (Deg R)
    '  PRS = Pressure(PSIA)
    '  ZED = real gas correction (Z-factor)
    '  SPG = Gas specific gravity (air = 1)
    '  RHO = Gas Density (GM/CC)
    '  VISG= Gas viscosity (CP)
    '
    Public Function GASVIS(ByRef TEM As Single, ByRef PRS As Single, ByRef ZED As Single, ByRef SPG As Single, ByRef VISG As Single) As String

        Dim K, W, X, Y, RHO As Single

        VISG = 0.0
        GASVIS = "OK"

        If TEM < 459.67 Or TEM > 1500.0 Then
            ErrMSG("GASVIS: TEMPERATURE = " & TEM & " is OUT of Range", "Temperature Error")
            GASVIS = "TEMP ERR"
            Exit Function
        End If

        If PRS < 0.0 Or PRS > 20000.0 Then
            ErrMSG("GASVIS: PRESSURE = " & PRS & " is OUT of Range", "Pressure Error")
            GASVIS = "PRES ERR"
            Exit Function
        End If

        If ZED < 0.25 Or ZED > 3.0 Then
            ErrMSG("GASVIS: Z-FACTOR = " & ZED & " is OUT of Range", "Z-Factor error")
            GASVIS = "Z-FACT ERR"
            Exit Function
        End If

        If SPG < 0.45 Or SPG > 2.0 Then
            ErrMSG("GASVIS: GAS GRAVITY = " & SPG & " is OUT of Range", "Gas Gravity Error")
            GASVIS = "GG ERR"
            Exit Function
        End If

        W = SPG * 28.96

        If PRS > 12.0 Then RHO = W * PRS / (ZED * 670.0 * TEM)
        If PRS <= 12.0 Then RHO = W * 12.0 / (ZED * 670.0 * TEM)

        K = ((7.77 + 0.0063 * W) * TEM ^ 1.5) / (122.4 + 12.9 * W + TEM)
        X = 2.57 + 1914.5 / TEM + 0.0095 * W
        Y = 1.11 + 0.04 * X
        VISG = 0.0001 * K * Math.Exp(X * RHO ^ Y)

    End Function
    '
    '---Calculates the Real gas pseudo-pressure (potential)
    '     M(P) which is 2 times the integral of PRS/(VISG*ZED) dPRS from
    '     PRSMIN (= 0) to PRS using Romberg integration.
    '
    ' DEFINITIONS
    ' PRS - Pressure(PSIA)
    '     TEM - Temperature (degrees Rankine)
    '     SPG - Gas specific gravity (air = 1)
    '
    'CALLS - GASVIS, ZFACT
    '

    Public Function GASRMP(ByRef TC As Single, ByRef PC As Single, ByRef PRS As Single, ByRef TEM As Single, ByRef SPG As Single, ByRef retStatus As String) As Single

        Dim MAXIT, I, J, K, N, iERROR As Int32
        Dim R(21), TOL, PRSMIN, H, PRSPR, TEMPR, ZED, VISG As Single
        Dim OLD1, SUM, PRSARG, DENOM, OLD2 As Single

        retStatus = "OK"
        GASRMP = 0.0
        MAXIT = 20
        TOL = 0.000001

        PRSMIN = 0.0
        H = PRS - PRSMIN
        '
        '---CALL ZFACT, GASVIS for PRSMIN (if a different value is wanted)
        '
        PRSPR = PRS / PC
        TEMPR = TEM / TC
        ZFACT(TEMPR, PRSPR, ZED, iERROR)
        If GASVIS(TEM, PRS, ZED, SPG, VISG) <> "OK" Then
            retStatus = "GASVIS ERROR"
            Exit Function
        End If
        '
        '---R(1) must be altered if a non-zero PRSMIN is used.  Using the integral
        '---Bounds as A and B and an integrand of F(X),  R(1)=H/2*(F(A)+F(B)).
        '
        R(1) = (H / 2.0) * (2.0 * PRS) / (VISG * ZED)
        N = 1

        For K = 1 To MAXIT
            OLD1 = R(1)
            SUM = 0.0
            PRSARG = PRSMIN + H / 2.0

            For J = 1 To N
                PRSPR = PRSARG / PC
                TEMPR = TEM / TC
                ZFACT(TEMPR, PRSPR, ZED, iERROR)
                If GASVIS(TEM, PRSARG, ZED, SPG, VISG) <> "OK" Then
                    retStatus = "GASVIS ERROR"
                    Exit Function

                End If
                SUM = SUM + (2.0 * PRSARG) / (VISG * ZED)
                PRSARG = PRSARG + H
            Next J

            R(1) = (OLD1 + H * SUM) / 2.0
            DENOM = 4.0

            For I = 2 To K + 1
                OLD2 = R(I)
                R(I) = R(I - 1) + (R(I - 1) - OLD1) / (DENOM - 1.0)
                OLD1 = OLD2
                DENOM = DENOM * 4.0
            Next I

            H = H / 2.0
            N = N * 2.0

            If Math.Abs((R(K + 1) - R(K)) / R(K + 1)) < TOL Then
                GASRMP = R(K + 1)
                Exit Function
            End If
        Next K

        ErrMSG("GASRMP: no convergence!", " Calculation Error")
        retStatus = "NO CON"

    End Function
    '
    '   SUBROUTINE TO CALCULATE CRIFICAL TEMPERATURE AND PRESSURE
    '     BASED ON STANDING AND KATZ APPROACH WITH WEICKERT-AZIZ
    '     COORECTIONS FOR PRESENCE OF NON HYDROCARBONS. SEE HP41
    '     FLUID PACK CORRELATIONS.  ALSO CALCULATES GAS MOL WT.
    '
    '   INPUTS: IRESTYP    (0=GAS RESERVOIR; 1=OIL RESERVOIR)
    '           G          FULL WELLSTREAM GAS GRAVITY
    '           GHC        HYDROCARBON GAS GRAVITY
    '           YN2        MOLE FRACTION NITROGEN
    '           YCO2       MOLE FRACTION CARBON DIOXIDE
    '           YH2S       MOLE FRACTION HYDROGEN SULFIDE
    '
    '   NOTE: WILL USE GHC IF NON-ZERO OR WILL CALCULATE FROM G
    '         IF GHC = 0.
    '
    '   OUTPUTS: TC        CRITICAL TEMPERATURE
    '            PC        CRITICAL PRESSURE
    '            GASMW     MOLECULAR WEIGHT OF FULL WELLSTREAM GAS
    '
    Public Sub GASTCPC(ByRef IRESTYP As Int32, ByRef G As Single, ByRef GHC As Single, ByRef YN2 As Single, ByRef YCO2 As Single, ByRef YH2S As Single, ByRef TC As Single, _
                       ByRef PC As Single, ByRef GASMW As Single)

        Dim TCHC, PCHC, CWA As Single

        If G <> GHC Then
            If G <> 0.0 And GHC = 0.0 Then
                GHC = (G - 0.9672 * YN2 - 1.5195 * YCO2 - 1.1765 * YH2S) / (1.0 - YN2 - YCO2 - YH2S)
                GASMW = 28.96 * G
            ElseIf G = 0.0 And GHC <> 0.0 Then
                G = GHC * (1.0 - YN2 - YCO2 - YH2S) + (0.9672 * YN2 + 1.5195 * YCO2 + 1.1765 * YH2S)
                GASMW = 28.96 * G
            End If
        End If

        If IRESTYP = 0 Then
            TCHC = 187.0 + 330.0 * GHC - 71.5 * GHC ^ 2.0
            PCHC = 706.0 - 51.7 * GHC - 11.1 * GHC ^ 2.0
        End If

        If IRESTYP = 1 Then
            TCHC = 168.0 + 325.0 * GHC - 12.5 * GHC ^ 2.0
            PCHC = 677.0 - 15.0 * GHC - 37.5 * GHC ^ 2.0
        End If

        TC = (1.0 - YN2 - YCO2 - YH2S) * TCHC + 227.3 * YN2 + 547.6 * YCO2 + 672.4 * YH2S
        PC = (1.0 - YN2 - YCO2 - YH2S) * PCHC + 493.0 * YN2 + 1071.0 * YCO2 + 1306.0 * YH2S

        If YH2S <> 0.0 Then
            CWA = 120 * ((YCO2 + YH2S) ^ 0.9 - (YCO2 + YH2S) ^ 1.6) + 15.0 * (YCO2 ^ 0.5 - YH2S ^ 4.0)
            TC = TC - CWA
            PC = (PC * TC) / ((TC + CWA) + YH2S * (1.0 - YH2S) * CWA)
        End If

    End Sub
    '
    '---CALCULATE CRITICAL TEMPERATURE AND PRESSURE
    '   BASED ON STANDING AND KATZ APPROACH WITH WEICKERT-AZIZ
    '   COORECTIONS FOR PRESENCE OF NON HYDROCARBONS. SEE HP41
    '   FLUID PACK CORRELATIONS.  ALSO CALCULATES GAS MOL WT.
    '
    'Updated with "Sutton 2007 Correlation" (K. Wilson - MHA 6/2/15)
    '
    Public Sub GASTCPC_Sutton(ByRef IRESTYP As Int32, ByRef G As Single, ByRef GHC As Single, ByRef YN2 As Single, ByRef YCO2 As Single, ByRef YH2S As Single, ByRef TC As Single, _
                       ByRef PC As Single, ByRef GASMW As Single)

        Dim TCHC, PCHC, CWA As Single

        If G <> GHC Then
            If G <> 0.0 And GHC = 0.0 Then
                GHC = (G - 0.9672 * YN2 - 1.5195 * YCO2 - 1.1765 * YH2S) / (1.0 - YN2 - YCO2 - YH2S)
                GASMW = 28.96 * G
            ElseIf G = 0.0 And GHC <> 0.0 Then
                G = GHC * (1.0 - YN2 - YCO2 - YH2S) + (0.9672 * YN2 + 1.5195 * YCO2 + 1.1765 * YH2S)
                GASMW = 28.96 * G
            End If
        End If

        If IRESTYP = 0 Then
            TCHC = 187.0 + 330.0 * GHC - 71.5 * GHC ^ 2.0
            PCHC = 706.0 - 51.7 * GHC - 11.1 * GHC ^ 2.0
        End If

        If IRESTYP = 1 Then
            TCHC = 168.0 + 325.0 * GHC - 12.5 * GHC ^ 2.0
            PCHC = 677.0 - 15.0 * GHC - 37.5 * GHC ^ 2.0
        End If

        TC = (1.0 - YN2 - YCO2 - YH2S) * TCHC + 227.3 * YN2 + 547.6 * YCO2 + 672.4 * YH2S
        PC = (1.0 - YN2 - YCO2 - YH2S) * PCHC + 493.0 * YN2 + 1071.0 * YCO2 + 1306.0 * YH2S

        If YH2S <> 0.0 Then
            CWA = 120 * ((YCO2 + YH2S) ^ 0.9 - (YCO2 + YH2S) ^ 1.6) + 15.0 * (YCO2 ^ 0.5 - YH2S ^ 4.0)
            TC = TC - CWA
            PC = (PC * TC) / ((TC + CWA) + YH2S * (1.0 - YH2S) * CWA)
        End If

    End Sub
    '
    '---CALCULATE WATER PROPERTIES FROM CORRELATIONS
    '
    Public Sub WPVT(ByRef WTSOLIDS As Single, ByRef IERR As Int32)
        Dim strRow() As String
        Dim I, IPVTW, ii As Int32
        Dim A, B, C, X As Single
        Dim SOLIDS, MUW1ATM, VWTFAC, VWPFAC As Single
     
        IERR = 0
        '       STDTMP = STDTMP +459.67
        '       RESTEMP = RESTEMP +459.67
        '
        '---Formation volume factor of water
        '
        VWTFAC = 0.000133391 * RESTEMP + 0.000000550654 * RESTEMP ^ 2.0 - 0.010001

        For I = 1 To NPVTPRS
            VWPFAC = (-0.00000000195301 * PVTPRS(I) * RESTEMP) - (0.000000000000172834 * RESTEMP * PVTPRS(I) ^ 2.0) - (0.000000358922 * PVTPRS(I)) - (0.000000000225341 * PVTPRS(I) ^ 2.0)
            BW(I) = (1.0 + VWPFAC) * (1.0 + VWTFAC)
        Next I
        '
        '---Density of water at standard conditions
        '
        STDENWAT = 62.368 + 0.438603 * WTSOLIDS + 0.00160074 * WTSOLIDS ^ 2.0
        '
        '---Solids in mg/liter
        '
        SOLIDS = WTSOLIDS * 10000.0
        SOLIDS = SOLIDS * STDENWAT
        '
        '---Amount of gas dissolved in water
        '
        X = (-0.0840655 * WTSOLIDS * RESTEMP ^ (-0.285854))
        A = (8.15839 - 0.0612265 * RESTEMP + 0.000191663 * RESTEMP ^ 2.0 - 0.00000021654 * RESTEMP ^ 3.0)
        B = (0.0101021 - 0.0000744241 * RESTEMP + 0.000000305553 * RESTEMP ^ 2.0 - 0.000000000294883 * RESTEMP ^ 3.0)
        C = (-0.0000001) * (9.02505 - 0.130237 * RESTEMP + 0.000853425 * RESTEMP ^ 2.0 - 0.00000234122 * RESTEMP ^ 3.0 + 0.00000000237049 * RESTEMP ^ 4.0)

        For I = 1 To NPVTPRS     
            RSW(I) = A + B * PVTPRS(I) + C * PVTPRS(I) ^ 2.0
            RSW(I) = RSW(I) * 10.0 ^ X
        Next I
        '
        '---Water compressibility above bubble point
        '
        For I = 1 To NPVTPRS
            CW(I) = 1.0 / (7.033 * PVTPRS(I) + 0.5415 * (SOLIDS * STDENWAT / 16018.46) - 537.0 * RESTEMP + 403200.0)
            '            CW(I) = 1.0 / (7.033 * PVTPRS(I) + 0.5415 * SOLIDS - 537.0 * RESTEMP + 403200.0)
        Next I
        '
        '---Insitu water density
        '
        STDENGAS = STDPRS * 28.96 * 0.6 / ((STDTMP + 460.67) * 10.73)

        For I = 1 To NPVTPRS
            RHOW(I) = (STDENWAT + RSW(I) * STDENGAS) / BW(I)
        Next I
        '
        '---Water viscosity
        '
        A = (109.574 - 8.40564 * WTSOLIDS + 0.31331 * WTSOLIDS ^ 2.0 + 0.00872213 * WTSOLIDS ^ 3.0)
        B = (1.12166 - 0.0263951 * WTSOLIDS + 0.000679461 * WTSOLIDS ^ 2.0 + 0.0000547119 * WTSOLIDS ^ 3.0 - 0.00000155586 * WTSOLIDS ^ 4.0)
        MUW1ATM = A * RESTEMP ^ (-B)

        For I = 1 To NPVTPRS
            MUW(I) = MUW1ATM * (0.9994 + 0.000040295 * PVTPRS(I) + 0.0000000031062 * PVTPRS(I) ^ 2.0)
        Next I
        '
        '---Set up eclipse values
        '
        IPVTW = 1

        If NPVTPRS > 1 Then IPVTW = NPVTPRS / 2.0
        PWREF = PVTPRS(IPVTW)
        BWREF = BW(IPVTW)
        CWREF = CW(IPVTW)
        MUWREF = MUW(IPVTW)

        If IPVTW = 1 Then
            DMUWDP = 0.0
        Else
            DMUWDP = (MUW(IPVTW + 1) - MUW(IPVTW - 1)) / (PVTPRS(IPVTW + 1) - PVTPRS(IPVTW - 1))
        End If
        '
        '---Display water properties
        '
        frmStart.txtWPrtemp.Text = RESTEMP.ToString("F1")
        frmStart.txtWPweight.Text = WTSOLIDS.ToString("F1")
        frmStart.txtWPdiss.Text = (10000 * WTSOLIDS).ToString("F0")
        frmStart.txtWPtwd.Text = STDENWAT.ToString("F3")

        gridWP.DeleteAll()
        For ii = 1 To NPVTPRS
            strRow = New String() {PVTPRS(ii).ToString("F1"), BW(ii).ToString("F4"), RSW(ii).ToString("F1"), MUW(ii).ToString("F3"), CW(ii).ToString("F7"), _
                                   RHOW(ii).ToString("F1")}
            gridWP.AddRow(strRow)
        Next
    End Sub
    '
    '---Calc Rock Properties
    '
    Public Sub CalcRock()
        Dim K As Integer, i As Integer
        Dim A As Single, B As Single, C As Single, D As Single, DIV As Single
        Dim strRow() As String

        'Dim pvtSWet(15) As Single       'Output saturation divisions (wet)
        'Dim pvtSNWet(15) As Single      'Output saturation divisions (non-wet)
        'Dim NWet As Integer             'Size of output array

        'Dim pvtSD(15) As Single         'Satuation
        'Dim pvtKROil(15) As Single      'Permeability - Oil
        'Dim pvtKRWat(15) As Single      'Permeability - Water
        'Dim pvtKRGas(15) As Single      'Permeability - Gas
        '
        '---Init
        '
        For K = 1 To 15
            pvtKROil(K) = 0
            pvtKRGas(K) = 0
            pvtKRWat(K) = 0
        Next K
        pvtSWet(1) = SWC
        pvtSWet(15) = 1.0#
        '
        '---Oil-Water system (drainage curve)
        '
        If optRockFluids = SYS_OILWATER And optRockDisplace = DRAINAGE Then
            pvtSWet(8) = (pvtSWet(1) + pvtSWet(15)) / 2.0#
            DIV = (pvtSWet(15) - pvtSWet(1)) / 14.0#
            DIV = DIV / 2.0#
            pvtSWet(2) = pvtSWet(1) + DIV
            pvtSWet(3) = pvtSWet(2) + DIV
            pvtSWet(14) = pvtSWet(15) - DIV
            pvtSWet(13) = pvtSWet(14) - DIV
            DIV = (pvtSWet(13) - pvtSWet(3)) / 10.0#
            For K = 4 To 12
                pvtSWet(K) = pvtSWet(K - 1) + DIV
            Next K
        End If
        '
        '---Oil-Water system (Imbibition curve)
        '
        If optRockFluids = SYS_OILWATER And optRockDisplace = IMBIBITION Then
            pvtSWet(14) = 1.0# - SORW
            DIV = (pvtSWet(14) - pvtSWet(1)) / 13.0#
            DIV = DIV / 2.0#
            pvtSWet(2) = pvtSWet(1) + DIV
            pvtSWet(3) = pvtSWet(2) + DIV
            pvtSWet(13) = pvtSWet(14) - DIV
            pvtSWet(12) = pvtSWet(13) - DIV
            DIV = (pvtSWet(12) - pvtSWet(3)) / 9.0#
            For K = 4 To 11
                pvtSWet(K) = pvtSWet(K - 1) + DIV
            Next K
        End If
        '
        '---Gas-Oil system
        '
        If optRockFluids = SYS_GASOIL Then
            pvtSWet(14) = 1.0# - SGCrit
            pvtSWet(2) = pvtSWet(1) + SORG
            DIV = (pvtSWet(14) - pvtSWet(2)) / 12.0#
            DIV = DIV / 2.0#
            pvtSWet(3) = pvtSWet(2) + DIV
            pvtSWet(4) = pvtSWet(3) + DIV
            pvtSWet(13) = pvtSWet(14) - DIV
            pvtSWet(12) = pvtSWet(13) - DIV
            DIV = (pvtSWet(12) - pvtSWet(3)) / 8.0#
            For K = 5 To 11
                pvtSWet(K) = pvtSWet(K - 1) + DIV
            Next K
        End If

        For K = 1 To 15
            pvtSNWet(K) = 1.0# - pvtSWet(K)
        Next K
        '
        '---Define relative permeability characteristics
        '
        '---Oil-Water system (Drainage - Corey)
        '
        If optRockFluids = SYS_OILWATER And optRockDisplace = DRAINAGE And optRockCorr = COREY Then
            For K = 1 To 15
                pvtSD(K) = pvtSNWet(K) / (1.0# - pvtSWet(1))
                pvtKROil(K) = (pvtSD(K) ^ CoreyK) * (2.0# - pvtSD(K))
                pvtKRWat(K) = (1.0# - pvtSD(K)) ^ CoreyN
            Next K
        End If
        '
        '---Oil-Water system (Imbibition - Corey)
        '
        If optRockFluids = SYS_OILWATER And optRockDisplace = IMBIBITION And optRockCorr = COREY Then
            For i = 1 To 15
                pvtSD(i) = (pvtSWet(i) - SWC) / (1.0# - SWC - SORW)
                If pvtSD(i) > 1.0# Then pvtSD(i) = 1.0#
                pvtKROil(i) = (1.0# - pvtSD(i)) ^ CoreyN
            Next i

            For i = 1 To 15
                pvtSD(i) = (pvtSWet(i) - SWC) / (1.0# - SWC)
                pvtKRWat(i) = (pvtSD(i) ^ CoreyK) * (2.0# - pvtSD(i))
            Next i
        End If
        '
        '---Oil-Water system (Honarpour krw-Water and Oil-Water Wet - Sandstone)
        '
        If optRockFluids = SYS_OILWATER And optRockDisplace = IMBIBITION And optRockCorr = HONARPOUR Then
            If optRockLith = SANDSTONE Then

                If optRockWet = WATERWET Then               'Water Wet
                    A = 1.0# - SWC - SORW
                    For i = 1 To 14
                        B = pvtSWet(i) - SWC
                        C = pvtSWet(i) - SORW
                        pvtKRWat(i) = (0.035388 * B / A) - 0.010874 * (C / A) ^ 2.9
                        pvtKRWat(i) = pvtKRWat(i) + 0.56556 * B * pvtSWet(i) ^ 3.6
                    Next i
                    pvtKRWat(15) = 1.0#
                    '
                    '---Honarpour krw-Water and Oil - Oil/Intermediate wet - Sandstone
                    '
                Else
                    A = 1.0# - SWC
                    B = 1.0# - SWC - SORW
                    For i = 1 To 14
                        C = pvtSWet(i) - SWC
                        D = pvtSWet(i) - SORW
                        pvtKRWat(i) = 1.5814 * (C / A) ^ 1.91 - 0.58617 * (D / B) * C
                        pvtKRWat(i) = pvtKRWat(i) - 1.2848 * HRPoro * A * C
                    Next i
                    pvtKRWat(15) = 1.0#
                End If
                '
                '---Honarpour KRO - Water and Oil - Oil and Water - Sandstone
                '
                For i = 1 To 14
                    pvtKROil(i) = 0.76067 * ((((pvtSNWet(i) / (1.0# - SWC)) - SORW) / _
                                 (1.0# - SORW)) ^ 1.8) * (((pvtSNWet(i) - SORW) / _
                                 (1.0# - SWC - SORW)) ^ 2.0#) + 2.6318 * HRPoro * _
                                 (1.0# - SORW) * (pvtSNWet(i) - SORW)
                Next i
                pvtKROil(15) = 0.0#
            End If
            '
            '---Honarpour krw - Oil and Water - Water wet - Limestone
            '
            If optRockLith = LIMESTONE Then

                If optRockWet = WATERWET Then               'Water Wet
                    For i = 1 To 14
                        B = pvtSWet(i) - SWC
                        pvtKRWat(i) = 0.0020525 * B / (HRPoro ^ 2.15) - 0.051371 * B * _
                                      (1.0# / HRAir) ^ 0.43
                    Next i
                    pvtKRWat(15) = 1.0#
                    '
                    '---Honarpour krw - Oil and Water - Oil wet - Limestone
                    '
                Else                                 'Oil Wet
                    A = 1.0# - SWC
                    B = 1.0# - SWC - SORW
                    For i = 1 To 14
                        C = pvtSWet(i) - SWC
                        D = pvtSWet(i) - SORW
                        pvtKRWat(i) = (0.29986 * C / A) - 0.32797 * ((D / B) ^ 2.0#) * C
                        pvtKRWat(i) = pvtKRWat(i) + 0.413259 * (C / B) ^ 4.0#
                    Next i
                    pvtKRWat(15) = 1.0#
                End If
                '
                '---Hoonarpour kro - Oil and Water - Oil and Water wet - Liestone
                '
                For i = 1 To 14
                    A = pvtSNWet(i) - SORW
                    pvtKROil(i) = (1.2624 * A / (1.0# - SORW)) * (A / (1.0# - SWC - SORW)) ^ 2.0#
                Next i
                pvtKROil(15) = 0.0#
            End If
        End If
        '
        '---Gas-Oil system (Drainage - Corey)
        '
        If optRockFluids = SYS_GASOIL And optRockDisplace = DRAINAGE And optRockCorr = COREY Then
            For i = 1 To 15
                pvtSD(i) = (pvtSNWet(i) - SGCrit) / (1.0# - SWC)
                If pvtSD(i) < 0.0# Then pvtSD(i) = 0.0#
                pvtKRGas(i) = (pvtSD(i) ^ CoreyK) * (2.0# - pvtSD(i))
                pvtKROil(i) = (1.0# - pvtSD(i)) ^ CoreyN
            Next i
        End If
        '
        '---Gas-Oil system (Drainage - Corey)
        '
        If optRockFluids = SYS_GASOIL And optRockDisplace = IMBIBITION And optRockCorr = COREY Then
            For i = 1 To 15
                pvtSD(i) = (pvtSNWet(i) - SGCrit) / (1.0# - SWC - SORG)
                If pvtSD(i) < 0.0# Then pvtSD(i) = 0.0#
                If pvtSD(i) > 1.0# Then pvtSD(i) = 1.0#
                pvtKROil(i) = (1.0# - pvtSD(i)) ^ CoreyN
            Next i

            For i = 1 To 15
                pvtSD(i) = (pvtSNWet(i) - SGCrit) / (1.0# - SWC)
                pvtKRGas(i) = (pvtSD(i) ^ CoreyK) * (2.0# - pvtSD(i))
            Next i
        End If
        '
        '--- Honarpour kro- Oil and gas – oil and water wet- Sandstone
        '
        If optRockFluids = SYS_GASOIL And optRockDisplace = IMBIBITION And optRockCorr = HONARPOUR Then

            If optRockLith = SANDSTONE Then
                pvtKROil(1) = 0.0#
                pvtKRGas(1) = 1.0#
                For i = 2 To 15
                    pvtKROil(i) = 0.98372 * (((pvtSWet(i) - SWC) / (1.0# - SWC)) ^ 4.0#) * _
                                  (((pvtSWet(i) - SWC) - SORG) / (1.0# - SWC - SORG)) ^ 2.0#
                    '
                    '---Honarpour krg - Oil and Gas - oil and water wet Sandstone
                    '
                    pvtKRGas(i) = 1.1072 * ((pvtSNWet(i) - SGCrit) / (1.0# - SWC)) ^ 2.0# * _
                                  HRKrg + 2.7794 * (SORG * (pvtSNWet(i) - SGCrit)) * _
                                  HRKrg / (1.0# - SWC)
                Next i
                '
                '---Honarpour kro - Oil and Gas - oil and water wet Limestone
                '
            Else
                pvtKROil(1) = 0.0#
                pvtKRGas(1) = 1.0#
                For i = 2 To 15
                    pvtKROil(i) = 0.93752 * ((pvtSWet(i) - SWC) / (1.0# - SWC)) ^ 4.0# * _
                                  (((pvtSWet(i) - SWC) - SORG) / (1.0# - SWC - SORG)) ^ 2.0#
                    '
                    '---Honarpour kro - Oil and Gas - Oil and water wet - Limestone
                    '
                    pvtKRGas(i) = 1.8655 * (pvtSNWet(i) - SGCrit) * pvtSNWet(i) * HRKrg / _
                                  (1.0# - SWC) + (8.0053 * (pvtSNWet(i) - SGCrit) * _
                                  SORG ^ 2.0#) / (1.0# - SWC) - 0.2589 * (pvtSNWet(i) - _
                                  SGCrit) * (((1.0# - SWC - SORG - SGCrit) / (1.0# - SWC)) _
                                  ^ 2.0#) * ((HRAir / HRPoro) ^ 0.5) * _
                                  (1.0# - (1.0# - SWC - SORG - SGCrit) / (1.0# - SWC)) ^ 2.0#
                Next i
            End If
        End If
        '
        '---Apply known limits
        '
        NWet = 15
        For i = 1 To NWet
            If pvtKROil(i) < 0.0# Then pvtKROil(i) = 0.0#
            If pvtKROil(i) > 1.0# Then pvtKROil(i) = 1.0#
            If pvtKRWat(i) < 0.0# Then pvtKRWat(i) = 0.0#
            If pvtKRWat(i) > 1.0# Then pvtKRWat(i) = 1.0#
            If pvtKRGas(i) < 0.0# Then pvtKRGas(i) = 0.0#
            If pvtKRGas(i) > 1.0# Then pvtKRGas(i) = 1.0#
        Next i
        '
        '---Display Rock properties
        '
        frmStart.txtDispOut.Text = "Drainage"
        If optRockDisplace = 1 Then frmStart.txtDispOut.Text = "Imbibition"

        frmStart.txtLithOut.Text = "N/A"
        If optRockCorr = 1 Then
            frmStart.txtLithOut.Text = "Sandstone"
            If optRockLith = 1 Then frmStart.txtLithOut.Text = "Limestone"
        End If

        frmStart.txtCORROut.Text = "Corey"
        If optRockCorr = 1 Then frmStart.txtCORROut.Text = "Honarpour"
        '
        '---Oil-Water system
        '
        If optRockFluids = 0 Then
            frmStart.lblRockProp.Text = "Oil/Water System"

            gridRP.GridInit(frmStart.dgvRock, 5, bacCOLOR, selCOLOR, 30, False)
            gridRP.ColHeader(0, 150, "Water", aliRC)
            gridRP.ColHeader(1, 150, "Oil", aliRC)
            gridRP.ColHeader(2, 10, " ", aliRC)
            gridRP.ColHeader(3, 150, "Water", aliRC)
            gridRP.ColHeader(4, 150, "Oil", aliRC)

            For Each dgvCOL As DataGridViewColumn In frmStart.dgvRock.Columns
                dgvCOL.SortMode = DataGridViewColumnSortMode.NotSortable
            Next

            gridRP.DeleteAll()
            For ii = 1 To NWet
                strRow = New String() {pvtSWet(ii).ToString("F2"), pvtSNWet(ii).ToString("F2"), " ", pvtKRWat(ii).ToString("F4"), pvtKROil(ii).ToString("F4")}
                gridRP.AddRow(strRow)
            Next
            frmStart.lblRockSat.Left = frmStart.dgvRock.Left + 125
            frmStart.lblRockPerm.Left = frmStart.dgvRock.Left + 350
        End If
        '
        '---Gas-Oil system
        '
        If optRockFluids = 1 Then
            frmStart.lblRockProp.Text = "Gas/Oil System"

            gridRP.GridInit(frmStart.dgvRock, 6, bacCOLOR, selCOLOR, 30, False)
            gridRP.ColHeader(0, 125, "Gas", aliRC)
            gridRP.ColHeader(1, 125, "Soil", aliRC)
            gridRP.ColHeader(2, 125, "Swater", aliRC)
            gridRP.ColHeader(3, 10, " ", aliRC)
            gridRP.ColHeader(4, 125, "Gas", aliRC)
            gridRP.ColHeader(5, 125, "Oil", aliRC)

            For Each dgvCOL As DataGridViewColumn In frmStart.dgvRock.Columns
                dgvCOL.SortMode = DataGridViewColumnSortMode.NotSortable
            Next

            gridRP.DeleteAll()
            For ii = 1 To NWet
                strRow = New String() {pvtSNWet(ii).ToString("F4"), pvtSWet(ii).ToString("F2"), SWC.ToString("F2"), " ", pvtKRGas(ii).ToString("F4"), pvtKROil(ii).ToString("F4")}
                gridRP.AddRow(strRow)
            Next
            frmStart.lblRockSat.Left = frmStart.dgvRock.Left + 175
            frmStart.lblRockPerm.Left = frmStart.dgvRock.Left + 400
        End If
        '
        '---Newman compressibility
        '
        Select Case optRockLith
            Case 0
                NewmanCF = Sand_A / ((1.0 + Sand_B * Sand_C * NewmanPor) ^ (1 / Sand_B))      'Sandstone

            Case 1
                NewmanCF = Lime_A / ((1.0 + Lime_B * Lime_C * NewmanPor) ^ (1 / Lime_B))      'Limestone

        End Select

        frmStart.txtCompressibility.Text = NewmanCF.ToString("E7")

    End Sub
End Module
