﻿Module modP3_old
    Private Const PRESSURE_RANGE = 0
    Private Const DISCRETE_VALUES = 1
    Private Const GRAVITY_INERTS = 0
    Private Const COMPOSITION = 1
    Private Const RSI_ENTERED = 1
    Private Const PBP_ENTERED = 2

    Private Const SYS_OILWATER = 0
    Private Const SYS_GASOIL = 1
    Private Const DRAINAGE = 0
    Private Const IMBIBITION = 1
    Private Const SANDSTONE = 0
    Private Const LIMESTONE = 1
    Private Const WATERWET = 0
    Private Const OILWET = 1
    Private Const COREY = 0
    Private Const HONARPOUR = 1

    Private Const Lime_A = 0.853531
    Private Const Lime_B = 1.07538
    Private Const Lime_C = 2303040
    Private Const Sand_A = 0.00009732
    Private Const Sand_B = 0.699993
    Private Const Sand_C = 79.8181
    '
    '---Calc Rock Properties
    '
    Private Sub CalcRock()
        '    Public Sub CalcRock()
        Dim K As Integer, i As Integer
        Dim A As Single, B As Single, C As Single, D As Single, DIV As Single
        Dim strRow() As String

        'Dim pvtSWet(15) As Single       'Output saturation divisions (wet)
        'Dim pvtSNWet(15) As Single      'Output saturation divisions (non-wet)
        'Dim NWet As Integer             'Size of output array

        'Dim pvtSD(15) As Single         'Satuation
        'Dim pvtKROil(15) As Single      'Permeability - Oil
        'Dim pvtKRWat(15) As Single      'Permeability - Water
        'Dim pvtKRGas(15) As Single      'Permeability - Gas
        '
        '---Init
        '
        For K = 1 To 15
            pvtKROil(K) = 0
            pvtKRGas(K) = 0
            pvtKRWat(K) = 0
        Next K
        pvtSWet(1) = SWC
        pvtSWet(15) = 1.0#
        '
        '---Oil-Water system (drainage curve)
        '
        If optRockFluids = SYS_OILWATER And optRockDisplace = DRAINAGE Then
            pvtSWet(8) = (pvtSWet(1) + pvtSWet(15)) / 2.0#
            DIV = (pvtSWet(15) - pvtSWet(1)) / 14.0#
            DIV = DIV / 2.0#
            pvtSWet(2) = pvtSWet(1) + DIV
            pvtSWet(3) = pvtSWet(2) + DIV
            pvtSWet(14) = pvtSWet(15) - DIV
            pvtSWet(13) = pvtSWet(14) - DIV
            DIV = (pvtSWet(13) - pvtSWet(3)) / 10.0#
            For K = 4 To 12
                pvtSWet(K) = pvtSWet(K - 1) + DIV
            Next K
        End If
        '
        '---Oil-Water system (Imbibition curve)
        '
        If optRockFluids = SYS_OILWATER And optRockDisplace = IMBIBITION Then
            pvtSWet(14) = 1.0# - SORW
            DIV = (pvtSWet(14) - pvtSWet(1)) / 13.0#
            DIV = DIV / 2.0#
            pvtSWet(2) = pvtSWet(1) + DIV
            pvtSWet(3) = pvtSWet(2) + DIV
            pvtSWet(13) = pvtSWet(14) - DIV
            pvtSWet(12) = pvtSWet(13) - DIV
            DIV = (pvtSWet(12) - pvtSWet(3)) / 9.0#
            For K = 4 To 11
                pvtSWet(K) = pvtSWet(K - 1) + DIV
            Next K
        End If
        '
        '---Gas-Oil system
        '
        If optRockFluids = SYS_GASOIL Then
            pvtSWet(14) = 1.0# - SGCrit
            pvtSWet(2) = pvtSWet(1) + SORG
            DIV = (pvtSWet(14) - pvtSWet(2)) / 12.0#
            DIV = DIV / 2.0#
            pvtSWet(3) = pvtSWet(2) + DIV
            pvtSWet(4) = pvtSWet(3) + DIV
            pvtSWet(13) = pvtSWet(14) - DIV
            pvtSWet(12) = pvtSWet(13) - DIV
            DIV = (pvtSWet(12) - pvtSWet(3)) / 8.0#
            For K = 5 To 11
                pvtSWet(K) = pvtSWet(K - 1) + DIV
            Next K
        End If

        For K = 1 To 15
            pvtSNWet(K) = 1.0# - pvtSWet(K)
        Next K
        '
        '---Define relative permeability characteristics
        '
        '---Oil-Water system (Drainage - Corey)
        '
        If optRockFluids = SYS_OILWATER And optRockDisplace = DRAINAGE And optRockCorr = COREY Then
            For K = 1 To 15
                pvtSD(K) = pvtSNWet(K) / (1.0# - pvtSWet(1))
                pvtKROil(K) = (pvtSD(K) ^ CoreyK) * (2.0# - pvtSD(K))
                pvtKRWat(K) = (1.0# - pvtSD(K)) ^ CoreyN
            Next K
        End If
        '
        '---Oil-Water system (Imbibition - Corey)
        '
        If optRockFluids = SYS_OILWATER And optRockDisplace = IMBIBITION And optRockCorr = COREY Then
            For i = 1 To 15
                pvtSD(i) = (pvtSWet(i) - SWC) / (1.0# - SWC - SORW)
                If pvtSD(i) > 1.0# Then pvtSD(i) = 1.0#
                pvtKROil(i) = (1.0# - pvtSD(i)) ^ CoreyN
            Next i

            For i = 1 To 15
                pvtSD(i) = (pvtSWet(i) - SWC) / (1.0# - SWC)
                pvtKRWat(i) = (pvtSD(i) ^ CoreyK) * (2.0# - pvtSD(i))
            Next i
        End If
        '
        '---Oil-Water system (Imbibition - Honarpour - Sandstone)
        '
        If optRockFluids = SYS_OILWATER And optRockDisplace = IMBIBITION And optRockCorr = HONARPOUR Then
            If optRockLith = SANDSTONE Then

                If optRockWet = WATERWET Then               'Water Wet
                    A = 1.0# - SWC - SORW
                    For i = 1 To 14
                        B = pvtSWet(i) - SWC
                        pvtKRWat(i) = (0.035388 * B / A) - 0.010874 * (B / A) ^ 2.9
                        pvtKRWat(i) = pvtKRWat(i) + 0.56556 * B * pvtSWet(i) ^ 3.6
                    Next i
                    pvtKRWat(15) = 1.0#

                Else                                 'Oil Wet
                    A = 1.0# - SWC
                    B = 1.0# - SWC - SORW
                    For i = 1 To 14
                        C = pvtSWet(i) - SWC
                        D = pvtSWet(i) - SORW
                        pvtKRWat(i) = 1.5814 * (C / A) ^ 1.91 - 0.58617 * (D / B) * C
                        pvtKRWat(i) = pvtKRWat(i) - 1.2848 * HRPoro * A * C
                    Next i
                    pvtKRWat(15) = 1.0#
                End If

                For i = 1 To 14
                    pvtKROil(i) = 0.76067 * ((((pvtSNWet(i) / (1.0# - SWC)) - SORW) / _
                                 (1.0# - SORW)) ^ 1.8) * (((pvtSNWet(i) - SORW) / _
                                 (1.0# - SWC - SORW)) ^ 2.0#) + 2.6318 * HRPoro * _
                                 (1.0# - SORW) * (pvtSNWet(i) - SORW)
                Next i
                pvtKROil(15) = 0.0#
            End If
            '
            '---Oil-Water system (Imbibition - Honarpour - Limestone)
            '
            If optRockLith = LIMESTONE Then

                If optRockWet = WATERWET Then               'Water Wet
                    For i = 1 To 14
                        B = pvtSWet(i) - SWC
                        pvtKRWat(i) = 0.0020525 * B / (HRPoro ^ 2.15) - 0.051371 * B * _
                                      (1.0# / HRAir) ^ 0.43
                    Next i
                    pvtKRWat(15) = 1.0#

                Else                                 'Oil Wet
                    A = 1.0# - SWC
                    B = 1.0# - SWC - SORW
                    For i = 1 To 14
                        C = pvtSWet(i) - SWC
                        D = pvtSWet(i) - SORW
                        pvtKRWat(i) = (0.29986 * C / A) - 0.32797 * ((D / B) ^ 2.0#) * C
                        pvtKRWat(i) = pvtKRWat(i) + 0.413259 * (C / B) ^ 4.0#
                    Next i
                    pvtKRWat(15) = 1.0#
                End If

                For i = 1 To 14
                    A = pvtSNWet(i) - SORW
                    pvtKROil(i) = (1.2624 * A / (1.0# - SORW)) * (A / (1.0# - SWC - SORW)) ^ 2.0#
                Next i
                pvtKROil(15) = 0.0#
            End If
        End If
        '
        '---Gas-Oil system (Drainage - Corey)
        '
        If optRockFluids = SYS_GASOIL And optRockDisplace = DRAINAGE And optRockCorr = COREY Then
            For i = 1 To 15
                pvtSD(i) = (pvtSNWet(i) - SGCrit) / (1.0# - SWC)
                If pvtSD(i) < 0.0# Then pvtSD(i) = 0.0#
                pvtKRGas(i) = (pvtSD(i) ^ CoreyK) * (2.0# - pvtSD(i))
                pvtKROil(i) = (1.0# - pvtSD(i)) ^ CoreyN
            Next i
        End If
        '
        '---Gas-Oil system (Imbibition - Corey)
        '
        If optRockFluids = SYS_GASOIL And optRockDisplace = IMBIBITION And optRockCorr = COREY Then
            For i = 1 To 15
                pvtSD(i) = (pvtSNWet(i) - SGCrit) / (1.0# - SWC - SORG)
                If pvtSD(i) < 0.0# Then pvtSD(i) = 0.0#
                If pvtSD(i) > 1.0# Then pvtSD(i) = 1.0#
                pvtKROil(i) = (1.0# - pvtSD(i)) ^ CoreyN
            Next i

            For i = 1 To 15
                pvtSD(i) = (pvtSNWet(i) - SGCrit) / (1.0# - SWC)
                pvtKRGas(i) = (pvtSD(i) ^ CoreyK) * (2.0# - pvtSD(i))
            Next i
        End If
        '
        '---Gas-Oil system (Imbibition - Honarpour - Sandstone)
        '
        If optRockFluids = SYS_GASOIL And optRockDisplace = IMBIBITION And optRockCorr = HONARPOUR Then

            If optRockLith = SANDSTONE Then
                pvtKROil(1) = 0.0#
                pvtKRGas(1) = 1.0#
                For i = 2 To 15
                    pvtKROil(i) = 0.98372 * (((pvtSWet(i) - SWC) / (1.0# - SWC)) ^ 4.0#) * _
                                  (((pvtSWet(i) - SWC) - SORG) / (1.0# - SWC - SORG)) ^ 2.0#
                    pvtKRGas(i) = 1.1072 * ((pvtSNWet(i) - SGCrit) / (1.0# - SWC)) ^ 2.0# * _
                                  HRKrg + 2.7794 * (SORG * (pvtSNWet(i) - SGCrit)) * _
                                  HRKrg / (1.0# - SWC)
                Next i
                '
                '---Gas-Oil system (Imbibition - Honarpour - Limestone)
                '
            Else
                pvtKROil(1) = 0.0#
                pvtKRGas(1) = 1.0#
                For i = 2 To 15
                    pvtKROil(i) = 0.93752 * ((pvtSWet(i) - SWC) / (1.0# - SWC)) ^ 4.0# * _
                                  (((pvtSWet(i) - SWC) - SORG) / (1.0# - SWC - SORG)) ^ 2.0#
                    pvtKRGas(i) = 1.8655 * (pvtSNWet(i) - SGCrit) * pvtSNWet(i) * HRKrg / _
                                  (1.0# - SWC) + (8.0053 * (pvtSNWet(i) - SGCrit) * _
                                  SORG ^ 2.0#) / (1.0# - SWC) - 0.2589 * (pvtSNWet(i) - _
                                  SGCrit) * (((1.0# - SWC - SORG - SGCrit) / (1.0# - SWC)) _
                                  ^ 2.0#) * ((HRAir / HRPoro) ^ 0.5) * _
                                  (1.0# - (1.0# - SWC - SORG - SGCrit) / (1.0# - SWC)) ^ 2.0#
                Next i
            End If
        End If
        '
        '---Apply known limits
        '
        NWet = 15
        For i = 1 To NWet
            If pvtKROil(i) < 0.0# Then pvtKROil(i) = 0.0#
            If pvtKROil(i) > 1.0# Then pvtKROil(i) = 1.0#
            If pvtKRWat(i) < 0.0# Then pvtKRWat(i) = 0.0#
            If pvtKRWat(i) > 1.0# Then pvtKRWat(i) = 1.0#
            If pvtKRGas(i) < 0.0# Then pvtKRGas(i) = 0.0#
            If pvtKRGas(i) > 1.0# Then pvtKRGas(i) = 1.0#
        Next i
        '
        '---Display Rock properties
        '
        frmStart.txtDispOut.Text = "Drainage"
        If optRockDisplace = 1 Then frmStart.txtDispOut.Text = "Imbibition"

        frmStart.txtLithOut.Text = "N/A"
        If optRockCorr = 1 Then
            frmStart.txtLithOut.Text = "Sandstone"
            If optRockLith = 1 Then frmStart.txtLithOut.Text = "Limestone"
        End If

        frmStart.txtCORROut.Text = "Corey"
        If optRockCorr = 1 Then frmStart.txtCORROut.Text = "Honarpour"
        '
        '---Oil-Water system
        '
        If optRockFluids = 0 Then
            frmStart.lblRockProp.Text = "Oil/Water System"

            gridRP.GridInit(frmStart.dgvRock, 5, bacCOLOR, selCOLOR, 30, False)
            gridRP.ColHeader(0, 150, "Water", aliRC)
            gridRP.ColHeader(1, 150, "Oil", aliRC)
            gridRP.ColHeader(2, 10, " ", aliRC)
            gridRP.ColHeader(3, 150, "Water", aliRC)
            gridRP.ColHeader(4, 150, "Oil", aliRC)

            For Each dgvCOL As DataGridViewColumn In frmStart.dgvRock.Columns
                dgvCOL.SortMode = DataGridViewColumnSortMode.NotSortable
            Next

            gridRP.DeleteAll()
            For ii = 1 To NWet
                strRow = New String() {pvtSWet(ii).ToString("F2"), pvtSNWet(ii).ToString("F2"), " ", pvtKRWat(ii).ToString("F4"), pvtKROil(ii).ToString("F4")}
                gridRP.AddRow(strRow)
            Next
            frmStart.lblRockSat.Left = frmStart.dgvRock.Left + 125
            frmStart.lblRockPerm.Left = frmStart.dgvRock.Left + 350
        End If
        '
        '---Gas-Oil system
        '
        If optRockFluids = 1 Then
            frmStart.lblRockProp.Text = "Gas/Oil System"

            gridRP.GridInit(frmStart.dgvRock, 6, bacCOLOR, selCOLOR, 30, False)
            gridRP.ColHeader(0, 125, "Gas", aliRC)
            gridRP.ColHeader(1, 125, "Soil", aliRC)
            gridRP.ColHeader(2, 125, "Swater", aliRC)
            gridRP.ColHeader(3, 10, " ", aliRC)
            gridRP.ColHeader(4, 125, "Gas", aliRC)
            gridRP.ColHeader(5, 125, "Oil", aliRC)

            For Each dgvCOL As DataGridViewColumn In frmStart.dgvRock.Columns
                dgvCOL.SortMode = DataGridViewColumnSortMode.NotSortable
            Next

            gridRP.DeleteAll()
            For ii = 1 To NWet
                strRow = New String() {pvtSNWet(ii).ToString("F4"), pvtSWet(ii).ToString("F2"), SWC.ToString("F2"), " ", pvtKRGas(ii).ToString("F4"), pvtKROil(ii).ToString("F4")}
                gridRP.AddRow(strRow)
            Next
            frmStart.lblRockSat.Left = frmStart.dgvRock.Left + 175
            frmStart.lblRockPerm.Left = frmStart.dgvRock.Left + 400
        End If
        '
        '---Newman compressibility
        '
        Select Case optRockLith
            Case 0
                NewmanCF = Sand_A / ((1.0 + Sand_B * Sand_C * NewmanPor) ^ (1 / Sand_B))      'Sandstone

            Case 1
                NewmanCF = Lime_A / ((1.0 + Lime_B * Lime_C * NewmanPor) ^ (1 / Lime_B))      'Limestone

        End Select

        frmStart.txtCompressibility.Text = NewmanCF.ToString("E7")

    End Sub
End Module
