VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsPressureSpec"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private pvtPrRange As String               'Range Option - R=Range, I=User input
Private pvtPrMax As Single                 'Range: Max pressure
Private pvtPrIni As Single                 'Range: User entered bubble pt pressure
Private pvtPrMin As Single                 'Range: Min pressure

Private pvtPRS(1 To 20) As Single          'Pressure value array (up to 20)
Private pvtNPRS As Integer                 'Size of pressure array
Private pvtBPP As Single                   'Bubble point pressure (actual)
'
'---Class Initialize
'
Private Sub Class_Initialize()
  PrRange = "R"
  PrMax = 5000#
  PrIni = 0
  PrMin = 1000#
End Sub
'
'============================== Properties ====================================
'
'
'---Pressure range option
'
Public Property Get PrRange() As String
  PrRange = pvtPrRange
End Property

Public Property Let PrRange(ByVal vNewValue As String)
  If vNewValue <> pvtPrRange Then
    Select Case vNewValue
      'Case "R", "r": ??
      'Case "I", "i": ??
    End Select
    
    pvtPrRange = UCase(vNewValue)
  End If
End Property
'
'---Max pressure range
'
Public Property Get PrMax() As Single
  PrMax = pvtPrMax
End Property

Private Property Let PrMax(ByVal vNewValue As Single)
  pvtPrMax = vNewValue
End Property
'
'---Initial pressure range
'
Public Property Get PrIni() As Single
  PrIni = pvtPrIni
End Property

Private Property Let PrIni(ByVal vNewValue As Single)
  pvtPrIni = vNewValue
End Property
'
'---Min pressure range
'
Public Property Get PrMin() As Single
  PrMin = pvtPrMin
End Property

Private Property Let PrMin(ByVal vNewValue As Single)
  pvtPrMin = vNewValue
End Property
'
'---Size of pressure array
'
Public Property Get NPRS() As Integer
  NPRS = pvtNPRS
End Property

Private Property Let NPRS(ByVal vNewValue As Integer)
  pvtNPRS = vNewValue
End Property

'
'============================== Methods ====================================
'
'
'---Set Max pressure
'
Public Sub SetMaxP(pValue As String)
  Dim pBuf As String
  Dim temp As Single
  
  pBuf = pValue
  If Len(pBuf) = 0 Then pBuf = "0"
  
  On Error GoTo InvalidNum
  temp = CSng(pBuf)
  PrMax = temp
  Exit Sub
  
InvalidNum:
  Exit Sub
End Sub
'
'---Set Min pressure
'
Public Sub SetMinP(pValue As String)
  Dim pBuf As String
  Dim temp As Single
  
  pBuf = pValue
  If Len(pBuf) = 0 Then pBuf = "0"
  
  On Error GoTo InvalidNum
  temp = CSng(pBuf)
  PrMin = temp
  Exit Sub
  
InvalidNum:
  Exit Sub
End Sub
'
'---Set Init pressure
'
Public Sub SetIniP(pValue As String)
  Dim pBuf As String
  Dim temp As Single
  
  pBuf = pValue
  If Len(pBuf) = 0 Then pBuf = "0"
  
  On Error GoTo InvalidNum
  temp = CSng(pBuf)
  PrIni = temp
  Exit Sub
  
InvalidNum:
  Exit Sub
End Sub
'
'---Build pressure array based on Min, Max and Initial (bubble pt) pressure values.
'---Reduce array size if pressure increment falls below 50psi
'
Public Sub buildPRS()
  Dim pIncr As Single
  Dim pNext As Single
  Dim ii As Integer
  Dim iniFlag As Boolean
  
  NPRS = 16
  pIncr = Round((PrMax - PrMin) / (NPRS - 1), 0)
  If pIncr < 50# Then
    pIncr = 50#
    NPRS = CInt((PrMax - PrMin) / pIncr)
  End If
'
'---Build array.  Insert Initial pressure where appropriate
'
  iniFlag = False
  If PrIni > 0 And PrIni <> PrMax And PrIni <> PrMin Then iniFlag = True
  
  pNext = PrMin
  ii = 0
  Do
    ii = ii + 1
    pvtPRS(ii) = pNext
    If iniFlag And PrIni = pNext Then iniFlag = False
    
    If iniFlag And PrIni < pNext Then
      pvtPRS(ii) = PrIni
      ii = ii + 1
      iniFlag = False
    End If
    
    pNext = pNext + pIncr
  Loop While pNext < PrMax
'
'---Add Max Pressure
'
  ii = ii + 1
  pvtPRS(ii) = PrMax
  NPRS = ii
'
'---Check if initial pressure bracketed.  If not, extend array to include it
'
  If iniFlag Then
    pvtPRS(NPRS) = PrIni
    pvtPRS(NPRS + 1) = PrMax
    pvtPRS(NPRS + 2) = PrMax + 500#
    NPRS = NPRS + 2
  End If
    
End Sub

Public Function GetPRS(n As Integer) As Single
  Dim ii As Integer
  
  ii = n
  If ii < 1 Then ii = 1
  If ii > NPRS Then ii = NPRS
  
  GetPRS = pvtPRS(ii)

End Function
