VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsPrs"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private pvtPrRange As String               'Range Option - R=Range, I=User input
Private pvtPrMax As Single                 'Range: Max pressure
Private pvtPrIni As Single                 'Range: User entered bubble pt pressure
Private pvtPrMin As Single                 'Range: Min pressure

Private pvtPRSi(1 To 18) As Single         'Input Pressure value array (up to 18)
Private pvtNPRSi As Integer                'Size of input pressure array

Private pvtPRSo(1 To 30) As Single         'Output Pressure value array
Private pvtNPRSo As Integer                'Size of output pressure array
Private pvtBPPi As Integer                 'Index of bubble point pressure
'
'---Class Initialize
'
Private Sub Class_Initialize()
  PrRange = "R"
  PrMax = 5000#
  PrIni = 0
  PrMin = 1000#
  NPRSi = 0
  NPRSo = 0
End Sub
'
'============================== Properties ====================================
'
'
'---Pressure range option
'
Public Property Get PrRange() As String
  PrRange = pvtPrRange
End Property

Public Property Let PrRange(ByVal vNewValue As String)
  If vNewValue <> pvtPrRange Then
    Select Case vNewValue
      Case "R", "r":
      Case "I", "i": NPRSi = 0
    End Select
    
    pvtPrRange = UCase(vNewValue)
  End If
End Property
'
'---Max pressure range
'
Public Property Get PrMax() As Single
  PrMax = pvtPrMax
End Property

Private Property Let PrMax(ByVal vNewValue As Single)
  pvtPrMax = vNewValue
End Property
'
'---Initial pressure range
'
Public Property Get PrIni() As Single
  PrIni = pvtPrIni
End Property

Private Property Let PrIni(ByVal vNewValue As Single)
  pvtPrIni = vNewValue
End Property
'
'---Min pressure range
'
Public Property Get PrMin() As Single
  PrMin = pvtPrMin
End Property

Private Property Let PrMin(ByVal vNewValue As Single)
  pvtPrMin = vNewValue
End Property
'
'---Size of input pressure array
'
Public Property Get NPRSi() As Integer
  NPRSi = pvtNPRSi
End Property

Public Property Let NPRSi(ByVal vNewValue As Integer)
  pvtNPRSi = vNewValue
End Property
'
'---Size of output pressure array
'
Public Property Get NPRSo() As Integer
  NPRSo = pvtNPRSo
End Property

Public Property Let NPRSo(ByVal vNewValue As Integer)
  pvtNPRSo = vNewValue
End Property
'
'---Index of bubble point pressure
'
Public Property Get BPPi() As Integer
  BPPi = pvtBPPi
End Property

Public Property Let BPPi(ByVal vNewValue As Integer)
  pvtBPPi = vNewValue
End Property
'
'============================== Methods ====================================
'
'
'---Set Max pressure
'
Public Sub SetMaxP(pValue As String)
  PrMax = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set Min pressure
'
Public Sub SetMinP(pValue As String)
  PrMin = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set Init pressure
'
Public Sub SetIniP(pValue As String)
  PrIni = objUtil.Str2Nbr(pValue)
End Sub
'
'---Build output Pressure array.  Insert Bubble point pressure and sort
'
Public Sub BuildPRSo()
  Dim h As Integer
  Dim K As Integer
  Dim Temp As Single
'
'---Transfer input pressures to output
'
  NPRSo = NPRSi
  For K = 1 To NPRSi
    pvtPRSo(K) = pvtPRSi(K)
  Next K
'
'---Add Bubble point for Oil/Gas phase present
'
  If objFluid.PhaseOil Then
    NPRSo = NPRSo + 1
    pvtPRSo(NPRSo) = objOil.Pbp
  End If
'
'---Sort descending
'
  If NPRSo > 1 Then
    For h = 1 To NPRSo - 1
      For K = 1 To NPRSo - h
        If pvtPRSo(K + 1) > pvtPRSo(K) Then      'Asc (<), Desc (>)
          Temp = pvtPRSo(K)
          pvtPRSo(K) = pvtPRSo(K + 1)
          pvtPRSo(K + 1) = Temp
        End If
      Next K
    Next h
'
'---Remove any duplicate pressures
'
    K = 1
    Do
      If pvtPRSo(K + 1) = pvtPRSo(K) Then
        For h = K + 1 To NPRSo - 1
          pvtPRSo(h) = pvtPRSo(h + 1)
        Next h
        NPRSo = NPRSo - 1
      Else
        K = K + 1
      End If
    Loop Until K = NPRSo
  End If
'
'---Make sure there is at least one value less than Bubble point
'
  If objFluid.PhaseOil Then
    If pvtPRSo(NPRSo) = objOil.Pbp Then
      NPRSo = NPRSo + 1
      pvtPRSo(NPRSo) = Round(objOil.Pbp - 500#, 0)
      If pvtPRSo(NPRSo) < 10# Then pvtPRSo(NPRSo) = 10#
    End If
'
'---Make sure there is at least one value greater than Bubble point
'
    If pvtPRSo(1) = objOil.Pbp Then
      NPRSo = NPRSo + 1
      For K = NPRSo To 2 Step -1
        pvtPRSo(K) = pvtPRSo(K - 1)
      Next K
      pvtPRSo(1) = Round(objOil.Pbp + 500#, 0)
    End If
'
'---Note index of bubble point pressure
'
    For K = 1 To NPRSo
      If pvtPRSo(K) = objOil.Pbp Then
        BPPi = K
        Exit For
      End If
    Next K
  End If
  
End Sub
'
'---Add/Update/Remove Pressure array element
'
Public Function SetPRSi(pValue As String, N As Integer)
  Dim Temp As Single
  Dim ii As Integer
  
  Temp = objUtil.Str2Nbr(pValue)
  
  If Temp > 0 Then
    If N > NPRSi Then NPRSi = N
    pvtPRSi(N) = Temp
  Else
    If N <= NPRSi Then
      For ii = N To NPRSi
        pvtPRSi(ii) = pvtPRSi(ii + 1)
      Next ii
      NPRSi = NPRSi - 1
    End If
  End If
End Function
'
'---Build pressure array based on Min, Max and Initial (bubble pt) pressure values.
'---Reduce array size if pressure increment falls below 50psi
'
Public Sub BuildPRSi()
  Dim pIncr As Single
  Dim pNext As Single
  Dim ii As Integer
  Dim iniFlag As Boolean
  
  NPRSi = 16
  pIncr = Round((PrMax - PrMin) / (NPRSi - 1), 0)
  If pIncr < 50# Then
    pIncr = 50#
    NPRSi = CInt((PrMax - PrMin) / pIncr)
  End If
'
'---Build array.  Insert Initial pressure where appropriate
'
  iniFlag = False
  If PrIni > 0 And PrIni <> PrMax And PrIni <> PrMin Then iniFlag = True
  
  pNext = PrMin
  ii = 0
  Do
    ii = ii + 1
    pvtPRSi(ii) = pNext
    If iniFlag And PrIni = pNext Then iniFlag = False
    
    If iniFlag And PrIni < pNext Then
      pvtPRSi(ii) = PrIni
      ii = ii + 1
      pvtPRSi(ii) = pNext
      iniFlag = False
    End If
    
    pNext = pNext + pIncr
  Loop While pNext < PrMax
'
'---Add Max Pressure
'
  ii = ii + 1
  pvtPRSi(ii) = PrMax
  NPRSi = ii
'
'---Check if initial pressure bracketed.  If not, extend array to include it
'
  If iniFlag Then
    pvtPRSi(NPRSi) = PrIni
    pvtPRSi(NPRSi + 1) = PrMax
    pvtPRSi(NPRSi + 2) = PrMax + 500#
    NPRSi = NPRSi + 2
  End If
    
End Sub

Public Function GetPRSi(N As Integer) As Single
  Dim ii As Integer
  
  ii = N
  If ii < 1 Then ii = 1
  If ii > NPRSi Then ii = NPRSi
  
  GetPRSi = pvtPRSi(ii)

End Function

Public Function GetPRSo(N As Integer) As Single
  Dim ii As Integer
  
  ii = N
  If ii < 1 Then ii = 1
  If ii > NPRSo Then ii = NPRSo
  
  GetPRSo = pvtPRSo(ii)

End Function

