VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsRock"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Const SYS_OILWATER = 0
Private Const SYS_GASOIL = 1
Private Const DRAINAGE = 0
Private Const IMBIBITION = 1
Private Const SANDSTONE = 0
Private Const LIMESTONE = 1
Private Const WATERWET = 0
Private Const OILWET = 1
Private Const COREY = 0
Private Const HONARPOUR = 1


Private pvtFluids As Integer             '0=Oil-Water, 1=Gas-Oil
Private pvtDisplace As Integer           '0=Drainage, 1=Imbibition
Private pvtLith As Integer               '0=Sandstone, 1=Limestone
Private pvtWet As Integer                '0=Water Wet, 1=Oil Wet
Private pvtCORR As Integer               '0=Corey, 1=Honarpour

Private pvtCoreyK As Single              'Corey exponent (K)
Private pvtCoreyN As Single              'Corey exponent (N)

Private pvtHRPoro As Single              'Honarpour exponent (Porosity)
Private pvtHRAir As Single               'Honarpour exponent (Air permeability)
Private pvtHRKrg As Single               'Honarpour exponent (Krg)

Private pvtSWC As Single                 'Connate water
Private pvtSORW As Single                'Residual Oil to Water
Private pvtSORG As Single                'Residual Oil to Gas
Private pvtSGCrit As Single              'Critical Gas

Private pvtSWet(1 To 15) As Single       'Output saturation divisions (wet)
Private pvtSNWet(1 To 15) As Single      'Output saturation divisions (non-wet)
Private pvtNWet As Integer               'Size of output array

Private pvtSD(1 To 15) As Single         'Satuation
Private pvtKROil(1 To 15) As Single      'Permeability - Oil
Private pvtKRWat(1 To 15) As Single      'Permeability - Water
Private pvtKRGas(1 To 15) As Single      'Permeability - Gas
Private pvtGO As Boolean                 'Rock properties available (GO=TRUE)
'
'---Class Initialize
'
Private Sub Class_Initialize()
  Dim K As Integer
  
  Fluids = 0
  Displace = 0
  Lith = 0
  Wet = 0
  CORR = 0
  CoreyK = 3
  CoreyN = 4
  HRPoro = 0.25
  HRAir = 50
  HRKrg = 0.5
  SWC = 0.2
  SORW = 0.4
  SORG = 0.4
  SGCrit = 0.3
  
  NWet = 15
  GO = False
  
End Sub
'
'============================== Properties ====================================
'
'
'---Fluids type
'
Public Property Get Fluids() As Integer
  Fluids = pvtFluids
End Property

Public Property Let Fluids(ByVal vNewValue As Integer)
  pvtFluids = vNewValue
End Property
'
'---Drainage/Imbibition
'
Public Property Get Displace() As Integer
  Displace = pvtDisplace
End Property

Public Property Let Displace(ByVal vNewValue As Integer)
  pvtDisplace = vNewValue
End Property
'
'---Lithology
'
Public Property Get Lith() As Integer
  Lith = pvtLith
End Property

Public Property Let Lith(ByVal vNewValue As Integer)
  pvtLith = vNewValue
End Property
'
'---Wet option
'
Public Property Get Wet() As Integer
  Wet = pvtWet
End Property

Public Property Let Wet(ByVal vNewValue As Integer)
  pvtWet = vNewValue
End Property
'
'---Correlation
'
Public Property Get CORR() As Integer
  CORR = pvtCORR
End Property

Public Property Let CORR(ByVal vNewValue As Integer)
  pvtCORR = vNewValue
End Property
'
'---Corey K exponent
'
Public Property Get CoreyK() As Single
  CoreyK = pvtCoreyK
End Property

Private Property Let CoreyK(ByVal vNewValue As Single)
  pvtCoreyK = vNewValue
End Property
'
'---Corey N exponent
'
Public Property Get CoreyN() As Single
  CoreyN = pvtCoreyN
End Property

Private Property Let CoreyN(ByVal vNewValue As Single)
  pvtCoreyN = vNewValue
End Property
'
'---Honarpour porosity exponent
'
Public Property Get HRPoro() As Single
  HRPoro = pvtHRPoro
End Property

Private Property Let HRPoro(ByVal vNewValue As Single)
  pvtHRPoro = vNewValue
End Property
'
'---Honarpour air permeability exponent
'
Public Property Get HRAir() As Single
  HRAir = pvtHRAir
End Property

Private Property Let HRAir(ByVal vNewValue As Single)
  pvtHRAir = vNewValue
End Property
'
'---Honarpour Krg exponent
'
Public Property Get HRKrg() As Single
  HRKrg = pvtHRKrg
End Property

Private Property Let HRKrg(ByVal vNewValue As Single)
  pvtHRKrg = vNewValue
End Property
'
'---Connate water
'
Public Property Get SWC() As Single
  SWC = pvtSWC
End Property

Private Property Let SWC(ByVal vNewValue As Single)
  pvtSWC = vNewValue
End Property
'
'---Residual oil to water
'
Public Property Get SORW() As Single
  SORW = pvtSORW
End Property

Private Property Let SORW(ByVal vNewValue As Single)
  pvtSORW = vNewValue
End Property
'
'---Residual oil to gas
'
Public Property Get SORG() As Single
  SORG = pvtSORG
End Property

Private Property Let SORG(ByVal vNewValue As Single)
  pvtSORG = vNewValue
End Property
'
'---Critical gas
'
Public Property Get SGCrit() As Single
  SGCrit = pvtSGCrit
End Property

Private Property Let SGCrit(ByVal vNewValue As Single)
  pvtSGCrit = vNewValue
End Property
'
'---Rock output array size
'
Public Property Get NWet() As Integer
  NWet = pvtNWet
End Property

Private Property Let NWet(ByVal vNewValue As Integer)
  pvtNWet = vNewValue
End Property
'
'---Rock propertes calc'ed
'
Public Property Get GO() As Boolean
  GO = pvtGO
End Property

Private Property Let GO(ByVal vNewValue As Boolean)
  pvtGO = vNewValue
End Property
'
'============================== Methods ====================================
'
'
'---Set connate water
'
Public Sub SetSWC(pValue As String)
  SWC = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set residual oil to water
'
Public Sub SetSORW(pValue As String)
  SORW = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set residual oil to gas
'
Public Sub SetSORG(pValue As String)
  SORG = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set critical gas
'
Public Sub SetSGCrit(pValue As String)
  SGCrit = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set Corey K
'
Public Sub SetCoreyK(pValue As String)
  CoreyK = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set Corey N
'
Public Sub SetCoreyN(pValue As String)
  CoreyN = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set Honarpour porosity
'
Public Sub SetHRPoro(pValue As String)
  HRPoro = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set Honarpour air perm
'
Public Sub SetHRAir(pValue As String)
  HRAir = objUtil.Str2Nbr(pValue)
End Sub
'
'---Set Honarpour Krg
'
Public Sub SetHRKrg(pValue As String)
  HRKrg = objUtil.Str2Nbr(pValue)
End Sub
'
'---Return calculated value methods
'
Public Function GetSWet(N As Integer) As Single
  Dim ii As Integer
  
  ii = N
  If ii < 1 Then ii = 1
  If ii > NWet Then ii = NWet
  
  GetSWet = pvtSWet(ii)

End Function

Public Function GetSNWet(N As Integer) As Single
  Dim ii As Integer
  
  ii = N
  If ii < 1 Then ii = 1
  If ii > NWet Then ii = NWet
  
  GetSNWet = pvtSNWet(ii)

End Function

Public Function GetSD(N As Integer) As Single
  GetSD = pvtSD(N)
End Function

Public Function GetKROil(N As Integer) As Single
  GetKROil = pvtKROil(N)
End Function

Public Function GetKRWat(N As Integer) As Single
  GetKRWat = pvtKRWat(N)
End Function

Public Function GetKRGas(N As Integer) As Single
  GetKRGas = pvtKRGas(N)
End Function
'
'---Calc Oil Properties
'
Public Function CalcRock() As Integer
  Dim K As Integer, i As Integer
  Dim A As Single, B As Single, C As Single, D As Single, DIV As Single
  Dim CurX As Single
  Dim ResTf As Single
'
'---Init
'
  GO = False
  For K = 1 To 15
    pvtKROil(K) = 0
    pvtKRGas(K) = 0
    pvtKRWat(K) = 0
  Next K
  pvtSWet(1) = SWC
  pvtSWet(15) = 1#
'
'---Oil-Water system (drainage curve)
'
  If Fluids = SYS_OILWATER And Displace = DRAINAGE Then
    pvtSWet(8) = (pvtSWet(1) + pvtSWet(15)) / 2#
    DIV = (pvtSWet(15) - pvtSWet(1)) / 14#
    DIV = DIV / 2#
    pvtSWet(2) = pvtSWet(1) + DIV
    pvtSWet(3) = pvtSWet(2) + DIV
    pvtSWet(14) = pvtSWet(15) - DIV
    pvtSWet(13) = pvtSWet(14) - DIV
    DIV = (pvtSWet(13) - pvtSWet(3)) / 10#
    For K = 4 To 12
      pvtSWet(K) = pvtSWet(K - 1) + DIV
    Next K
  End If
'
'---Oil-Water system (Imbibition curve)
'
  If Fluids = SYS_OILWATER And Displace = IMBIBITION Then
    pvtSWet(14) = 1# - SORW
    DIV = (pvtSWet(14) - pvtSWet(1)) / 13#
    DIV = DIV / 2#
    pvtSWet(2) = pvtSWet(1) + DIV
    pvtSWet(3) = pvtSWet(2) + DIV
    pvtSWet(13) = pvtSWet(14) - DIV
    pvtSWet(12) = pvtSWet(13) - DIV
    DIV = (pvtSWet(12) - pvtSWet(3)) / 9#
    For K = 4 To 11
      pvtSWet(K) = pvtSWet(K - 1) + DIV
    Next K
  End If
'
'---Gas-Oil system
'
  If Fluids = SYS_GASOIL Then
    pvtSWet(14) = 1# - SGCrit
    pvtSWet(2) = pvtSWet(1) + SORG
    DIV = (pvtSWet(14) - pvtSWet(2)) / 12#
    DIV = DIV / 2#
    pvtSWet(3) = pvtSWet(2) + DIV
    pvtSWet(4) = pvtSWet(3) + DIV
    pvtSWet(13) = pvtSWet(14) - DIV
    pvtSWet(12) = pvtSWet(13) - DIV
    DIV = (pvtSWet(12) - pvtSWet(3)) / 8#
    For K = 5 To 11
      pvtSWet(K) = pvtSWet(K - 1) + DIV
    Next K
  End If

  For K = 1 To 15
    pvtSNWet(K) = 1# - pvtSWet(K)
  Next K
'
'---Define relative permeability characteristics
'
'---Oil-Water system (Drainage - Corey)
'
  If Fluids = SYS_OILWATER And Displace = DRAINAGE And CORR = COREY Then
    For K = 1 To 15
      pvtSD(K) = pvtSNWet(K) / (1# - pvtSWet(1))
      pvtKROil(K) = (pvtSD(K) ^ CoreyK) * (2# - pvtSD(K))
      pvtKRWat(K) = (1# - pvtSD(K)) ^ CoreyN
    Next K
  End If
'
'---Oil-Water system (Imbibition - Corey)
'
  If Fluids = SYS_OILWATER And Displace = IMBIBITION And CORR = COREY Then
    For i = 1 To 15
      pvtSD(i) = (pvtSWet(i) - SWC) / (1# - SWC - SORW)
      If pvtSD(i) > 1# Then pvtSD(i) = 1#
      pvtKROil(i) = (1# - pvtSD(i)) ^ CoreyN
    Next i
    
    For i = 1 To 15
      pvtSD(i) = (pvtSWet(i) - SWC) / (1# - SWC)
      pvtKRWat(i) = (pvtSD(i) ^ CoreyK) * (2# - pvtSD(i))
    Next i
  End If
'
'---Oil-Water system (Imbibition - Honarpour - Sandstone)
'
  If Fluids = SYS_OILWATER And Displace = IMBIBITION And CORR = HONARPOUR Then
    If Lith = SANDSTONE Then
    
      If Wet = WATERWET Then               'Water Wet
        A = 1# - SWC - SORW
        For i = 1 To 14
          B = pvtSWet(i) - SWC
          pvtKRWat(i) = (0.035388 * B / A) - 0.010874 * (B / A) ^ 2.9
          pvtKRWat(i) = pvtKRWat(i) + 0.56556 * B * pvtSWet(i) ^ 3.6
        Next i
        pvtKRWat(15) = 1#
      
      Else                                 'Oil Wet
        A = 1# - SWC
        B = 1# - SWC - SORW
        For i = 1 To 14
          C = pvtSWet(i) - SWC
          D = pvtSWet(i) - SORW
          pvtKRWat(i) = 1.5814 * (C / A) ^ 1.91 - 0.58617 * (D / B) * C
          pvtKRWat(i) = pvtKRWat(i) - 1.2848 * HRPoro * A * C
        Next i
        pvtKRWat(15) = 1#
      End If
                  
      For i = 1 To 14
        pvtKROil(i) = 0.76067 * ((((pvtSNWet(i) / (1# - SWC)) - SORW) / _
                     (1# - SORW)) ^ 1.8) * (((pvtSNWet(i) - SORW) / _
                     (1# - SWC - SORW)) ^ 2#) + 2.6318 * HRPoro * _
                     (1# - SORW) * (pvtSNWet(i) - SORW)
      Next i
      pvtKROil(15) = 0#
    End If
'
'---Oil-Water system (Imbibition - Honarpour - Limestone)
'
    If Lith = LIMESTONE Then
      
      If Wet = WATERWET Then               'Water Wet
        For i = 1 To 14
          B = pvtSWet(i) - SWC
          pvtKRWat(i) = 0.0020525 * B / (HRPoro ^ 2.15) - 0.051371 * B * _
                        (1# / HRAir) ^ 0.43
        Next i
        pvtKRWat(15) = 1#
      
      Else                                 'Oil Wet
        A = 1# - SWC
        B = 1# - SWC - SORW
        For i = 1 To 14
          C = pvtSWet(i) - SWC
          D = pvtSWet(i) - SORW
          pvtKRWat(i) = (0.29986 * C / A) - 0.32797 * ((D / B) ^ 2#) * C
          pvtKRWat(i) = pvtKRWat(i) + 0.413259 * (C / B) ^ 4#
        Next i
        pvtKRWat(15) = 1#
      End If
                  
      For i = 1 To 14
        A = pvtSNWet(i) - SORW
        pvtKROil(i) = (1.2624 * A / (1# - SORW)) * (A / (1# - SWC - SORW)) ^ 2#
      Next i
      pvtKROil(15) = 0#
    End If
  End If
'
'---Gas-Oil system (Drainage - Corey)
'
  If Fluids = SYS_GASOIL And Displace = DRAINAGE And CORR = COREY Then
    For i = 1 To 15
      pvtSD(i) = (pvtSNWet(i) - SGCrit) / (1# - SWC)
      If pvtSD(i) < 0# Then pvtSD(i) = 0#
      pvtKRGas(i) = (pvtSD(i) ^ CoreyK) * (2# - pvtSD(i))
      pvtKROil(i) = (1# - pvtSD(i)) ^ CoreyN
    Next i
  End If
'
'---Gas-Oil system (Imbibition - Corey)
'
  If Fluids = SYS_GASOIL And Displace = IMBIBITION And CORR = COREY Then
    For i = 1 To 15
      pvtSD(i) = (pvtSNWet(i) - SGCrit) / (1# - SWC - SORG)
      If pvtSD(i) < 0# Then pvtSD(i) = 0#
      If pvtSD(i) > 1# Then pvtSD(i) = 1#
      pvtKROil(i) = (1# - pvtSD(i)) ^ CoreyN
    Next i
    
    For i = 1 To 15
      pvtSD(i) = (pvtSNWet(i) - SGCrit) / (1# - SWC)
      pvtKRGas(i) = (pvtSD(i) ^ CoreyK) * (2# - pvtSD(i))
    Next i
  End If
'
'---Gas-Oil system (Imbibition - Honarpour - Sandstone)
'
  If Fluids = SYS_GASOIL And Displace = IMBIBITION And CORR = HONARPOUR Then
    
    If Lith = SANDSTONE Then
      pvtKROil(1) = 0#
      pvtKRGas(1) = 1#
      For i = 2 To 15
        pvtKROil(i) = 0.98372 * (((pvtSWet(i) - SWC) / (1# - SWC)) ^ 4#) * _
                      (((pvtSWet(i) - SWC) - SORG) / (1# - SWC - SORG)) ^ 2#
        pvtKRGas(i) = 1.1072 * ((pvtSNWet(i) - SGCrit) / (1# - SWC)) ^ 2# * _
                      HRKrg + 2.7794 * (SORG * (pvtSNWet(i) - SGCrit)) * _
                      HRKrg / (1# - SWC)
      Next i
'
'---Gas-Oil system (Imbibition - Honarpour - Limestone)
'
    Else
      pvtKROil(1) = 0#
      pvtKRGas(1) = 1#
      For i = 2 To 15
        pvtKROil(i) = 0.93752 * ((pvtSWet(i) - SWC) / (1# - SWC)) ^ 4# * _
                      (((pvtSWet(i) - SWC) - SORG) / (1# - SWC - SORG)) ^ 2#
        pvtKRGas(i) = 1.8655 * (pvtSNWet(i) - SGCrit) * pvtSNWet(i) * HRKrg / _
                      (1# - SWC) + (8.0053 * (pvtSNWet(i) - SGCrit) * _
                      SORG ^ 2#) / (1# - SWC) - 0.2589 * (pvtSNWet(i) - _
                      SGCrit) * (((1# - SWC - SORG - SGCrit) / (1# - SWC)) _
                      ^ 2#) * ((HRAir / HRPoro) ^ 0.5) * _
                      (1# - (1# - SWC - SORG - SGCrit) / (1# - SWC)) ^ 2#
      Next i
    End If
  End If
'
'---Apply known limits
'
  For i = 1 To 15
    If pvtKROil(i) < 0# Then pvtKROil(i) = 0#
    If pvtKROil(i) > 1# Then pvtKROil(i) = 1#
    If pvtKRWat(i) < 0# Then pvtKRWat(i) = 0#
    If pvtKRWat(i) > 1# Then pvtKRWat(i) = 1#
    If pvtKRGas(i) < 0# Then pvtKRGas(i) = 0#
    If pvtKRGas(i) > 1# Then pvtKRGas(i) = 1#
  Next i
  GO = True
  
  Exit Function
  
ErrExit:
  Call ErrMSG("Error at X " & CurX & " - Please adjust input paramemters!", _
              "Rock Properties Error")
End Function


